#!/usr/bin/env python
#*********************************************************************
# $Id: client-display.py 134 2006-12-06 12:47:05Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import getopt

	import pygame
	from pygame import image
	from pygame.locals import *
	from twisted.internet import reactor
	import time
	import datetime

	from client import *
#	from display import Display
	import plugins
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PygameDisplayView:
	def __init__(self, evManager,DisplayOptions,pluginmgr):
		(Fullscreen,ArgPresentSize,ScreenSize,SAVEFILENAME,FREQUENCY) = DisplayOptions

		self.evManager = evManager
		self.evManager.RegisterListener(self)
		self.pluginmgr = pluginmgr

		pygame.init()

		#find the maximum supported window resolution
		modes = pygame.display.list_modes()
		MaxModeX = 0
		MaxModeY = 0
		RequestedModeFound = False
		for mode in modes:
			if mode[0] > MaxModeX:
				MaxModeX = mode[0]
				MaxModeY = mode[1]
			if mode[0] == MaxModeX:
				if mode[1] > MaxModeY:
					MaxModeY = mode[1]

			if ArgPresentSize:
				if ScreenSize == mode:
					RequestedModeFound = True


		if RequestedModeFound or not Fullscreen:
			self.window = pygame.display.set_mode(ScreenSize)
		else:
			self.window = pygame.display.set_mode(modes[0])

		for plugin in self.pluginmgr.List():
			plugin.setWindow(self.window)

		pygame.display.set_caption('CTF Visualization')
		if Fullscreen:
			pygame.display.toggle_fullscreen()

		self.scoreboard = display(pygame.display, self.window, self.evManager, self.pluginmgr, SAVEFILENAME,FREQUENCY)

 	def Notify(self, event):
		self.scoreboard.Notify(event)


class display:
	Spritegroup = pygame.sprite.Group()
	def __init__(self,display,window,evManager,pluginmgr,savefilename="",imagesavefrequency=5):
		self.display = display
		self.window = window
		self.evManager = evManager
		self.pluginmgr = pluginmgr
		self.savefilename = savefilename
		self.imagestring = ""
		self.imagesavefrequency = imagesavefrequency
		self.lastimagewritetime = time.time()

	def Notify(self, event):
		if isinstance(event, TickEvent):
			self.Update()
		else:
			for plugin in self.pluginmgr.List():
				if plugin.HandledEvent(event):
					(clear,show,newSpritegroup) = plugin.HandleEvent(event,"")

					if clear:
						BACKGROUNDCOLOR = (0,0,0)
						background = pygame.Surface(self.window.get_size())
						background.fill(BACKGROUNDCOLOR)
						self.window.blit(background, (0,0))
						self.Spritegroup.clear(self.window, background)


					if show:
						self.Spritegroup = newSpritegroup
						for S in self.Spritegroup.sprites():
							self.window.blit(S.image, S.rect)
						self.display.flip()


	def Update(self):
		for S in self.Spritegroup.sprites():
			S.update()
			self.window.blit(S.image, S.rect)
		self.display.flip()
		if self.savefilename != "":
			displaycontents = self.display.get_surface()
			newimagestring = pygame.image.tostring(displaycontents,"RGB")
			if newimagestring != self.imagestring:
				TimePassed = time.time() - self.lastimagewritetime
	                        if TimePassed >= self.imagesavefrequency:
					print "writing image"
					#try:
					pygame.image.save(displaycontents,self.savefilename)
					#execpt:
					self.imagestring = newimagestring
					self.lastimagewritetime = time.time()

class KeyboardDisplayController:
	def __init__(self, evManager):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

	def Notify(self, event):
		if isinstance(event, TickEvent):
			#Handle Input Events
			for event in pygame.event.get():
				ev = None
				if event.type == QUIT:
					ev = QuitEvent()
				elif event.type == KEYDOWN and event.key == K_ESCAPE:
					ev = QuitEvent()
				if ev:
					self.evManager.Notify(ev)

class PluginManager:
	def __init__(self):
		print "Loading Plugins..."
		PluginsCreators = getPlugins(ICtfVisPluginCreator,plugins)
		self.list = []
		for PluginCreator in PluginsCreators:
			self.list.append(PluginCreator.PluginCreateDisplay())
		self.list.sort()
		for Plugin in self.list:
			print "Got Plugin",Plugin.pluginname


	def List(self):
		return self.list

def ParseOpt(argv):
	Fullscreen = False
	ArgPresentSize = False
	ScreenWidth = 1024
	ScreenHeight = 768
	CONNECTIP="127.0.0.1"
	CONNECTPORT=8000
	USERNAME = None
	PASSWORD = None
	IMAGEFILE = None
	FREQUENCY = 5
	try:
		opts,args=getopt.getopt(argv, "hfs:U:P:i:p:F:I:", ["help","fullscreen","size=","username=","password=","connect-ip=","connect-port=","image-file=","frequency="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-f","--fullscreen"):
			Fullscreen = True
		if opt in ("-s","--size"):
			try:
				ArgPresentSize = True
				ScreenWidth,ScreenHeight = map(int,arg.split("x"))
				assert ScreenWidth >= 320 and ScreenWidth < 4096
				assert ScreenHeight >= 200 and ScreenHeight < 4096
			except:
				opterr("invalid parameter for --size")
		if opt in ("-U","--username"):
			USERNAME = arg
		if opt in ("-P","--password"):
			PASSWORD = arg
		if opt in ("-i","--connect-ip"):
			CONNECTIP = arg
		if opt in ("-p","--connect-port"):
			try:
				CONNECTPORT = int(arg)
				assert CONNECTPORT >= 0 and CONNECTPORT <= 65535
			except:
				opterr("invalid parameter for --connect-port")
		if opt in ("-F","--frequency"):
			try:
				FREQUENCY = int(arg)
			except:
				opterr("invalid parameter for --connect-ip")
		if opt in ("-I","--image-file"):
			IMAGEFILE = arg
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	if IMAGEFILE != "":
		if not os.path.isdir(os.path.dirname(os.path.abspath(IMAGEFILE))):
			print "Directory %s for writing image does not exist."%os.path.dirname(os.path.abspath(IMAGEFILE))
			sys.exit(3)

	DisplayOptions = (Fullscreen,ArgPresentSize,(ScreenWidth,ScreenHeight),IMAGEFILE,FREQUENCY)
	ConnectOptions = (CONNECTIP,CONNECTPORT)
	UserOptions = (USERNAME,PASSWORD)
	return (DisplayOptions,ConnectOptions,UserOptions)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --connect-ip           ip/hostname to connect to"
	print "  -p, --connect-port         port to connect to"
	print "  -U, --username             username to authenticate with"
	print "  -P, --password             password to authenticate with"
	print "  -F, --frequency            max frequency to write Imagefile"
	print "  -I, --image-file           Imagefile to write to"
	print "  -f, --fullscreen           Fullscreen"
	print "  -s, --size WIDTHxHEIGHT    viewable area"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(DisplayOptions,ConnectOptions,UserOptions) = ParseOpt(argv)

	evManager = EventManager()

	PlugManager = PluginManager()

	keybddisplay = KeyboardDisplayController(evManager)
	spinner = CPUSpinnerController(evManager,2)
	pygameDisplayView = PygameDisplayView(evManager,DisplayOptions,PlugManager)

	phonyModel = PhonyModel(evManager)

	serverController = NetworkServerController(evManager, reactor)
	serverView = NetworkServerView(evManager, serverController, ConnectOptions, UserOptions, clientToServerEvents)

	spinner.Run()
	sys.exit(0)

if __name__=="__main__":
	main(sys.argv[1:])

