#!/usr/bin/env python
#*********************************************************************
# $Id: client-gamemaster-gtk.py 179 2008-12-24 00:44:07Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	#imports have to be in this order!
	import sys
	import getopt
	import os

	from twisted.internet import gtk2reactor
	gtk2reactor.install()

	import pygtk
	pygtk.require('2.0')
	import gtk

	import gobject

	from twisted.internet import reactor
	from twisted.spread import pb

	from client import *
	import plugins

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)



class Gui:
	DisplayDragAndDrop = [("DISPLAY",gtk.TARGET_SAME_APP,0)]
	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

        DragAndDropFieldDivider = ":::"
        DragAndDropFieldSubDivider = ";;;"
        DragAndDropFieldSubSubDivider = "|||"

	def __init__(self, evManager,DisplayOptions,Plugins):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.DisplayCount, self.DisplayCountAutomatic = DisplayOptions


		self.plugins = Plugins


		self.videofile = os.path.join(os.getcwd(),"video.mpg")
		self.imagefile = os.path.join(os.getcwd(),"image.png")
		self.pdffile = os.path.join(os.getcwd(),"presentation.pdf")
		self.creditsfile = os.path.join(os.getcwd(),"credits.txt")
		self.subject = "Testmessage"
		self.text = ["Line1","Line2","Line3"]
		self.logos = True


		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_title("CTF Management")

		self.window.connect("delete_event", self.delete_event)

		self.window.set_border_width(10)

		self.MainTable = gtk.Table(self.DisplayCount + 1,3)
		self.window.add(self.MainTable)


		self.Playlists = {}
		self.PlaylistTreeviews = {}
		self.DisplaylistTreeviews = {}
		self.TemplateTreeviews = {}

		for i in range(1,self.DisplayCount+1):
			self.AddPlaylist(i)


		self.scrolled_window_displays = gtk.ScrolledWindow()
		self.scrolled_window_displays.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.model_displays = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
		self.tree_view_displays = gtk.TreeView(self.model_displays)
		self.model_displays.append(("Test1", None))
		self.scrolled_window_displays.add_with_viewport (self.tree_view_displays)
		cell1 = gtk.CellRendererText()
		column1 = gtk.TreeViewColumn("Displays", cell1, text=0)
		cell2 = gtk.CellRendererToggle()
		cell2.set_property("activatable", True)
		column2 = gtk.TreeViewColumn("active", cell2)
		column2.add_attribute(cell2, "active", 1)
		cell2.connect("toggled", self.col1_toggled, self.model_displays)
		self.tree_view_displays.append_column(column1)
		self.tree_view_displays.append_column(column2)
	#	self.tree_view_displays.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.tree_view_displays.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, self.DisplayDragAndDrop, gtk.gdk.ACTION_COPY)
		self.tree_view_displays.connect("drag_data_get", self.drag_data_get_display)
		self.tree_view_displays.set_enable_search(False)
		self.tree_view_displays.show()
		self.scrolled_window_displays.show()


		self.scrolled_window_clients = gtk.ScrolledWindow()
		self.scrolled_window_clients.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.model_clients = gtk.ListStore(gobject.TYPE_STRING)
		self.tree_view_clients = gtk.TreeView(self.model_clients)
		self.scrolled_window_clients.add_with_viewport (self.tree_view_clients)
		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Clients", cell, text=0)
		self.tree_view_clients.append_column(column)
		self.tree_view_clients.set_enable_search(False)
		self.tree_view_clients.show()
		self.scrolled_window_clients.show()


		self.FrameCommands = gtk.Frame("Commands")
		self.ButtonsTable = gtk.Table(2,len(self.plugins.list)+2)
		self.FrameCommands.add(self.ButtonsTable)


		line = 0

		self.AdjDuration = gtk.Adjustment(20.0, 0.0, 301.0, 1.0, 10.0, 1.0)
		self.hscale = gtk.HScale(self.AdjDuration)
		self.hscale.set_update_policy(gtk.UPDATE_CONTINUOUS)
		self.hscale.set_digits(1)
		self.hscale.set_value_pos(gtk.POS_TOP)
		self.hscale.set_draw_value(True)
		self.ButtonsTable.attach(self.hscale, 0, 2, line, line+1, xpadding = 5, ypadding = 5)
		self.hscale.show()

		line += 1

		self.ButtonNewPlaylist = gtk.Button("Send Playlist")
		self.ButtonNewPlaylist.connect("clicked", self.queuenewbutton, "newplaylist")
		self.ButtonsTable.attach(self.ButtonNewPlaylist, 0, 1, line, line+1, xpadding = 5, ypadding = 5)
		self.ButtonNewPlaylist.show()

		self.ButtonActivateAutoUpdate = gtk.ToggleButton("Update")
		self.ButtonActivateAutoUpdate.connect("toggled", self.queueautoupdatebutton, "autoupdate")
		self.ButtonsTable.attach(self.ButtonActivateAutoUpdate, 1, 2, line, line+1, xpadding = 5, ypadding = 5)
		self.ButtonActivateAutoUpdate.show()

		line += 1

		for plugin in self.plugins.List():
			plugin.setGamemasterCallbacks(self.hscale.get_value,self.ActiveDisplays,self.evManager.Notify)

			firebutton = plugin.GuiFireButton()
			if firebutton:
				self.ButtonsTable.attach(firebutton, 0, 1, line, line+1, xpadding = 5, ypadding = 5)
				configbutton = plugin.GuiConfigureButton()
				if configbutton:
					self.ButtonsTable.attach(configbutton, 1, 2, line, line+1, xpadding = 5, ypadding = 5)

			line += 1



		self.AttachControls(self.DisplayCount)

		self.ButtonsTable.show()
		self.FrameCommands.show()


		self.MainTable.show()
		self.window.show()


	def AddPlaylist(self, i):
		Playlist_scrolled_window = gtk.ScrolledWindow()
		Playlist_scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		Playlist_model = gtk.ListStore(gobject.TYPE_STRING,gobject.TYPE_INT,gobject.TYPE_STRING)
		Playlist_treeview = gtk.TreeView(Playlist_model)
		Playlist_scrolled_window.add_with_viewport (Playlist_treeview)
		cell1 = gtk.CellRendererText()
		column1 = gtk.TreeViewColumn("Item", cell1, text=0)
		cell2 = gtk.CellRendererText()
		column2 = gtk.TreeViewColumn("Time", cell2, text=1)
		cell3 = gtk.CellRendererText()
		column3 = gtk.TreeViewColumn("Options", cell3, text=2)
		Playlist_treeview.append_column(column1)
		Playlist_treeview.append_column(column2)
		Playlist_treeview.append_column(column3)
		self.MainTable.attach(Playlist_scrolled_window, i-1, i, 0, 1, xpadding = 5, ypadding = 5)
		Playlist_treeview.set_enable_search(False)
		Playlist_treeview.enable_model_drag_dest(self.PlaylistDragAndDrop,gtk.gdk.ACTION_COPY)
		Playlist_treeview.connect("drag_data_received",self.drag_data_received_playlist)
		Playlist_treeview.connect("key_press_event", self.playlistkeypress)
		Playlist_treeview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		Playlist_treeview.connect("drag_data_get", self.drag_data_get_playlist)
		Playlist_treeview.show()
		Playlist_scrolled_window.show()
		self.PlaylistTreeviews[i] = (Playlist_scrolled_window, Playlist_model, Playlist_treeview)


		Displaylist_scrolled_window = gtk.ScrolledWindow()
		Displaylist_scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		Displaylist_model = gtk.ListStore(gobject.TYPE_STRING)
		Displaylist_treeview = gtk.TreeView(Displaylist_model)
		Displaylist_scrolled_window.add_with_viewport (Displaylist_treeview)
		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Active Displays for Playlist %d"%i, cell, text=0)
		Displaylist_treeview.append_column(column)
		self.MainTable.attach(Displaylist_scrolled_window, i-1, i, 1, 2, xpadding = 5, ypadding = 5)
		Displaylist_treeview.enable_model_drag_dest(self.DisplayDragAndDrop,gtk.gdk.ACTION_COPY)
		Displaylist_treeview.connect("drag_data_received",self.drag_data_received_display)
		Displaylist_treeview.connect("key_press_event", self.displaylistkeypress)
		Displaylist_treeview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, self.DisplayDragAndDrop, gtk.gdk.ACTION_COPY)
		Displaylist_treeview.connect("drag_data_get", self.drag_data_get_display)
		Displaylist_treeview.set_enable_search(False)
		Displaylist_treeview.show()
		Displaylist_scrolled_window.show()
		self.DisplaylistTreeviews[i] = (Displaylist_scrolled_window, Displaylist_model, Displaylist_treeview)


		Template_scrolled_window = gtk.ScrolledWindow()
		Template_scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		Template_model = gtk.ListStore(gobject.TYPE_STRING,gobject.TYPE_INT,gobject.TYPE_STRING)
		Template_treeview = gtk.TreeView(Template_model)
		Template_scrolled_window.add_with_viewport (Template_treeview)
		cell1 = gtk.CellRendererText()
		column1 = gtk.TreeViewColumn("Item", cell1, text=0)
		cell2 = gtk.CellRendererText()
		column2 = gtk.TreeViewColumn("Time", cell2, text=1)
		cell3 = gtk.CellRendererText()
		column3 = gtk.TreeViewColumn("Options", cell3, text=2)
		Template_treeview.append_column(column1)
		Template_treeview.append_column(column2)
		Template_treeview.append_column(column3)
		self.MainTable.attach(Template_scrolled_window, i-1, i, 2, 3, xpadding = 5, ypadding = 5)
		Template_treeview.set_enable_search(False)
		Template_treeview.enable_model_drag_dest(self.PlaylistDragAndDrop,gtk.gdk.ACTION_COPY)
		Template_treeview.connect("drag_data_received",self.drag_data_received_playlist)
		Template_treeview.connect("key_press_event", self.playlistkeypress)
		Template_treeview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		Template_treeview.connect("drag_data_get", self.drag_data_get_playlist)
		Template_treeview.show()
		Template_scrolled_window.show()
		self.TemplateTreeviews[i] = (Template_scrolled_window, Template_model, Template_treeview)

	def AttachControls(self,row):
		self.MainTable.attach(self.scrolled_window_displays, row, row + 1, 1, 2, xpadding = 5, ypadding = 5)
		self.MainTable.attach(self.scrolled_window_clients, row, row + 1, 2, 3, xpadding = 5, ypadding = 5)
		self.MainTable.attach(self.FrameCommands, row, row + 1, 0, 1, xpadding = 5, ypadding = 5)

	def col1_toggled(self, cell, path, model):
		model[path][1] = not model[path][1]
		return

	def ActiveDisplays(self):
		activedisplays = []
		iter = self.model_displays.get_iter_first()
		displayname = self.model_displays.get_value(iter, 0)
		active = self.model_displays.get_value(iter, 1)
		if active == True:
			activedisplays.append(displayname)
		while iter != None:
			iter = self.model_displays.iter_next(iter)
			if iter != None:
				displayname = self.model_displays.get_value(iter, 0)
				active = self.model_displays.get_value(iter, 1)
				if active == True:
					activedisplays.append(displayname)
		return activedisplays



	def ClearClientList(self):
		self.model_clients.clear()
	def UpdateClientList(self, list):
		for client in list:
			iter = self.model_clients.append()
			self.model_clients.set(iter, 0, client)

	def ClearDisplayList(self):
		self.model_displays.clear()
	def UpdateDisplayList(self, list):
		for client in list:
			self.model_displays.append((client, None))


	def delete_Message(self, widget, data):
		if data == "ok":
			self.subject = self.entrysubject.get_text()
			textbuffer = self.textview.get_buffer()
			startiter = textbuffer.get_start_iter()
			enditer = textbuffer.get_end_iter()
			self.text = textbuffer.get_text(startiter,enditer).split("\n")
			self.logos = self.checklogos.get_active()

		self.dialog.hide()

	def queuenewbutton(self, widget, data):
		Queues = []
		for i in range(1,self.DisplayCount+1):
			NewQueue = self.QueueFromPlaylist(self.PlaylistTreeviews[i][1],self.PlaylistDisplays(i))
			#NewQueue.printqueue()
			TemplateQueue = self.QueueFromPlaylist(self.TemplateTreeviews[i][1],self.PlaylistDisplays(i))
			#TemplateQueue.printqueue()
			NewQueue.template = TemplateQueue

			Queues.append(NewQueue)

		playlistevent = NewPlaylistRequest(Queues)
		self.evManager.Notify(playlistevent)

	def QueueFromPlaylist(self,playlistmodel,displays):
		NewQueue = Queue()

		iter = playlistmodel.get_iter_first()
		if iter != None:
			event = self.EventFromIter(playlistmodel,iter,displays)
			NewQueue.AddQueueEvent(event)
			while iter != None:
				iter = playlistmodel.iter_next(iter)
				if iter != None:
					event = self.EventFromIter(playlistmodel,iter,displays)
					NewQueue.AddQueueEvent(event)


		return NewQueue
	
	def EventFromIter(self,model,iter,clientlist):
		eventname = model.get_value(iter, 0)
		duration = model.get_value(iter, 1)
		options = model.get_value(iter, 2)

		HandledEvent = False
		for plugin in self.plugins.List():
			if plugin.OptionStringStart() == eventname:
				HandledEvent = True
				ev = plugin.EventFromOptions(options)
				
		if HandledEvent:
			event = QueueEvent(ev,clientlist,duration)
			return event
		else:
			print "Couldn't build event %s"%eventname

		
	def PlaylistDisplays(self,i):
		playlistdisplays = []
		model = self.DisplaylistTreeviews[i][1]
		iter = model.get_iter_first()
		if iter != None:
			displayname = model.get_value(iter, 0)
			playlistdisplays.append(displayname)
			while iter != None:
				iter = model.iter_next(iter)
				if iter != None:
					displayname = model.get_value(iter, 0)
					playlistdisplays.append(displayname)
		return playlistdisplays


	def queueautoupdatebutton(self, widget, data):
		# Toggle Button is queried everytime the value is needed
		pass
	
	def drag_data_get_display(self, treeview, context, selection, target_id, etime):
		treeselection = treeview.get_selection()
		model, iter = treeselection.get_selected()
		data = model.get_value(iter, 0)
		selection.set(selection.target, 8, data)

	def drag_data_received_display(self, treeview, contest, x, y, selection, info, etime):
		model = treeview.get_model()
		data = selection.data
		drop_info = treeview.get_dest_row_at_pos(x, y)
		if drop_info:
			path, position = drop_info
			iter = model.get_iter(path)
			model.insert_after(iter, [data])
		else:
			model.append([data])
		return


	def drag_data_received_playlist(self, treeview, contest, x, y, selection, info, etime):
		model = treeview.get_model()
		data = selection.data

		try:
			Type, DurationStr, Options = data.split(self.DragAndDropFieldDivider)
			Duration = int(DurationStr)


			drop_info = treeview.get_dest_row_at_pos(x, y)
			if drop_info:
				path, position = drop_info
				iter = model.get_iter(path)
				data = (Type,Duration,Options)
				if (position == gtk.TREE_VIEW_DROP_BEFORE or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
					model.insert_before(iter, data)
				else:
					model.insert_after(iter, data)
			else:
				iter = model.append()
				model.set_value(iter,0,Type)
				model.set_value(iter,1,Duration)
				model.set_value(iter,2,Options)
		except:
			print "failed"


	def displaylistkeypress(self, widget, event):
		keyname = gtk.gdk.keyval_name(event.keyval)
		if keyname == "Delete":
			if isinstance(widget, gtk.TreeView):
				treeselection = widget.get_selection()
				(model, iter) = treeselection.get_selected()
				if iter:
					model.remove(iter)

	def playlistkeypress(self, widget, event):
		keyname = gtk.gdk.keyval_name(event.keyval)
		if keyname == "Delete":
			if isinstance(widget, gtk.TreeView):
				treeselection = widget.get_selection()
				(model, iter) = treeselection.get_selected()
				if iter:
					model.remove(iter)


	def drag_data_get_playlist(self, treeview, context, selection, target_id, etime):
		treeselection = treeview.get_selection()
		model, iter = treeselection.get_selected()
		data = self.DragAndDropFieldDivider.join([model.get_value(iter, 0),str(int(model.get_value(iter, 1))),model.get_value(iter, 2)])
		selection.set(selection.target, 8, data)

	def delete_event(self, widget, event, data=None):
		ev = QuitEvent()
		self.evManager.Notify(ev)

		return False


 	def Notify(self, event):
		if isinstance(event, ClientListEvent):
			self.ClearDisplayList()
			self.UpdateDisplayList(event.DisplayList)
			#if self.DisplayCountAutomatic:
			#	if len(event.DisplayList) > self.DisplayCount:
			#		# FIXME Add Playlists
			#	elif len(event.DisplayList) > self.DisplayCount:
			#		# FIXME Remove Playlists
			#	#else: # no change

			self.ClearClientList()
			self.UpdateClientList(event.DatabaseList)
			self.UpdateClientList(event.MasterList)
		elif isinstance(event, NewPlaylistEvent):
			if self.ButtonActivateAutoUpdate.get_active():
				if len(event.playlists) < self.DisplayCount:
					#FIXME
					print "Got less Playlists than you can display"
				elif len(event.playlists) > self.DisplayCount:
					#FIXME
					print "Got more Playlists than you can display"
				else: # len(event.playlists) == self.DisplayCount:
					i=1
					for playlist in event.playlists:
						self.ClearPlaylist(self.PlaylistTreeviews[i][1])
						self.UpdatePlaylist(self.PlaylistTreeviews[i][1],playlist.eventlist)

						self.ClearPlaylist(self.TemplateTreeviews[i][1])
						self.UpdatePlaylist(self.TemplateTreeviews[i][1],playlist.template.eventlist)

						self.ClearPlaylistDisplays(self.DisplaylistTreeviews[i][1])
						if len(playlist.eventlist)>0:
							# only reading clientlist from first event in playlist
							# this assumes all events in a playlist have the same displays
							self.UpdatePlaylistDisplays(self.DisplaylistTreeviews[i][1], playlist.eventlist[0].clientlist)

						i += 1

	def ClearPlaylistDisplays(self, model):
		model.clear()
	def UpdatePlaylistDisplays(self, model, displays):
		for display in displays:
			iter = model.append()
			model.set(iter, 0, display)
		

	def ClearPlaylist(self,model):
		model.clear()
	def UpdatePlaylist(self, model, playlist):
		for event in playlist:
			iter = model.append()
			self.IterFromEvent(model, iter, event)

	def IterFromEvent(self, model, iter, event):
		eventname = ""
		duration = event.duration
		options = ""

		for plugin in self.plugins.List():
			if plugin.HandledEvent(event.event):
				(eventname,options) = plugin.OptionsFromEvent(event.event)


		model.set(iter, 0, eventname)
		model.set(iter, 1, duration)
		model.set(iter, 2, options)

class PluginManager:
	def __init__(self):
		print "Loading Plugins..."
		PluginsCreators = getPlugins(ICtfVisPluginCreator,plugins)
		self.list = []
		for PluginCreator in PluginsCreators:
			self.list.append(PluginCreator.PluginCreateGamemasterGtk())
		self.list.sort()
		for Plugin in self.list:
			print "Got Plugin",Plugin.pluginname


	def List(self):
		return self.list

def ParseOpt(argv):
	CONNECTIP="127.0.0.1"
	CONNECTPORT=8000
	USERNAME = None
	PASSWORD = None
	DISPLAYCONTROLAUTOMATIC = True
	DISPLAYCOUNT = 4
	try:
		opts,args=getopt.getopt(argv, "hU:P:i:p:d:", ["help","username=","password=","connect-ip=","connect-port=","displaycount="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-U","--username"):
			USERNAME = arg
		if opt in ("-P","--password"):
			PASSWORD = arg
		if opt in ("-i","--connect-ip"):
			CONNECTIP = arg
		if opt in ("-p","--connect-port"):
			try:
				CONNECTPORT = int(arg)
				assert CONNECTPORT >= 0 and CONNECTPORT <= 65535
			except:
				opterr("invalid parameter for --connect-port")
		if opt in ("-d","--displaycount"):
			try:
				DISPLAYCOUNT = int(arg)
				assert DISPLAYCOUNT >= 0 and DISPLAYCOUNT <= 8
				DISPLAYCONTROLAUTOMATIC = False
			except:
				opterr("invalid parameter for --displaycount")
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	ConnectOptions = (CONNECTIP,CONNECTPORT)
	UserOptions = (USERNAME,PASSWORD)
	Displayoptions = (DISPLAYCOUNT,DISPLAYCONTROLAUTOMATIC)
	return (ConnectOptions,UserOptions,Displayoptions)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --connect-ip           ip/hostname to connect to"
	print "  -p, --connect-port         port to connect to"
	print "  -U, --username             username to authenticate with"
	print "  -P, --password             password to authenticate with"
	print "  -d, --displaycount         maximum displays to be used"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(ConnectOptions,UserOptions,DisplayOptions) = ParseOpt(argv)

	plugins = PluginManager()

	evManager = EventManager()

	myGui = Gui(evManager,DisplayOptions,plugins)



	phonyModel = PhonyModel(evManager)

	serverController = NetworkServerController(evManager, reactor)
	serverView = NetworkServerView(evManager, serverController, ConnectOptions, UserOptions, clientToServerEvents)

	event = TickEvent()
	evManager.Notify(event)

	factory = pb.PBClientFactory()
	reactor.connectTCP("localhost",8000,factory)
	reactor.run()



if __name__=="__main__":
	main(sys.argv[1:])
