#!/usr/bin/env python
#*********************************************************************
# $Id: client-display.py 162 2008-12-16 12:48:49Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import getopt

	from twisted.internet import reactor
	import datetime

	from client import *
	import plugins
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class HtmloutView:
	def __init__(self, evManager,pluginmgr,directory,htmlfilename):
		self.evManager = evManager
		self.evManager.RegisterListener(self)
		self.pluginmgr = pluginmgr
		self.directory = directory
		self.htmlfilename = htmlfilename

		for Plugin in self.pluginmgr.List():
			Plugin.setDirectory(directory)

 	def Notify(self, event):
		if isinstance(event, TickEvent):
			pass
		else:
			for plugin in self.pluginmgr.List():
				if plugin.HandledEvent(event):
					(show,newHtmlTitle,newHtmlPage) = plugin.HandleEvent(event,"")

					if show:
						try:
							print "Writing %s"%os.path.join(self.directory,self.htmlfilename)
							f = open(os.path.join(self.directory,self.htmlfilename), 'w')
							f.write(self.Format(newHtmlTitle,newHtmlPage))
							f.close()
						except:
							print "writing htmlpage failed"

	def Format(self, newHtmlTitle,newHtmlPage):
		page  = "<html>"
		page += "<head>"
		page += "<title>%s</title>"%newHtmlTitle
		page += '<meta http-equiv="refresh" content="5;file:///home/timo/Projekte/ctf-vis/htmloutput/index.html">'
		page += "</head>"
		page += "<body>"
		page += newHtmlPage
		page += "</body>"
		page += "</html>"
		return page


class PluginManager:
	def __init__(self):
		print "Loading Plugins..."
		PluginsCreators = getPlugins(ICtfVisPluginCreator,plugins)
		self.list = []
		for PluginCreator in PluginsCreators:
			self.list.append(PluginCreator.PluginCreateHtmlout())
		self.list.sort()
		for Plugin in self.list:
			print "Got Plugin",Plugin.pluginname


	def List(self):
		return self.list

def ParseOpt(argv):
	DIRECTORY = None
	FILENAME = None
	CONNECTIP="127.0.0.1"
	CONNECTPORT=8000
	USERNAME = None
	PASSWORD = None
	try:
		opts,args=getopt.getopt(argv, "hf:d:U:P:i:p:", ["help","filename=","directory=","username=","password=","connect-ip=","connect-port="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-d","--directory"):
			DIRECTORY = arg
		if opt in ("-f","--filename"):
			FILENAME = arg
		if opt in ("-U","--username"):
			USERNAME = arg
		if opt in ("-P","--password"):
			PASSWORD = arg
		if opt in ("-i","--connect-ip"):
			CONNECTIP = arg
		if opt in ("-p","--connect-port"):
			try:
				CONNECTPORT = int(arg)
				assert CONNECTPORT >= 0 and CONNECTPORT <= 65535
			except:
				opterr("invalid parameter for --connect-port")
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	ConnectOptions = (CONNECTIP,CONNECTPORT)
	UserOptions = (USERNAME,PASSWORD)
	return (DIRECTORY,FILENAME,ConnectOptions,UserOptions)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --connect-ip           ip/hostname to connect to"
	print "  -p, --connect-port         port to connect to"
	print "  -U, --username             username to authenticate with"
	print "  -P, --password             password to authenticate with"
	print "  -d, --directory            Directory to write HTML and images to"
	print "  -f, --filename             HTMLpage to write"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(Directory,Filename,ConnectOptions,UserOptions) = ParseOpt(argv)

	evManager = EventManager()

	PlugManager = PluginManager()

	spinner = CPUSpinnerController(evManager,2)
	htmloutView = HtmloutView(evManager,PlugManager,Directory,Filename)

	phonyModel = PhonyModel(evManager)

	serverController = NetworkServerController(evManager, reactor)
	serverView = NetworkServerView(evManager, serverController, ConnectOptions, UserOptions, clientToServerEvents)

	spinner.Run()
	sys.exit(0)

if __name__=="__main__":
	main(sys.argv[1:])
