#!/usr/bin/env python
#*********************************************************************
# $Id: client-sql.py 151 2006-12-28 21:54:49Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006,2008 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import getopt

	import psycopg2
	from twisted.internet import reactor
	import time

	from client import *
	#from plugins.PluginDisplayScrollingScore import CtfTeam, ScoreState, NewTimeAnnouncementRequest, NewScoreAnnouncementRequest, Score
	from plugins.PluginDisplayScore import CtfTeam, ScoreState, NewTimeAnnouncementRequest, NewScoreAnnouncementRequest, Score

	from plugins.PluginDisplayScoreHistory import NewScoreHistoryAnnouncementRequest
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class MyERROR(Exception):
	def __init__(self, value):
		self.value = value

	def __str__(self):
		return repr(self.value)

class GameDatabaseConnection:
	def __init__(self, evManager, PgsqlOptions, GameOptions, Frequency):
		(self.PGSQLHOST,self.PGSQLUSERNAME,self.PGSQLPASSWORD,self.PGSQLDATABASE) = PgsqlOptions
		self.Frequency = Frequency

		self.LastPoll = time.time()

		self.TimeStart = 0
		self.TimeEnd = 0
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.ScoreState = ScoreState()

		try:
			self.db = psycopg2.connect(host = self.PGSQLHOST, user = self.PGSQLUSERNAME, password = self.PGSQLPASSWORD, database = self.PGSQLDATABASE)
		except psycopg2.OperationalError,err:
			print "PgSQL Connect Error: ", err
			raise MyERROR("Couldn't connect")
		else:
			try:
				cursor = self.db.cursor()

				cursor.execute("""SELECT uid, team_name, team_host FROM teams ORDER BY uid""")
				teamidsfromgame = cursor.fetchall()
				for teamidrow in teamidsfromgame:
					self.ScoreState.AddTeam(teamidrow[0],teamidrow[1],teamidrow[2],"")
					print "added Team with id = %d name = %s vulnbox = %s teamcount = %d"%(teamidrow[0], teamidrow[1], teamidrow[2], self.ScoreState.Count())
			except:
				print "PgSQL Error: "
				raise MyERROR("Error in Query")


	def PollScores(self):
		try:
			for TeamID in self.ScoreState.TeamIDs():
				cursor = self.db.cursor()
				cursor.execute("""SELECT team_points_offensive, team_points_defensive, team_points_advisories, team_points_hacking FROM teams WHERE uid = %s """%TeamID )
				ScoreFromTeam = cursor.fetchone()
				try:
					OffScore = int(ScoreFromTeam[0])
				except:
					OffScore = 0
				try:
					DefScore = int(ScoreFromTeam[1])
				except:
					DefScore = 0
				try:
					AdvisoryScore = int(ScoreFromTeam[2])
				except:
					AdvisoryScore = 0
				try:
					RuleComplianceScore = int(ScoreFromTeam[3])
				except:
					RuleComplianceScore = 0

				print "building score from off=%d, def=%d, adv=%d, rule=%d for %s"%(OffScore,DefScore,AdvisoryScore,RuleComplianceScore,self.ScoreState.TeamName(TeamID))
				score = Score(OffScore,DefScore,AdvisoryScore,RuleComplianceScore)
				self.ScoreState.NewScore(TeamID,score)
				#print "Score for",self.ScoreState.TeamName(TeamID),"is now",self.ScoreState.TeamScore(TeamID)
			#for PlaceID in self.ScoreState.Places():
				#print "Placed Team",self.ScoreState.TeamName(PlaceID),"with score of",self.ScoreState.TeamScore(PlaceID).get_OffScore(),self.ScoreState.TeamScore(PlaceID).get_DefScore(),self.ScoreState.TeamScore(PlaceID).get_AdvScore(),self.ScoreState.TeamScore(PlaceID).get_ExtraScore() 
			ev = NewScoreAnnouncementRequest(self.ScoreState.String())
			self.evManager.Notify(ev)
		except:
			print "PgSQL Error: "
			raise MyERROR("Error in Query")


	def PollTimes(self):
		try:
			cursor = self.db.cursor()
			cursor.execute("""SELECT stats_time FROM stats_times WHERE time_type='start' """)
			starttimes = cursor.fetchone()
			self.TimeStart = starttimes[0]

			cursor = self.db.cursor()
			cursor.execute("""SELECT stats_time FROM stats_times WHERE time_type='end' """)
			stoptimes = cursor.fetchone()
			self.TimeEnd = stoptimes[0]

			ev = NewTimeAnnouncementRequest(self.TimeStart,self.TimeEnd)
			self.evManager.Notify(ev)
		except:
			print "PgSQL Error: "
			raise MyERROR("Error in Query")


	def PollScoreHistory(self):
		try:
			cursor = self.db.cursor()
			cursor.execute("""SELECT stats_time FROM stats_times ORDER BY stats_time""")
			rawtimestamps = cursor.fetchall()
			timestamps = [t[0] for t in rawtimestamps]


			for TeamID in self.ScoreState.TeamIDs():
				scorehistory = []
				for timestamp in timestamps:
					cursor = self.db.cursor()
					cursor.execute("""SELECT stats_points_offensive, stats_points_defensive, stats_points_advisory, stats_points_rulecompliance FROM stats_points WHERE stats_team=%s AND stats_time=%s """,(self.ScoreState.TeamName(TeamID),timestamp))
					teamscorehistory = cursor.fetchone()
					try:
						offscore = int(teamscorehistory[0])
					except:
						offscore = 0
					try:
						defscore = int(teamscorehistory[1])
					except:
						defscore = 0
					try:
						advscore  = int(teamscorehistory[2])
					except:
						advscore  = 0
					try:
						rulescore = int(teamscorehistory[3])
					except:
						rulescore = 0
					scorehistoryvalue  = offscore + defscore + advscore + rulescore
					scorehistory.append(scorehistoryvalue)

					#print """Team %s has score %s,%s,%s,%s at time %s"""%(self.ScoreState.TeamName(TeamID),offscore,defscore,advscore,rulescore,timeakt[0])

				self.ScoreState.NewScoreHistory(TeamID,scorehistory)

			ev = NewScoreHistoryAnnouncementRequest(self.ScoreState.String())
			self.evManager.Notify(ev)
		except:
			print "PgSQL Error: "
			raise MyERROR("Error in Query")
	


 	def Notify(self, event):
		if isinstance(event, DatabasePollEvent):
			if (event.times == True):
				self.PollTimes()
			if (event.scores == True):
				self.PollScores()
			if (event.scorehistory == True):
				self.PollScoreHistory()
		elif isinstance(event, ServerConnectEvent):
			print "Polling the DB (initial)"
			self.PollAll()
			self.LastPoll = time.time()
		elif isinstance(event, TickEvent):
			TimePassed = time.time() - self.LastPoll
			if TimePassed >= self.Frequency:
				print "Polling the DB (scheduled)"
				self.PollAll()
				self.LastPoll = time.time()


	def PollAll(self):
		self.PollTimes()
		self.PollScores()
		self.PollScoreHistory()

		#	for TeamID in self.ScoreState.TeamIDs():
		#		print """Team %s (id = %s) has score %s """%(self.ScoreState.TeamName(TeamID),TeamID,self.ScoreState.TeamScore(TeamID))



def ParseOpt(argv):
	CONNECTIP="127.0.0.1"
	CONNECTPORT=8000
	USERNAME = None
	PASSWORD = None
	PGSQLHOST = "127.0.0.1"
	PGSQLUSERNAME = None
	PGSQLPASSWORD = None
	PGSQLDATABASE = "ctf"
	FREQUENCY = 60
	try:
		opts,args=getopt.getopt(argv, "hm:u:w:d:f:i:p:U:P:", ["help","pgsql-host=","pgsql-username=","pgsql-password=","pgsql-database=","frequency=","username=","password=","connect-ip=","connect-port="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-m","--pgsql-host"):
			PGSQLHOST = arg
		if opt in ("-u","--pgsql-username"):
			PGSQLUSERNAME = arg
		if opt in ("-w","--pgsql-password"):
			PGSQLPASSWORD = arg
		if opt in ("-d","--pgsql-database"):
			PGSQLDATABASE = arg
		if opt in ("-f","--frequency"):
			try:
				FREQUENCY = int(arg)
				assert FREQUENCY >= 0 and FREQUENCY <= 600
			except:
				opterr("invalid parameter for --frequency")
		if opt in ("-U","--username"):
			USERNAME = arg
		if opt in ("-P","--password"):
			PASSWORD = arg
		if opt in ("-i","--connect-ip"):
			CONNECTIP = arg
		if opt in ("-p","--connect-port"):
			try:
				CONNECTPORT = int(arg)
				assert CONNECTPORT >= 0 and CONNECTPORT <= 65535
			except:
				opterr("invalid parameter for --connect-port")
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	ConnectOptions = (CONNECTIP,CONNECTPORT)
	UserOptions = (USERNAME,PASSWORD)
	PgsqlOptions = (PGSQLHOST,PGSQLUSERNAME,PGSQLPASSWORD,PGSQLDATABASE)
	GameOptions = None
	return (ConnectOptions,UserOptions,PgsqlOptions,GameOptions,FREQUENCY)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --connect-ip           ip/hostname to connect to"
	print "  -p, --connect-port         port to connect to"
	print "  -U, --username             username to authenticate with"
	print "  -P, --password             password to authenticate with"
	print "  -m, --pgsql-host           pgsql servers hostname"
	print "  -u, --pgsql-username       username for the pgsql server"
	print "  -w, --pgsql-password       password for the pgsql server"
	print "  -d, --pgsql-database       database for the pgsql server"
	print "  -f, --frequency            pollfrequency"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(ConnectOptions,UserOptions,PgsqlOptions,GameOptions,Frequency) = ParseOpt(argv)

	evManager = EventManager()

	log = TextLogView(evManager)
	spinner = CPUSpinnerController(evManager,2)
	phonyModel = PhonyModel(evManager)

	clientToServerEvents.extend([NewScoreAnnouncementRequest,NewScoreHistoryAnnouncementRequest,NewTimeAnnouncementRequest])


	serverController = NetworkServerController(evManager, reactor)
	serverView = NetworkServerView(evManager, serverController, ConnectOptions, UserOptions, clientToServerEvents)
	try:
		gameDatabaseConn = GameDatabaseConnection(evManager,PgsqlOptions,GameOptions, Frequency)
	except MyERROR:
		sys.exit(1)
	else:
		spinner.Run()

if __name__=="__main__":
	main(sys.argv[1:])
