#!/usr/bin/env python
#*********************************************************************
# $Id: client-sniff.py 165 2008-12-22 21:19:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:



	import sys
	import getopt

	from twisted.internet import reactor
	#from plugins.PluginDisplayScrollingScore import CtfTeam, ScoreState, NewScoreAnnouncementEvent
	from plugins.PluginDisplayScore import CtfTeam, ScoreState, NewScoreAnnouncementEvent
	from plugins.PluginDisplayEtherape import NewSniffSummaryAnnouncementRequest
	import IPy
	import pcap
	import string
	import datetime
	import time
	import socket
	import struct

	from client import *
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class NetworkSniffer:
	protocols={socket.IPPROTO_TCP:'tcp', socket.IPPROTO_UDP:'udp', socket.IPPROTO_ICMP:'icmp'}
	SniffedData = {}
	SniffedDataHistory = {}
	TeamIDs = []
	TeamNetworks = {}
	VulnBoxes = []
	sniffing = False

	def __init__(self, teamid, device, filter, evManager, frequency):
		self.TeamID = teamid
		self.frequency = frequency

		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.p = pcap.pcapObject()
		net, mask = pcap.lookupnet(device)
		self.p.open_live(device, 1600, 0, 100)
		self.p.setfilter(filter, 0, 0)
		self.p.setnonblock(1)
		self.LastSummarySent = datetime.datetime.now()

 	def Notify(self, event):
		if isinstance(event, TickEvent):
			if self.sniffing:
				self.p.dispatch(1, self.HandlePacket)

				Time = datetime.datetime.now()
				if (Time - self.LastSummarySent) > datetime.timedelta(seconds=self.frequency):
					self.SaveHistory()
					self.SendSummary()
					self.SniffedData = self.InitializePacketStore()
					self.LastSummarySent = datetime.datetime.now()
		else:
			if isinstance(event, NewScoreAnnouncementEvent):
				if not self.sniffing:
					print "got a team-list"
					newScoreState = ScoreState()
					newScoreState.FromString(event.ScoreString)
					self.TeamIDs.extend(newScoreState.TeamIDs())
					for TeamID in self.TeamIDs:
						Team = newScoreState.Team(TeamID)
						self.TeamNetworks[TeamID] = Team.get_network()
						self.VulnBoxes.append(Team.get_vulnbox())

					self.SniffedData = self.InitializePacketStore()
					self.SniffedDataHistory = self.InitializePacketStoreHistory()
					self.sniffing = True
					print "Starting to sniff"



	def HandlePacket(self, pktlen, data, timestamp):
		if not data:
			return

		if data[12:14] == '\x08\x00':
			decoded = self.ParsePacket(data[14:])
			print "%s.%f %s > %s"%(time.strftime('%H:%M', time.localtime(timestamp)), timestamp % 60, decoded['SourceAddress'], decoded['DestinationAddress'])
			if self.VulnBox(decoded['DestinationAddress']):
				SourceTeamID = self.TeamFromIP(decoded['SourceAddress'])
				DestinationTeamID = self.TeamFromIP(decoded['DestinationAddress'])
				print "Got a Packet from %d to %d"%(SourceTeamID, DestinationTeamID)
				if self.TeamID:
					if self.TeamID == DestinationTeamID:
						self.SavePacket(SourceTeamID, DestinationTeamID, decoded)
				else:
					self.SavePacket(SourceTeamID, DestinationTeamID, decoded)


	def TeamFromIP(self, IP):
		for TeamID in self.TeamIDs:
			TeamNet = IPy.IP(self.TeamNetworks[TeamID])
			if IPy.IP(IP) in TeamNet:
				return TeamID
		return 0

	def VulnBox(self, IP):
		return (IP in self.VulnBoxes)

	def SavePacket(self, SourceTeamID, DestinationTeamID, Packet):
		self.SniffedData[SourceTeamID][DestinationTeamID] += 1


	def InitializePacketStoreHistory(self):
		HistoryLength = 10

		SniffedDataHistory={}
		SniffedDataHistory[0]={}
		SniffedDataHistory[0][0]=[]
		for i in range(0,HistoryLength):
			SniffedDataHistory[0][0].append(0)
		for SourceTeamID in self.TeamIDs:
			SniffedDataHistory[SourceTeamID]={}
			SniffedDataHistory[SourceTeamID][0]=[]
			SniffedDataHistory[0][SourceTeamID]=[]
			for i in range(0,HistoryLength):
				SniffedDataHistory[SourceTeamID][0].append(0)
				SniffedDataHistory[0][SourceTeamID].append(0)
			for DestinationTeamID in self.TeamIDs:
				SniffedDataHistory[SourceTeamID][DestinationTeamID] = []
				for i in range(0,HistoryLength):
					SniffedDataHistory[SourceTeamID][DestinationTeamID].append(0)
		return SniffedDataHistory

	def SaveHistory(self):
		for SourceTeamID in self.TeamIDs:
			for DestinationTeamID in self.TeamIDs:
				self.SniffedDataHistory[SourceTeamID][DestinationTeamID].pop(0)
				self.SniffedDataHistory[SourceTeamID][DestinationTeamID].append(self.SniffedData[SourceTeamID][DestinationTeamID])

	def InitializePacketStore(self):
		SniffedData={}
		SniffedData[0]={}
		SniffedData[0][0]=0
		for SourceTeamID in self.TeamIDs:
			SniffedData[SourceTeamID]={}
			SniffedData[SourceTeamID][0]=0
			SniffedData[0][SourceTeamID]=0
			for DestinationTeamID in self.TeamIDs:
				SniffedData[SourceTeamID][DestinationTeamID]=0
		return SniffedData

	def SendSummary(self):
		Max = 0
		SniffedData = self.InitializePacketStore()
		for SourceTeamID in self.TeamIDs:
			for DestinationTeamID in self.TeamIDs:
				Akt = 0
				for I in self.SniffedDataHistory[SourceTeamID][DestinationTeamID]:
					Akt += I
				if Akt > Max:
					Max = Akt

		print "Maximum is ",Max

		for SourceTeamID in self.TeamIDs:
			for DestinationTeamID in self.TeamIDs:
				for I in self.SniffedDataHistory[SourceTeamID][DestinationTeamID]:
					SniffedData[SourceTeamID][DestinationTeamID] += I
				if Max != 0:
					SniffedData[SourceTeamID][DestinationTeamID] = SniffedData[SourceTeamID][DestinationTeamID] * 100/ Max
				else:
					SniffedData[SourceTeamID][DestinationTeamID] = 0
				
		ev = NewSniffSummaryAnnouncementRequest(SniffedData)
		self.evManager.Notify(ev)
		
	def ParsePacket(self,s):
		d={}
		d['Version']=(ord(s[0]) & 0xf0) >> 4
		d['HeaderLen']=ord(s[0]) & 0x0f
		d['Tos']=ord(s[1])
		d['TotalLen']=socket.ntohs(struct.unpack('H',s[2:4])[0])
		d['Id']=socket.ntohs(struct.unpack('H',s[4:6])[0])
		d['Flags']=(ord(s[6]) & 0xe0) >> 5
		d['FragmentOffset']=socket.ntohs(struct.unpack('H',s[6:8])[0] & 0x1f)
		d['Ttl']=ord(s[8])
		d['Protocol']=ord(s[9])
		d['Checksum']=socket.ntohs(struct.unpack('H',s[10:12])[0])
		d['SourceAddress']=pcap.ntoa(struct.unpack('i',s[12:16])[0])
		d['DestinationAddress']=pcap.ntoa(struct.unpack('i',s[16:20])[0])
		if d['HeaderLen']>5:
			d['Options']=s[20:4*(d['HeaderLen']-5)]
		else:
			d['Options']=None
		d['Data']=s[4*d['HeaderLen']:]
		return d




def ParseOpt(argv):
	CONNECTIP="127.0.0.1"
	CONNECTPORT=8000
	USERNAME = None
	PASSWORD = None
	TEAMID = None
	FREQUENCY = 60
	DEVICE = "eth0"
	try:
		opts,args=getopt.getopt(argv, "hf:i:p:U:P:I:D:", ["help","frequency=","username=","password=","connect-ip=","connect-port=", "team-id=","device="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-f","--frequency"):
			try:
				FREQUENCY = int(arg)
				assert FREQUENCY >= 0 and FREQUENCY <= 600
			except:
				opterr("invalid parameter for --frequency")
		if opt in ("-U","--username"):
			USERNAME = arg
		if opt in ("-P","--password"):
			PASSWORD = arg
		if opt in ("-D","--device"):
			DEVICE = arg
		if opt in ("-I","--team-id"):
			try:
				TEAMID  = int(arg)
			except:
				opterr("invalid parameter for --team-id")
		if opt in ("-i","--connect-ip"):
			CONNECTIP = arg
		if opt in ("-p","--connect-port"):
			try:
				CONNECTPORT = int(arg)
				assert CONNECTPORT >= 0 and CONNECTPORT <= 65535
			except:
				opterr("invalid parameter for --connect-port")
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	ConnectOptions = (CONNECTIP,CONNECTPORT)
	UserOptions = (USERNAME,PASSWORD)
	return (ConnectOptions,UserOptions,FREQUENCY,TEAMID, DEVICE)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --connect-ip           ip/hostname to connect to"
	print "  -p, --connect-port         port to connect to"
	print "  -U, --username             username to authenticate with"
	print "  -P, --password             password to authenticate with"
	print "  -f, --frequency            pollfrequency"
	print "  -I, --team-id              ID of the team where the service is running"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(ConnectOptions,UserOptions,Frequency,TeamID,Device) = ParseOpt(argv)

	evManager = EventManager()

	log = TextLogView(evManager)
	spinner = CPUSpinnerController(evManager,2)
	phonyModel = PhonyModel(evManager)

	clientToServerEvents.extend([NewSniffSummaryAnnouncementRequest])


	serverController = NetworkServerController(evManager, reactor)
	serverView = NetworkServerView(evManager, serverController, ConnectOptions, UserOptions, clientToServerEvents)
	Sniffer = NetworkSniffer(TeamID,Device,"port 80", evManager, Frequency)
	spinner.Run()

if __name__=="__main__":
	main(sys.argv[1:])
