
#*********************************************************************
# $Id: client.py 162 2008-12-16 12:48:49Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys

	from twisted.spread import pb
	from twisted.cred import credentials
	from twisted.internet import reactor

	from events import *
	from manager import *
	from network import *
	import network
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class NetworkServerView(pb.Root):
	"""We SEND events to the server through this object"""
	STATE_PREPARING = 0
	STATE_CONNECTING = 1
	STATE_CONNECTED = 2

	def __init__(self, evManager, NetworkServerController, ConnectOptions, UserOptions, clientToServerEvents):
		(self.ConnectHost,self.ConnectPort) = ConnectOptions
		(self.Username,self.Password) = UserOptions 

		self.NetworkServerController = NetworkServerController
		self.clientToServerEvents = clientToServerEvents

		self.credentials = credentials.UsernamePassword(self.Username,self.Password)

		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.state = NetworkServerView.STATE_PREPARING
		self.server = None


	def Connected(self, server):
		print "CONNECTED"
		self.server = server
		self.state = NetworkServerView.STATE_CONNECTED
		ev = ServerConnectEvent(server)
		self.evManager.Notify(ev)

	def ConnectFailed(self, server):
		print "CONNECTION FAILED"
		#TODO sleep a bit
		self.state = NetworkServerView.STATE_PREPARING

	def Notify(self, event):
		ev = event
		if isinstance(event, TickEvent) and self.state == NetworkServerView.STATE_PREPARING:
			self.state = NetworkServerView.STATE_CONNECTING
			factory = pb.PBClientFactory()
			reactor.connectTCP(self.ConnectHost, self.ConnectPort, factory)
			factory.login(self.credentials, self.NetworkServerController).addCallbacks(self.Connected,self.ConnectFailed)

		if not isinstance(event, pb.Copyable):
			evName = event.__class__.__name__
			if not hasattr(network, "Copyable" + evName):
				return
			copyableClass = getattr(network, "Copyable" + evName)
			if copyableClass not in clientToServerEvents:
				return
			ev = copyableClass(event)

		elif ev.__class__ not in self.clientToServerEvents:
			return 

		remoteCall = self.server.callRemote("ClientEvent",ev) 



class NetworkServerController(pb.Referenceable):
	"""We RECEIVE events from the server through this object"""
	def __init__(self, evManager, twistedReactor):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.reactor = twistedReactor

	def remote_ServerEvent(self, event):
		print "================= GOT AN EVENT FROM SERVER:", str(event)
		self.evManager.Notify(event)
		return 1

	def remote_Ping(self):
		pong = 1
		return pong

	def Notify(self, event):
		if isinstance(event, ServerConnectEvent):
			# say hello
			remoteResponse = event.server.callRemote("ClientConnect", self)
		if isinstance(event, TickEvent):
			self.reactor.iterate()


class PhonyEventManager(EventManager):
	"""this object is responsible for coordinating most communication between the Model, View, and Controller."""
	def Notify(self, event):
		pass

class PhonyModel:
	def __init__(self, evManager):
		self.game = None
		self.server = None
		self.phonyEvManager = PhonyEventManager()
		self.realEvManager = evManager
		self.neededObjects = [] 
		self.waitingObjectStack = []

		self.realEvManager.RegisterListener(self)

 	def Notify(self, event):
		pass
