
#*********************************************************************
# $Id: manager.py 162 2008-12-16 12:48:49Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys

	import pygame
	from twisted.plugin import getPlugins,IPlugin

	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	import plugins
	from events import *
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class EventManager:
	"""this object is responsible for coordinating most communication between the Model, View, and Controller."""
	def __init__(self):
		from weakref import WeakKeyDictionary
		self.listeners = WeakKeyDictionary()
		self.eventQueue = []

	def RegisterListener(self, listener):
		self.listeners[listener] = 1

	def UnregisterListener(self, listener):
		if listener in self.listeners.keys():
			del self.listeners[listener]

	def Notify(self, event):
		for listener in self.listeners.keys():
			#If the weakref has died, remove it and continue through the list
			if listener is None:
				del self.listeners[listener]
				continue
			listener.Notify(event)



class CPUSpinnerController:
	def __init__(self, evManager,clockfreq=50):
		self.evManager = evManager
		self.clockfreq = clockfreq
		self.evManager.RegisterListener(self)
		if self.clockfreq != 0:
			self.clockfreq = 50
		self.clock = pygame.time.Clock()
		self.keepGoing = 1

	def Run(self):
		while self.keepGoing:
			self.clock.tick(self.clockfreq)
			event = TickEvent()
			self.evManager.Notify(event)

 	def Notify(self, event):
		if isinstance(event, QuitEvent):
			self.keepGoing = 0

class TextLogView:
	def __init__(self, evManager):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

 	def Notify(self, event):
		if not (isinstance(event, TickEvent) or isinstance(event, SecondEvent)) :
			print "== Event:",event.name


class QueueEvent:
	"""a Event on a Playlist"""
	def __init__(self, event, clientlist, duration):
		self.event = event
		self.clientlist = clientlist
		self.duration = duration

	def FireEvent(self):
		return AdressedEvent(self.event, self.clientlist)

	def Tick(self):
		self.duration -= 1

	def TickedOut(self):
		return self.duration <= 0
	
#	def printevent(self):
#		clients = " ".join(client for client in self.clientlist)
#		print "event %s duration %d clients %s"%(str(self.event),self.duration,clients)


class Queue:
	"""A Playlist"""
	def __init__(self):
		self.eventlist = []
		self.template = None
	
	def AddQueueEvent(self, queueevent):
		self.eventlist.append(queueevent)

	def AddQueue(self, Queue):
		self.eventlist.extend(Queue.eventlist)

	def Tick(self,evManager):
		if not self.Empty():
			self.eventlist[0].Tick()
			if self.eventlist[0].TickedOut():
				self.eventlist.pop(0)
				if self.Empty():
					self.Reload()
				self.Fire(evManager)
				return True
		return False
			
	def Install(self,evManager):
		self.Fire(evManager)
	
	def Fire(self,evManager):
		if not self.Empty():
			event = self.eventlist[0].FireEvent()
			evManager.Notify(event)

	def Empty(self):
		return len(self.eventlist) <= 0

	def Reload(self):
		if self.template:
			self.eventlist = []
			for queueevent in self.template.eventlist:
				newqueueevent = QueueEvent(queueevent.event, queueevent.clientlist, queueevent.duration)
				self.eventlist.append(newqueueevent)

	
#	def printqueue(self):
#		itemnumber = 1
#		for event in self.eventlist:
#			event.printevent()




