#!/usr/bin/env python
#*********************************************************************
# $Id: pdfreader.py 169 2008-12-23 14:26:17Z timo $
#
# pdfreader
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# partly taken from keyJnote
# http://keyjnote.sourceforge.net/
#
# (C) 2006 Timo Boettcher / Martin J. Fiedler
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import getopt
	import os

	import tempfile
	import pygame
	from pygame.locals import *
	import Image
	import TiffImagePlugin
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)

def num(s):
	s=s.strip()
	r=""
	while s[0] in "0123456789":
		r+=s[0]
		s=s[1:]
	try:
		return int(r)
	except ValueError:
		return -1


class PdfReader:
	def __init__(self,filename,screensize=(0,0),directory="/tmp"):
		self.filename = filename
		self.directory = directory
		self.AktPage = 0
		
		#Get PageCount and Resolution
		try:
			f = file(self.filename,"rb")
			pdf = f.read()
			f.close()
			box = map(float,pdf.split("/MediaBox",1)[1].split("]",1)[0].split("[",1)[1].strip().split())
			self.PdfWidth = box[2]-box[0]
			self.PdfHeight = box[3]-box[1]
			self.Resolution = min(screensize[0]*72.0/self.PdfWidth, screensize[1]*72.0/self.PdfHeight)
			self.PageCount = max(map(num,pdf.split("/Count")[1:]))
			print "File `%s'"%self.filename, "has `%d'"%self.PageCount, "pages"
		except:
			self.Resolution = 72.0
		if not self.PageCount:
			print "Cannot analyze `%s'"%self.filename


	def PageCount(self):
		return self.PageCount


	def PageToImage(self,Pagenumber):
		TempFileName = self.PageToImageFile(Pagenumber)
		if TempFileName != None:

			# open the TIFF file
			try:
				img=pygame.image.load(TempFileName)
			except:
				print "GhostScript produced an unreadable file!"
				return None
			else:
				# try to delete the file again
				try:
					os.remove(TempFileName)
				except OSError:
					return None
				return img
		else:
			return None

	def PageToImageFile(self,PageNumber):
		if PageNumber <= self.PageCount:
			TempFileName = tempfile.mkstemp(suffix=".tif", prefix="pdfreader-",dir=self.directory)[1]
			GhostscriptCommand = "gs"
			GhostscriptArgs = ("-q", "-dBATCH", "-dNOPAUSE", "-sDEVICE=tiff24nc", "-sOutputFile=%s"%TempFileName, \
						"-dFirstPage=%d"%PageNumber, "-dLastPage=%d"%PageNumber, \
						"-r%dx%d"%(self.Resolution,self.Resolution), "-dTextAlphaBits=1", "-dGraphicsAlphaBits=1", \
						self.filename)

			try:
				assert 0 == os.spawnvp(os.P_WAIT,GhostscriptCommand, GhostscriptArgs)
			except OSError, (errcode,errmsg):
				print "Cannot start GhostScript:",errmsg
				return None
			except AssertionError:
				print "There was an error while rendering page %d!"%PageNumber
				return None
			return TempFileName
		else:
			return None



	def ShowFirstPage(self):
		self.AktPage = 1
		return self.PageToImage(self.AktPage)

	def ShowNextPage(self):
		if self.AktPage + 1 <= self.PageCount:
			self.AktPage += 1
		return self.PageToImage(self.AktPage)

	def ShowPreviousPage(self):
		if self.AktPage >= 2:
			self.AktPage -= 1
		return self.PageToImage(self.AktPage)





#*********************************************************************
#
# FOR STANDALONE USE ONLY
#
#*********************************************************************

def main(argv):
	ArgPresentSize = 0
	ArgPresentFullscreen = 0
	ArgPresentResize = 0
	try:
		opts,args=getopt.getopt(argv, "hfrs:", ["help","fullscreen","resize","size="])
	except getopt.GetoptError,message:
		opterr(message)
		
	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-f","--fullscreen"):
			ArgPresentFullscreen = 1
		if opt in ("-r","--resize"):
			ArgPresentResize = 1
		if opt in ("-s","--size"):
			try:
				ArgPresentSize = 1
				ScreenWidth,ScreenHeight = map(int,arg.split("x"))
				assert ScreenWidth >= 320 and ScreenWidth < 4096
				assert ScreenHeight >= 200 and ScreenHeight < 4096
			except:
				opterr("invalid parameter for --size")
	if len(args)<1:
		opterr("no file to play")
	elif len(args)>1:
		opterr("too much files to play")
	FileName=args[0]

	pygame.init()

	#find the maximum supported window resolution
	modes = pygame.display.list_modes()
	MaxModeX = 0
	MaxModeY = 0
	RequestedModeFound = 0
	for mode in modes:
		if mode[0] > MaxModeX:
			MaxModeX = mode[0]
			MaxModeY = mode[1]
		if mode[0] == MaxModeX:
			if mode[1] > MaxModeY:
				MaxModeY = mode[1]

		if ArgPresentSize == 1:
			if ScreenWidth == mode[0] and ScreenHeight == mode[1]:
				RequestedModeFound = 1
				
		
	if RequestedModeFound == 1:	
		window = pygame.display.set_mode((ScreenWidth,ScreenHeight))
	else:
		window = pygame.display.set_mode(modes[0])
	pygame.display.set_caption("PdfViewer %s"%FileName)
	background = pygame.Surface(window.get_size())
	background.fill((0,0,0))
	window.blit(background, (0,0))

	Reader = PdfReader(FileName,window.get_size())

	Img = Reader.ShowFirstPage()
	if ArgPresentFullscreen == 1:
		try:
			pygame.display.toggle_fullscreen()
		except:
			print "Couldn't activate fullscreen mode"

	window.blit(Img, (pygame.display.get_surface().get_width()/2 - Img.get_width()/2, pygame.display.get_surface().get_height()/2 - Img.get_height()/2))
	pygame.display.flip()

	clock = pygame.time.Clock()
	while True:
		pygame.display.update()
		clock.tick(300)
		event = pygame.event.wait()
		if event.type is QUIT:
			sys.exit(0)
		elif event.type is KEYDOWN and event.key == K_q:
			sys.exit(0)
		elif event.type is KEYDOWN and event.key == K_r:
			if ArgPresentResize == 1:
				ArgPresentResize = 0
			else:
				ArgPresentResize = 1
			background.fill((0,0,0))
			window.blit(background, (0,0))
		elif event.type is KEYDOWN and event.key == K_f:
			try:
				pygame.display.toggle_fullscreen()
			except:
				print "Couldn't activate fullscreen mode"
		elif event.type is KEYDOWN and event.key == K_ESCAPE:
			sys.exit(0)
		elif event.type is KEYDOWN and event.key == K_BACKSPACE:
			Img = Reader.ShowPreviousPage()
		elif event.type is KEYDOWN and event.key == K_SPACE:
			Img = Reader.ShowNextPage()
		elif event.type is KEYDOWN and event.key == K_LEFT:
			Img = Reader.ShowPreviousPage()
		elif event.type is KEYDOWN and event.key == K_RIGHT:
			Img = Reader.ShowNextPage()
		elif event.type is KEYDOWN and event.key == K_PAGEUP:
			Img = Reader.ShowPreviousPage()
		elif event.type is KEYDOWN and event.key == K_PAGEDOWN:
			Img = Reader.ShowNextPage()
		Img.convert()
		if ArgPresentResize == 1:
			ImgScaled = pygame.transform.scale(Img, window.get_size())
			window.blit(ImgScaled, (pygame.display.get_surface().get_width()/2 - ImgScaled.get_width()/2, pygame.display.get_surface().get_height()/2 - ImgScaled.get_height()/2))
		else:
			window.blit(Img, (pygame.display.get_surface().get_width()/2 - Img.get_width()/2, pygame.display.get_surface().get_height()/2 - Img.get_height()/2))
		pygame.display.flip()


def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)
	
def helpexit():
	print "Usage `%s [option] file.pdf'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -f, --fullscreen           displays PDF in Fullscreen"
	print "  -r, --resize               resizes PDF to match viewable area"
	print "  -s, --size WIDTHxHEIGHT    sets the viewable area"
	print "  -h, --help                 shows this help and exits "
	print ""
	print ""
	print "Keycodes when running:"
	print " q, [ESC]                    quit"
	print " f                           toggles fullscreen mode"
	print " r                           switches resizing on and off"
	print " [SPACE],[RIGHT],[PAGEDOWN]  next page"
	print " [BACKSPACE],[LEFT],[PAGEUP] previous page"
	print ""


	sys.exit(0)

if __name__=="__main__":
	main(sys.argv[1:])

	
