
#*********************************************************************
# $Id: plugininterface.py 164 2008-12-17 01:00:47Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys

	from zope.interface import Interface, Attribute
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class ICtfVisPluginCreator(Interface):
	def __init__():
		"""build it"""

	def PluginCreateServer():
		"""return a plugin-instance for the server"""

	def PluginCreateGamemasterGtk():
		"""return a plugin-instance for the GamemasterGTK"""

	def PluginCreateDisplay():
		"""return a plugin-instance for the display"""

	def PluginCreateHtmlout():
		"""return a plugin-instance for the htmlout"""

	def EventDirections(self):
		"""return a tuple of serverToDisplayEvents, 
		serverToDatabaseEvents, serverToMasterEvents,
		clientToServerEvents"""

class ICtfVisPlugin(Interface):
	pluginname = Attribute("""Name""")
	pluginshortname = Attribute("""Name""")

	def __init__(component):
		"""
		initialize the plugin to be run on component,
		where component is either
		display or server or gamemastergtk
		"""

	def setGamemasterCallbacks(CallbackDuration,CallbackDisplays,CallbackNotify):
		"""
		set a function to query for the duration of an event,
		the list of displays and the eventhandler
		only relevant to set on the gamemaster
		"""

	def setServerCallbacks(CallbackNotify):
		"""
		set a function to query for eventhandler
		only relevant to set on the gamemaster
		"""

	def setWindow(Window):
		"""
		set the pygame window to display stuff on
		only relevant on the display
		"""

	def HandledEvent(event):
		"""
		true if the event can be handled on component 
		where component is either display or server or gamemastergtk
		this must include the event to display something,
		and may include other events to synchronize state
		"""

	def GuiFireButton():
		"""
		create a button to fire the plugin
		this should return something that can be attached
		into one cell of an gtk.Table
		"""

	def GuiFireCallback(widget, data):
		"""
		create the event from the GUI
		return the event
		"""

	def GuiFireDragDropCallback(widget, context, selection, targetType, eventTime):
		"""
		create the optionsstring for DragAndDrop
		this string must not contain ":"
		return thee optionstring that must be understood by EventFromDragDrop
		"""

	def OptionStringStart():
		"""
		return the shortname of the plugin to be matched as the first part 
		of the optionstring
		"""
	
	
	def EventFromOptions(optionstring):
		"""
		create the event from the optionstring
		return the event
		"""

	def OptionsFromEvent(event):
		"""
		create the options from the event
		return a tuple of optionstringstart and options (not the duration)
		"""

	def GuiConfigureButton():
		"""
		create a button to configure the event
		this should return something that can be attached
		into one cell of an gtk.Table
		"""

	def GuiConfigureCallback(widget, data):
		"""
		configure the event
		for example, display a dialog which file is to be displayed
		configuration state is to be kept internally
		"""

	def HandleEvent(event, eventsource):
		"""
		Handle events that are registered to be handled by this plugin
		The eventsource is one of "display, database, gamemaster" on server, 
		otherwise none.
		This can store or retrieve state on all components, and must 
		return (if component is display) two booleans indicating if the
		screen needs to be cleared, if there is anything to display, 
		and the spritegroup to display, if the event is fired
		"""

class IEvent(Interface):
	eventname = Attribute("""Name""")

	def __init__(options):
		"""
		event
		"""

	def Acknowledge():
		"""
		for a client to server event,
		return the matching server to client event
		"""

	def Targets():
		"""
		list of components (display, server, gamemastergtk)
		where the event should be forwarded to
		"""

	def DragDropString():
		"""
		create the optionstring to display the event in the playlist
		optionstring must not contain ":"
		"""

