
#*********************************************************************
# $Id: PluginDisplayEtherape.py 179 2008-12-24 00:44:07Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *
	import datetime
	import math

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)











# Sniff-Client requests the Server to Announce new SniffSummary
class NewSniffSummaryAnnouncementRequest(Event):
	def __init__(self,SniffSummaryString):
		self.name = "New Etherape Announcement Request"
		self.SniffSummaryString = SniffSummaryString
	def Acknowledge(self):
		return NewSniffSummaryAnnouncementEvent(self.SniffSummaryString)
# Server announces new SniffSummary
class NewSniffSummaryAnnouncementEvent(Event):
	def __init__(self,SniffSummaryString):
		self.name = "New Etherape Announcement Event"
		self.SniffSummaryString = SniffSummaryString


# Client requests Server to show Etherape on Display-Client
class ShowEtherapeRequest(Event):
	def __init__(self,Wait):
		self.name = "Show Etherape Request"
		self.Wait = Wait
	def Acknowledge(self):
		return ShowEtherapeEvent(self.Wait)
# Server requests Display-Client to show Etherape
class ShowEtherapeEvent(Event):
	def __init__(self,Wait):
		self.name = "Show Etherape Event"
		self.Wait = Wait


# NewSniffSummaryAnnouncementRequest
# Direction: Client to Server only
MixInCopyClasses(NewSniffSummaryAnnouncementRequest)
pb.setUnjellyableForClass(NewSniffSummaryAnnouncementRequest, NewSniffSummaryAnnouncementRequest)

# NewSniffSummaryAnnouncementEvent
# Direction: Server to Client only
MixInCopyClasses(NewSniffSummaryAnnouncementEvent)
pb.setUnjellyableForClass(NewSniffSummaryAnnouncementEvent, NewSniffSummaryAnnouncementEvent)



# ShowEtherapeRequest
# Direction: Client to Server only
MixInCopyClasses(ShowEtherapeRequest)
pb.setUnjellyableForClass(ShowEtherapeRequest, ShowEtherapeRequest)

# ShowEtherapeEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowEtherapeEvent)
pb.setUnjellyableForClass(ShowEtherapeEvent, ShowEtherapeEvent)



class CtfTeam:
	"""Information about a playing Team including name and score"""
	def __init__(self,id,name, vulnbox, network):
		self.id = id
		self.name = name
		self.ipnetwork = network
		self.ipvulnbox = vulnbox
		self.Score = Score(0,0,0)
		self.Scorehistory = []
		self.PositionChange = 0

	def FromString(self, String):
		self.id = String[0]
		self.name = String[1]
		self.Score = Score(0,0,0)
		self.Score.FromString(String[2])
		self.Scorehistory = String[3]
		self.ipnetwork = String[4]
		self.ipvulnbox = String[5]

	def set_score(self,Score):
		self.Score = Score

	def set_scorehistory(self,Scorehistory):
		self.Scorehistory = Scorehistory

	def set_positionchange(self,positionchange):
		self.PositionChange = positionchange

	def get_positionchange(self):
		return self.PositionChange

	def get_id(self):
		return self.id

	def get_name(self):
		return self.name

	def get_score(self):
		return self.Score

	def get_scorehistory(self):
		return self.Scorehistory

	def get_string(self):
		return (self.id, self.name, self.Score.get_string(), self.Scorehistory, self.ipnetwork, self.ipvulnbox)

	def get_network(self):
		return self.ipnetwork

	def get_vulnbox(self):
		return self.ipvulnbox


class ScoreState:
	"""Collection of all CtfTeams, with functions to sort by score"""
	def __init__(self):
		self.Teams = []
		self.OldTeams = []

	def FromString(self, String):
		self.OldTeams = self.Teams
		self.Teams = []
		for Team in String:
			NewTeam = CtfTeam(0, "dummy", "dummy", "dummy")
			NewTeam.FromString(Team)
			self.Teams.append(NewTeam)
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		self.ScoreDifferences()

	def ScoreDifferences(self):
		self.Differences(self.OldTeams, self.Teams)
				
	def Differences(self, Oldlist, Newlist):
		NewPos = 0
		for New in Newlist:
			OldPos = 0
			for Old in Oldlist:
				if New.get_id() == Old.get_id():
					New.set_positionchange(OldPos - NewPos)
				OldPos += 1
			NewPos += 1
		

	def AddTeam(self,TeamID,TeamName, VulnBox, Network):
		NewTeam = CtfTeam(TeamID, TeamName, VulnBox, Network)
		self.Teams.append(NewTeam)

	def NewScore(self,TeamID,Score):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_score(Score)

	def NewScoreHistory(self,TeamID,ScoreHistory):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_scorehistory(ScoreHistory)

	def Count(self):
		return len(self.Teams)

	def Normalize(self,Teams):
		max_offscore = 1
		max_defscore = 1
		max_extrascore = 1
		max_score = 1
		for Team in Teams:
			if Team.get_score().get_OffScore() >= max_offscore:
				max_offscore = Team.get_score().get_OffScore()
			if Team.get_score().get_DefScore() >= max_defscore:
				max_defscore = Team.get_score().get_DefScore()
			if Team.get_score().get_ExtraScore() >= max_extrascore:
				max_extrascore = Team.get_score().get_ExtraScore()
		for Team in Teams:
			Team.get_score().set_MaxScores(max_offscore, max_defscore, max_extrascore)
			if Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_ExtraNormScore() >= max_score:
				max_score = Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_ExtraNormScore()
		#print "MaxScores:",max_offscore,max_defscore,max_extrascore,max_score
		for Team in self.Teams:
			Team.get_score().set_MaxScore(max_score)
		

	def Sort(self,Teams):
		Teams.sort(key = lambda obj:obj.get_score().get_SortScore())


		#for Team in Teams:
		#	print "sorted: Team Name=",Team.get_id(),"Team Score:",Team.get_score().get_OffScore() ,Team.get_score().get_DefScore() ,Team.get_score().get_ExtraScore() ,Team.get_score().get_TotalScore()
		#	print "sorted: Team Name=",Team.get_id(),"Norm Score:",Team.get_score().get_OffNormScore() ,Team.get_score().get_DefNormScore() ,Team.get_score().get_ExtraNormScore() ,Team.get_score().get_TotalNormScore()
		#print "----"
		Teams.reverse()

	def Places(self):
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		placelist = []
		for Team in self.Teams:
			placelist.append(Team.get_id())
		return placelist

	def TeamIDs(self):
		TeamIDs = []
		for Team in self.Teams:
			TeamIDs.append(Team.get_id())
		return TeamIDs

	def Team(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team

	def TeamName(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_name()

	def TeamScore(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_score()

	def TeamNetwork(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_network()

	def TeamVuln(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_vulnbox()

	def TeamScoreHistory(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_scorehistory()

	def String(self):
		string = []
		for Team in self.Teams:
			string.append(Team.get_string())
		return string


MixInCopyClasses(ScoreState)
pb.setUnjellyableForClass(ScoreState, ScoreState)











class PluginDisplayEtherapeCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(NewSniffSummaryAnnouncementRequest)
		self.serverToDisplayEvents.append(NewSniffSummaryAnnouncementEvent)
		self.serverToMasterEvents.append(NewSniffSummaryAnnouncementEvent)
		self.clientToServerEvents.append(ShowEtherapeRequest)
		self.serverToDisplayEvents.append(ShowEtherapeEvent)

	def PluginCreateServer(self):
		return PluginDisplayEtherape("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayEtherape("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayEtherape("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayEtherape("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayEtherape(object):
	implements(IPlugin, ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None
	SniffSummaryEvent = None

	# local constants
	pluginname = "DisplayEtherape"
	pluginshortname = "Etherape"

	# local variables
	SpriteGroup = pygame.sprite.Group()

	ScoreState = ScoreState()
	Wait = 2


	def __init__(self,component):
		self.EndTime = datetime.datetime.now()

		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowEtherapeRequest)
		elif self.component == "server":
			return isinstance(event, ShowEtherapeRequest) or isinstance(event, NewSniffSummaryAnnouncementRequest) or isinstance(event, ClientConnectEvent)
		elif self.component == "display":
			return isinstance(event, ShowEtherapeEvent) or isinstance(event, NewSniffSummaryAnnouncementEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowEtherapeEvent) or isinstance(event, NewSniffSummaryAnnouncementEvent)

	def GuiFireButton(self):
		self.ButtonEtherape = gtk.Button(self.pluginshortname)
		self.ButtonEtherape.connect("clicked", self.GuiFireCallback, "score")
		self.ButtonEtherape.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonEtherape.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonEtherape.show()
		return self.ButtonEtherape

	def GuiFireCallback(self, widget, data):
		ev = ShowEtherapeRequest(self.Wait)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Options = str(self.Wait)
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		try:
			Wait = int(OptionString)
		except:
			Wait = 2
		return ShowEtherapeRequest(Wait)

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowEtherapeRequest):
			return (self.OptionStringStart(), str(event.Wait))
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonEtherapeDialog = gtk.Button("Rotating")
		self.ButtonEtherapeDialog.connect("clicked", self.GuiConfigureCallback, "etherape")
		self.ButtonEtherapeDialog.show()
		return self.ButtonEtherapeDialog

	def GuiConfigureCallback(self, widget, data):
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.set_title("Rotating Etherape")

		self.dialog.connect("delete_event", self.delete_RotatingEtherape)

		self.dialog.set_border_width(10)

		vbox = gtk.VBox()
		self.dialog.add(vbox)

		MainTable = gtk.Table(2,1)
		vbox.pack_start(MainTable, False, True, 0)
		
		AdjWait = gtk.Adjustment(self.Wait, 0.0, 21.0, 1.0, 1.0, 1.0)
		self.WaitSpin = gtk.SpinButton(AdjWait, 0, 0)
		MainTable.attach(self.WaitSpin, 0, 1, 0, 1, xpadding = 5, ypadding = 5)
		self.WaitSpin.show()

		MainTable.show()

		separator = gtk.VSeparator()
		vbox.pack_start(separator, False, True, 0)
		separator.show()

		bbox = gtk.HButtonBox()
		bbox.set_border_width(5)
		bbox.set_layout(gtk.BUTTONBOX_SPREAD)
		bbox.set_spacing(20)

		button = gtk.Button(stock=gtk.STOCK_OK)
		button.connect("clicked", self.delete_RotatingEtherape,"ok")
		bbox.add(button)
		button.show()

		button = gtk.Button(stock=gtk.STOCK_CANCEL)
		button.connect("clicked", self.delete_RotatingEtherape,"cancel")
		bbox.add(button)
		button.show()

		vbox.pack_end(bbox, False, True, 0)
		bbox.show()

		vbox.show()
		self.dialog.show()

	def delete_RotatingEtherape(self, widget, data):
		if data == "ok":
			self.Wait = self.WaitSpin.get_value_as_int()

		self.dialog.hide()

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				# reformat event to change direction
				if isinstance(event, ShowEtherapeRequest):
					self.CallbackNotify(event.Acknowledge())

			elif eventsource == "database":
				# cache sniffsummarys on server
				if isinstance(event, NewSniffSummaryAnnouncementRequest):
					self.SniffSummaryEvent = event
					self.CleanupSniffSummary()
					self.CallbackNotify(event.Acknowledge())

			else:
				# send cached scores to newly connected client
				if isinstance(event, ClientConnectEvent):
					if self.SniffSummaryEvent != None:
						self.CallbackNotify(self.SniffSummaryEvent)
			
		elif self.component == "display":
			if isinstance(event, NewSniffSummaryAnnouncementEvent):
				self.SniffSummaryEvent = event
				
			#	print  event.SniffSummaryString
			#	print " "
			#	print " "
			#	for SourceTeam in event.SniffSummaryString:
			#		A= "Src=`%s`  "%SourceTeam
			#		for TargetTeam in event.SniffSummaryString[SourceTeam]:
			#			A += "Tgt=`%s`,`%s` "%(TargetTeam,event.SniffSummaryString[SourceTeam][TargetTeam])
			#		print A
			#	print " "
			#	print " "

				return (False,False,self.SpriteGroup)
			elif isinstance(event, ShowEtherapeEvent):
				if self.SniffSummaryEvent != None:
					self.SpriteGroup = pygame.sprite.Group()

					EtherapeSprite = SpriteEtherape(40,self.Window.get_size(),self.SniffSummaryEvent.SniffSummaryString,event.Wait)
					self.SpriteGroup.add(EtherapeSprite)

					return (True,True,self.SpriteGroup)
				else:
					print "No Sniffed Data available"
					return (False,False,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, NewSniffSummaryAnnouncementEvent):
				self.SniffSummaryEvent = event
				return (False,"","")
			elif isinstance(event, ShowEtherapeEvent):
				htmltitle = 'CTF-Vis'
				htmlpage=''
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")

	def CleanupSniffSummary(self):
		pass


class SpriteEtherape(pygame.sprite.Sprite):
	TeamCount = 0
	SniffSummaryString = {}
	Teams = []
	SourceTeam = 0
	LastStep = 0
	WaitTime = 2
	def __init__(self,fontsize,rectsize,SniffSummaryString,WaitTime):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()
		self.rect.topleft = (0,0)
		self.fontsize = fontsize
		self.rectsize = rectsize
		self.WaitTime = WaitTime

		self.SniffSummaryString = SniffSummaryString
		self.TeamCount = len(self.SniffSummaryString)
		for Team in self.SniffSummaryString:
			self.Teams.append(Team)

	
		self.paint()
		self.LastStep = datetime.datetime.now()

	def clear(self):
		print "clear called"

	def update(self):
		if self.WaitTime != 0:
			Time = datetime.datetime.now()
			if (Time - self.LastStep) > datetime.timedelta(seconds=self.WaitTime):
				self.paint()
				self.LastStep = datetime.datetime.now()
				if self.SourceTeam + 1 < self.TeamCount:
					self.SourceTeam += 1
				else:
					self.SourceTeam = 0

	def paint(self):
		self.image.fill((0,0,0))

		TEAMDOTCOLOR = (0,0,255)
		TRAFFICCOLOR = (0,255,0)
		TEXTCOLOR = (255,0,0)

		if self.WaitTime == 0:
			for SourceTeam in range(0,self.TeamCount):
				SourceTeamID = self.Teams[SourceTeam]
				for TargetTeam in range(0,self.TeamCount):
					TargetTeamID = self.Teams[TargetTeam]
					value = self.SniffSummaryString[SourceTeamID][TargetTeamID] 
					value = value / 10
					pygame.draw.line(self.image, TRAFFICCOLOR, self.Position(SourceTeam,self.TeamCount), self.Position(TargetTeam,self.TeamCount), value)
		else:
			SourceTeamID = self.Teams[self.SourceTeam]
			pygame.draw.circle(self.image, TEXTCOLOR, self.Position(self.SourceTeam,self.TeamCount), 22)
			for TargetTeam in range(0,self.TeamCount):
				TargetTeamID = self.Teams[TargetTeam]
				value = self.SniffSummaryString[SourceTeamID][TargetTeamID]
				value = value / 10
				pygame.draw.line(self.image, TRAFFICCOLOR, self.Position(self.SourceTeam,self.TeamCount), self.Position(TargetTeam,self.TeamCount), value)

		font = pygame.font.Font(None, self.fontsize)
		for Team in range(0,self.TeamCount):
			textPlace = font.render("Team %d"%self.Teams[Team], 1, TEXTCOLOR)
			pos = self.PositionText(Team,self.TeamCount)
			self.image.blit(textPlace, (pos[0] - textPlace.get_width()/2, pos[1] - textPlace.get_height()/2))
			pygame.draw.circle(self.image, TEAMDOTCOLOR, self.Position(Team,self.TeamCount), 20)



	def Position(self,Number,Count):
		TeamAngle = 360.0/Count * Number
		Angle = TeamAngle * 2 * math.pi / 360.0
		
		x = self.rectsize[0]/2 + self.rectsize[0]/3 * math.cos(Angle) 
		y = self.rectsize[1]/2 + self.rectsize[1]/3 * math.sin(Angle)
		return (int(x),int(y))
		


	def PositionText(self,Number,Count):
		TeamAngle = 360.0/Count * Number
		Angle = TeamAngle * 2 * math.pi / 360.0
		
		x = self.rectsize[0]/2 + self.rectsize[0]/7*3 * math.cos(Angle) 
		y = self.rectsize[1]/2 + self.rectsize[1]/7*3 * math.sin(Angle)
		return (int(x),int(y))
		


plugin = PluginDisplayEtherapeCreator()
