
#*********************************************************************
# $Id: PluginDisplayImage.py 180 2008-12-24 01:15:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import shutil
	import tempfile

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PluginDisplayImageCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(ShowImageRequest)
		self.serverToDisplayEvents.append(ShowImageEvent)

	def PluginCreateServer(self):
		return PluginDisplayImage("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayImage("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayImage("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayImage("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayImage(object):
	implements(ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None

	# local constants
	pluginname = "DisplayImage"
	pluginshortname = "Image"

	# local variables
	ImageFile = ""
	SpriteGroup = pygame.sprite.Group()

	def __init__(self,component):
		self.ImageFile = os.path.join(os.getcwd(),"image.png")
		
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowImageRequest)
		elif self.component == "server":
			return isinstance(event, ShowImageRequest)
		elif self.component == "display":
			return isinstance(event, ShowImageEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowImageEvent)

	def GuiFireButton(self):
		self.ButtonImage = gtk.Button(self.pluginshortname)
		self.ButtonImage.connect("clicked", self.GuiFireCallback, "image")
		self.ButtonImage.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonImage.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonImage.show()
		return self.ButtonImage

	def GuiFireCallback(self, widget, data):
		ev = ShowImageRequest(self.ImageFile)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Options = self.ImageFile
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		return ShowImageRequest(OptionString)

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowImageRequest):
			return (self.OptionStringStart(),event.filename)
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonImageFile = gtk.Button("Image File")
		self.ButtonImageFile.connect("clicked", self.GuiConfigureCallback, "imagefile")
		self.ButtonImageFile.show()
		return self.ButtonImageFile


	def GuiConfigureCallback(self, widget, data):
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		chooser.set_current_folder(os.path.split(self.ImageFile)[0])
		chooser.set_filename(self.ImageFile)

		filter = gtk.FileFilter()
		filter.set_name("Images")
		filter.add_mime_type("image/png")
		filter.add_mime_type("image/jpeg")
		filter.add_mime_type("image/gif")
		filter.add_pattern("*.png")
		filter.add_pattern("*.jpg")
		filter.add_pattern("*.gif")
		filter.add_pattern("*.tif")
		chooser.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		chooser.add_filter(filter)


		response = chooser.run()
		if response == gtk.RESPONSE_OK:
			self.ImageFile = chooser.get_filename()
		chooser.destroy()

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				if isinstance(event, ShowImageRequest):
					#reformat event to change direction
					self.CallbackNotify(event.Acknowledge())
			
		elif self.component == "display":
			if isinstance(event, ShowImageEvent):
				self.SpriteGroup = pygame.sprite.Group()
				Sprite = SpriteImage(self.Window.get_size(),event.filename)
				self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, ShowImageEvent):
				
				TempImageFileName = tempfile.mkstemp(suffix=os.path.splitext(event.filename)[-1], prefix='image-', dir=self.Directory)[1]
				print "Copying %s to %s"%(event.filename, TempImageFileName)
				shutil.copyfile(event.filename, TempImageFileName)
				htmltitle = 'CTF-Vis'
				htmlpage = '<img width="%%100" src="%s" />'%TempImageFileName
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpriteImage(pygame.sprite.Sprite):
	def __init__(self,rectsize,filename):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()

		BACKGROUNDCOLOR = (0,0,0)

		self.image.fill(BACKGROUNDCOLOR)
		try:
			if os.path.exists(filename):
				ImgImg = pygame.image.load(filename)
				ImgImg.convert()
				ImgScaled = pygame.transform.scale(ImgImg, rectsize)
				self.image.blit(ImgScaled, (0,0))
			else:
				print "File `%s' does not exist"%filename
		except:
			print "Error opening `%s'"%filename



# Client requests Server to show Image on Display-Client
class ShowImageRequest(Event):
	def __init__(self,filename):
		self.name = "Show Image Request"
		self.filename = filename
	def Acknowledge(self):
		return ShowImageEvent(self.filename)
# Server requests Display-Client to show Image
class ShowImageEvent(Event):
	def __init__(self,filename):
		self.name = "Show Image Event"
		self.filename = filename


# ShowImageRequest
# Direction: Client to Server only
MixInCopyClasses(ShowImageRequest)
pb.setUnjellyableForClass(ShowImageRequest, ShowImageRequest)

# ShowImageEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowImageEvent)
pb.setUnjellyableForClass(ShowImageEvent, ShowImageEvent)



plugin = PluginDisplayImageCreator()
