
#*********************************************************************
# $Id: PluginDisplayImage.py 167 2008-12-23 14:02:40Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2008 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import tempfile

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb
	from twisted.spread import banana # just to find out how large the content may get

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *

	from PIL import Image
	import base64

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PluginDisplayImageCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(ShowImageContentRequest)
		self.serverToDisplayEvents.append(ShowImageContentEvent)

	def PluginCreateServer(self):
		return PluginDisplayImage("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayImage("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayImage("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayImage("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayImage(object):
	implements(ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None

	# local constants
	pluginname = "DisplayImage"
	pluginshortname = "Image"

	# local variables
	ImageFileName = None
	ImageFileSize = (0,0)
	ImageFileString = ""
	SpriteGroup = pygame.sprite.Group()

	def __init__(self,component):
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

		self.ImageFileName = os.path.join(os.getcwd(),"image.png")

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowImageContentRequest)
		elif self.component == "server":
			return isinstance(event, ShowImageContentRequest)
		elif self.component == "display":
			return isinstance(event, ShowImageContentEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowImageContentEvent)

	def GuiFireButton(self):
		self.ButtonImage = gtk.Button(self.pluginshortname)
		self.ButtonImage.connect("clicked", self.GuiFireCallback, "image")
		self.ButtonImage.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonImage.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonImage.show()
		return self.ButtonImage

	def GuiFireCallback(self, widget, data):
		ev = ShowImageContentRequest(self.ImageFileString, self.ImageFileSize)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Sizeoption = self.DragAndDropFieldSubSubDivider.join([str(self.ImageFileSize[0]),str(self.ImageFileSize[1])])
		Options = self.DragAndDropFieldSubDivider.join([self.ImageFileString,Sizeoption])
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		imagefilecontent,imagefilesizestring = OptionString.split(self.DragAndDropFieldSubDivider)
		imagefilesize = imagefilesizestring.split(self.DragAndDropFieldSubSubDivider)
		imagefilesizetuple = (int(imagefilesize[0]), int(imagefilesize[1]))
		return ShowImageContentRequest(imagefilecontent,imagefilesizetuple)

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowImageContentRequest):
			Options = self.DragAndDropFieldSubDivider.join([event.imagecontent,event.imagesize])
			return (self.OptionStringStart(),Options)
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonImageFile = gtk.Button("Image File")
		self.ButtonImageFile.connect("clicked", self.GuiConfigureCallback, "imagefile")
		self.ButtonImageFile.show()
		return self.ButtonImageFile


	def GuiConfigureCallback(self, widget, data):
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		chooser.set_current_folder(os.path.split(self.ImageFileName)[0])
		chooser.set_filename(self.ImageFileName)

		filter = gtk.FileFilter()
		filter.set_name("Images")
		filter.add_mime_type("image/png")
		filter.add_mime_type("image/jpeg")
		filter.add_mime_type("image/gif")
		filter.add_pattern("*.png")
		filter.add_pattern("*.jpg")
		filter.add_pattern("*.gif")
		filter.add_pattern("*.tif")
		chooser.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		chooser.add_filter(filter)


		response = chooser.run()
		if response == gtk.RESPONSE_OK:
			ImageFileName = chooser.get_filename()
			if os.path.isfile(ImageFileName):
				try:
					ImageFile = pygame.image.load(ImageFileName)
					ImageFileString = pygame.image.tostring(ImageFile,"RGBA")
					if len(ImageFileString) < banana.SIZE_LIMIT - 1024:
						self.ImageFileString = base64.standard_b64encode(ImageFileString)
						self.ImageFileName = ImageFileName
						self.ImageFileSize = ImageFile.get_size()
					else:
						print "Image %s is to large: %s instead of %s"%(ImageFileName,len(ImageFileString),(banana.SIZE_LIMIT-1024))
						print "Change SIZE_LIMIT in twisted.spread.banana.py to something like 4 * 1024 * 1024"
				except:
					print "Could not load imagefile %s"%ImageFileName
		chooser.destroy()


	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				if isinstance(event, ShowImageContentRequest):
					#reformat event to change direction
					self.CallbackNotify(event.Acknowledge())
			
		elif self.component == "display":
			if isinstance(event, ShowImageContentEvent):
				self.SpriteGroup = pygame.sprite.Group()
				Sprite = SpriteImage(self.Window.get_size(),base64.standard_b64decode(event.imagecontent),event.imagesize)
				self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, ShowImageContentEvent):
				NewImagefile = Image.fromstring("RGBA",event.imagesize,base64.standard_b64decode(event.imagecontent))
				TempImageFileName = tempfile.mkstemp(suffix=".png", prefix='image-', dir=self.Directory)[1]
				NewImagefile.save(TempImageFileName)
				
				htmltitle = 'CTF-Vis'
				htmlpage = '<img width="%%100" src="%s" />'%TempImageFileName
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpriteImage(pygame.sprite.Sprite):
	def __init__(self,rectsize,imagecontent,imagesize):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()

		BACKGROUNDCOLOR = (0,0,0)

		self.image.fill(BACKGROUNDCOLOR)
		ImgImg = pygame.image.fromstring(imagecontent,imagesize,"RGBA")
		ImgImg.convert()
		ImgScaled = pygame.transform.scale(ImgImg, rectsize)
		self.image.blit(ImgScaled, (0,0))


# Client requests Server to show Image on Display-Client
class ShowImageContentRequest(Event):
	def __init__(self,imagecontent,imagesize):
		self.name = "Show Image Request"
		self.imagecontent = imagecontent
		self.imagesize = imagesize
	def Acknowledge(self):
		return ShowImageContentEvent(self.imagecontent,self.imagesize)
# Server requests Display-Client to show Image
class ShowImageContentEvent(Event):
	def __init__(self,imagecontent,imagesize):
		self.name = "Show Image Event"
		self.imagecontent = imagecontent
		self.imagesize = imagesize


# ShowImageContentRequest
# Direction: Client to Server only
MixInCopyClasses(ShowImageContentRequest)
pb.setUnjellyableForClass(ShowImageContentRequest, ShowImageContentRequest)

# ShowImageContentEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowImageContentEvent)
pb.setUnjellyableForClass(ShowImageContentEvent, ShowImageContentEvent)



plugin = PluginDisplayImageCreator()
