
#*********************************************************************
# $Id: PluginDisplayMessage.py 180 2008-12-24 01:15:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PluginDisplayMessageCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(ShowMessageRequest)
		self.serverToDisplayEvents.append(ShowMessageEvent)

	def PluginCreateServer(self):
		return PluginDisplayMessage("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayMessage("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayMessage("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayMessage("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayMessage(object):
	implements(IPlugin, ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None
	FontSizeSubjectLine = 30
	FontSizeTextLine = 20

	# local constants
	pluginname = "DisplayMessage"
	pluginshortname = "Message"

	# local variables
	Subject = ""
	MessageLines = [""]
	Logos = True
	SpriteGroup = pygame.sprite.Group()

	# configurable
	LEFTLOGO = ""
	RIGHTLOGO = "artwork/25c3.png"

	def __init__(self,component):
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window
		self.FontSizeSubjectLine = self.Window.get_width()/14
		self.FontSizeTextLine = self.Window.get_width()/20

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowMessageRequest)
		elif self.component == "server":
			return isinstance(event, ShowMessageRequest)
		elif self.component == "display":
			return isinstance(event, ShowMessageEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowMessageEvent)

	def GuiFireButton(self):
		self.ButtonMessage = gtk.Button(self.pluginshortname)
		self.ButtonMessage.connect("clicked", self.GuiFireCallback, "message")
		self.ButtonMessage.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonMessage.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonMessage.show()
		return self.ButtonMessage

	def GuiFireCallback(self, widget, data):
		ev = ShowMessageRequest(self.Subject,self.MessageLines,self.Logos)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		if self.Logos:
			Options = self.DragAndDropFieldSubDivider.join([self.Subject,self.DragAndDropFieldSubSubDivider.join(self.MessageLines),"1"])
		else:
			Options = self.DragAndDropFieldSubDivider.join([self.Subject,self.DragAndDropFieldSubSubDivider.join(self.MessageLines),"0"])
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		subject,text,logos = OptionString.split(self.DragAndDropFieldSubDivider)
		text = text.split(self.DragAndDropFieldSubSubDivider)
		# FIXME error handling

		return ShowMessageRequest(subject,text,logos)

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowMessageRequest):
			Options = self.DragAndDropFieldSubDivider.join([event.subject,self.DragAndDropFieldSubSubDivider.join(event.text),event.logos])
			return (self.OptionStringStart(),Options)
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonMessageDialog = gtk.Button("Content")
		self.ButtonMessageDialog.connect("clicked", self.GuiConfigureCallback, "messagecontent")
		self.ButtonMessageDialog.show()
		return self.ButtonMessageDialog

	def GuiConfigureCallback(self, widget, data):
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.set_title("Message")

		self.dialog.connect("delete_event", self.delete_Message)

		self.dialog.set_border_width(10)

		vbox = gtk.VBox()
		self.dialog.add(vbox)

		MainTable = gtk.Table(3,1)
		vbox.pack_start(MainTable, False, True, 0)
		self.entrysubject = gtk.Entry(max=0)
		self.entrysubject.set_text(self.Subject)
		MainTable.attach(self.entrysubject, 0, 1, 0, 1, xpadding = 5, ypadding = 5)
		self.entrysubject.show()

		textbuffer = gtk.TextBuffer(table=None)
		textbuffer.set_text("\n".join(self.MessageLines))
		self.textview = gtk.TextView(buffer=textbuffer)
		self.textview.set_justification(gtk.JUSTIFY_CENTER)
		MainTable.attach(self.textview, 0, 1, 1, 2, xpadding = 5, ypadding = 5)
		self.textview.show()
		
		self.checklogos = gtk.CheckButton("Logos")
		self.checklogos.set_active(self.Logos)
		MainTable.attach(self.checklogos, 0, 1, 2, 3, xpadding = 5, ypadding = 5)
		self.checklogos.show()

		MainTable.show()

		separator = gtk.VSeparator()
		vbox.pack_start(separator, False, True, 0)
		separator.show()

		bbox = gtk.HButtonBox()
		bbox.set_border_width(5)
		bbox.set_layout(gtk.BUTTONBOX_SPREAD)
		bbox.set_spacing(20)

		button = gtk.Button(stock=gtk.STOCK_OK)
		button.connect("clicked", self.delete_Message,"ok")
		bbox.add(button)
		button.show()

		button = gtk.Button(stock=gtk.STOCK_CANCEL)
		button.connect("clicked", self.delete_Message,"cancel")
		bbox.add(button)
		button.show()

		vbox.pack_end(bbox, False, True, 0)
		bbox.show()

		vbox.show()
		self.dialog.show()

	def delete_Message(self, widget, data):
		if data == "ok":
			self.Subject = self.entrysubject.get_text()
			textbuffer = self.textview.get_buffer()
			startiter = textbuffer.get_start_iter()
			enditer = textbuffer.get_end_iter()
			self.MessageLines = textbuffer.get_text(startiter,enditer).split("\n")
			self.Logos = self.checklogos.get_active()

		self.dialog.hide()

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				if isinstance(event, ShowMessageRequest):
					#reformat event to change direction
					self.CallbackNotify(event.Acknowledge())
			
		elif self.component == "display":
			if isinstance(event, ShowMessageEvent):
				self.SpriteGroup = pygame.sprite.Group()
				middle = (self.Window.get_width()/2,self.Window.get_height()*1/3)
				Sprite = SpriteMessage(self.Window.get_size(),middle,self.FontSizeSubjectLine,event.subject)
				self.SpriteGroup.add(Sprite)
				linenumber = 0
				for line in event.text:

					font = pygame.font.Font(None, self.FontSizeTextLine)
					middle = (self.Window.get_width()/2,self.Window.get_height()*1/2 + font.get_linesize() * linenumber)
					Sprite = SpriteMessage(self.Window.get_size(),middle,self.FontSizeTextLine,line)
					self.SpriteGroup.add(Sprite)
					linenumber += 1

				self.SpriteGroup.add(Sprite)
				if event.logos:
					if os.path.exists(self.LEFTLOGO):
						Sprite = SpriteLogo(self.Window.get_size(),self.LEFTLOGO,"TL")
						self.SpriteGroup.add(Sprite)
					if os.path.exists(self.RIGHTLOGO):
						Sprite = SpriteLogo(self.Window.get_size(),self.RIGHTLOGO,"TR")
						self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, ShowMessageEvent):
				htmltitle = 'CTF-Vis'
				htmlpage = '<h1>%s</h1>'%event.subject
				for line in event.text:
					htmlpage += "<h3>%s</h3>"%line
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpriteMessage(pygame.sprite.Sprite):
	def __init__(self,rectsize,middle,fontsize,text):
		pygame.sprite.Sprite.__init__(self)


		BACKGROUNDCOLOR = (0,0,0)
		TEXTCOLOR = (255,255,255)

		font = pygame.font.Font(None, fontsize)
		self.image = font.render(text, 1, TEXTCOLOR)
		
		self.rect = self.image.get_rect()
		self.rect.center = middle



class SpriteLogo(pygame.sprite.Sprite):
	def __init__(self,rectsize,filename,PosForCorner):
		pygame.sprite.Sprite.__init__(self)

		maxheight = rectsize[1]/7
		margin = (rectsize[1]/6 - rectsize[1]/7)/2
		try:
			if os.path.exists(filename):
				self.image = pygame.image.load(filename)
				self.image.convert()
				if self.image.get_height() > maxheight:
					self.image = pygame.transform.scale(self.image, (self.image.get_width() * maxheight / self.image.get_height(), maxheight))

				self.rect = self.image.get_rect()


				if PosForCorner == "TL":
					self.rect.topleft = (margin,margin)
				elif PosForCorner == "TR":
					self.rect.topright = (rectsize[0] - margin, margin)
				elif PosForCorner == "BL":
					self.rect.bottomleft = (margin, rectsize[1] - margin)
				elif PosForCorner == "BR":
					self.rect.bottomright = (rectsize[0] - margin, rectsize[1]- margin)
				else:
					print "Unknown position for Logo"
			else:
				print "File `%s' does not exist"%filename
		except:
			print "Error opening `%s'"%filename



# Client requests Server to show Text on Display-Client
class ShowMessageRequest(Event):
	def __init__(self,subject,text,logos):
		self.name = "Show Text Request"
		self.subject = subject
		self.text = text
		self.logos = logos
	def Acknowledge(self):
		return ShowMessageEvent(self.subject,self.text,self.logos)
# Server requests Display-Client to show Text
class ShowMessageEvent(Event):
	def __init__(self,subject,text,logos):
		self.name = "Show Text Event"
		self.subject = subject
		self.text = text
		self.logos = logos


# ShowMessageRequest
# Direction: Client to Server only
MixInCopyClasses(ShowMessageRequest)
pb.setUnjellyableForClass(ShowMessageRequest, ShowMessageRequest)

# ShowMessageEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowMessageEvent)
pb.setUnjellyableForClass(ShowMessageEvent, ShowMessageEvent)



plugin = PluginDisplayMessageCreator()
