
#*********************************************************************
# $Id: PluginDisplayPdf.py 180 2008-12-24 01:15:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import shutil
	import tempfile

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *

	from pdfreader import PdfReader
	from PIL import Image
	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PluginDisplayPdfCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(ShowPdfRequest)
		self.serverToDisplayEvents.append(ShowPdfEvent)
		self.clientToServerEvents.append(ShowPdfNumberedRequest)
		self.serverToDisplayEvents.append(ShowPdfNumberedEvent)
		self.clientToServerEvents.append(ShowPdfNextPageRequest)
		self.serverToDisplayEvents.append(ShowPdfNextPageEvent)
		self.clientToServerEvents.append(ShowPdfPreviousPageRequest)
		self.serverToDisplayEvents.append(ShowPdfPreviousPageEvent)

	def PluginCreateServer(self):
		return PluginDisplayPdf("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayPdf("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayPdf("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayPdf("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayPdf(object):
	implements(IPlugin, ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None

	# local constants
	pluginname = "DisplayPdf"
	pluginshortname = "Pdf"

	# local variables
	PdfFile = ""
	Page = 1
	SpriteGroup = pygame.sprite.Group()

	def __init__(self,component):
		self.PdfFile = os.path.join(os.getcwd(),"presentation.pdf")
		
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowPdfNumberedRequest)
		elif self.component == "server":
			return isinstance(event, ShowPdfNumberedRequest)
		elif self.component == "display":
			return isinstance(event, ShowPdfNumberedEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowPdfNumberedEvent)

	def GuiFireButton(self):
		self.ButtonPdf = gtk.Button(self.pluginshortname)
		self.ButtonPdf.connect("clicked", self.GuiFireCallback, "pdf")
		self.ButtonPdf.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonPdf.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonPdf.show()
		return self.ButtonPdf

	def GuiFireCallback(self, widget, data):
		ev = ShowPdfNumberedRequest(self.PdfFile,self.Page)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Options = self.DragAndDropFieldSubDivider.join([self.PdfFile,str(self.Page)])
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		filename,page = OptionString.split(self.DragAndDropFieldSubDivider)
		return ShowPdfNumberedRequest(filename,int(page))

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowPdfNumberedRequest):
			Options = self.DragAndDropFieldSubDivider.join([event.filename,str(event.page)])
			return (self.OptionStringStart(),Options)
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonPdfConfig = gtk.Button("Pdf Config")
		self.ButtonPdfConfig.connect("clicked", self.GuiConfigureCallback, "pdffile")
		self.ButtonPdfConfig.show()
		return self.ButtonPdfConfig



	def GuiConfigureCallback(self, widget, data):
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.set_title("Pdf")

		self.dialog.connect("delete_event", self.delete_PdfConfig)

		self.dialog.set_border_width(10)

		vbox = gtk.VBox()
		self.dialog.add(vbox)

		MainTable = gtk.Table(3,1)
		vbox.pack_start(MainTable, False, True, 0)

		ButtonPdfFile = gtk.Button("Pdf File")
		ButtonPdfFile.connect("clicked", self.Fileselector, "pdffile")
		MainTable.attach(ButtonPdfFile, 0, 1, 0, 1, xpadding = 5, ypadding = 5)
		ButtonPdfFile.show()

		AdjPdfPage = gtk.Adjustment(self.Page, 0.0, 301.0, 1.0, 10.0, 1.0)
		self.PdfPageSpin = gtk.SpinButton(AdjPdfPage, 0, 0)
		MainTable.attach(self.PdfPageSpin, 0, 1, 1, 2, xpadding = 5, ypadding = 5)
		self.PdfPageSpin.show()

		MainTable.show()


		separator = gtk.VSeparator()
		vbox.pack_start(separator, False, True, 0)
		separator.show()

		bbox = gtk.HButtonBox()
		bbox.set_border_width(5)
		bbox.set_layout(gtk.BUTTONBOX_SPREAD)
		bbox.set_spacing(20)

		button = gtk.Button(stock=gtk.STOCK_OK)
		button.connect("clicked", self.delete_PdfConfig,"ok")
		bbox.add(button)
		button.show()

		button = gtk.Button(stock=gtk.STOCK_CANCEL)
		button.connect("clicked", self.delete_PdfConfig,"cancel")
		bbox.add(button)
		button.show()

		vbox.pack_end(bbox, False, True, 0)
		bbox.show()

		vbox.show()
		self.dialog.show()


	def Fileselector(self, widget, data):
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		chooser.set_current_folder(os.path.split(self.PdfFile)[0])
		chooser.set_filename(self.PdfFile)

		filter = gtk.FileFilter()
		filter.set_name("Pdf-files")
		filter.add_mime_type("application/pdf")
		filter.add_pattern("*.pdf")
		chooser.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		chooser.add_filter(filter)


		response = chooser.run()
		if response == gtk.RESPONSE_OK:
			self.PdfFile = chooser.get_filename()
		chooser.destroy()


	def delete_PdfConfig(self, widget, data):
		if data == "ok":
			self.Page = self.PdfPageSpin.get_value_as_int()

		self.dialog.hide()
	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				if isinstance(event, ShowPdfNumberedRequest):
					#reformat event to change direction
					self.CallbackNotify(event.Acknowledge())
			
		elif self.component == "display":
			if isinstance(event, ShowPdfNumberedEvent):
				self.SpriteGroup = pygame.sprite.Group()
				Sprite = SpritePdf(self.Window.get_size(),event.filename,event.page)
				self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, ShowPdfNumberedEvent):
				self.PdfReader = PdfReader(event.filename,(1024,768))
				Imagefile = self.PdfReader.PageToImageFile(event.page)
				PdfImage = Image.open(Imagefile)
				TempFileName = tempfile.mkstemp(suffix=".png", prefix="pdfreader-",dir=self.Directory)[1]
				PdfImage.save(TempFileName, "PNG")
				os.remove(Imagefile)
				htmltitle = 'CTF-Vis'
				htmlpage = '<img width="%%100" src="%s" />'%TempFileName
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpritePdf(pygame.sprite.Sprite):
	def __init__(self,rectsize,filename,page):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()

		BACKGROUNDCOLOR = (0,0,0)

		self.image.fill(BACKGROUNDCOLOR)
		self.PdfReader = PdfReader(filename,rectsize)
		ImgImg = self.PdfReader.PageToImage(page)
		ImgImg.convert()
		self.image.blit(ImgImg, (0,0))



# Client requests Server to show PDF on Display-Client
class ShowPdfRequest(Event):
	def __init__(self,filename):
		self.name = "Show Pdf Request"
		self.filename = filename
	def Acknowledge(self):
		return ShowPdfEvent(self.filename)
# Server requests Display-Client to show PDF
class ShowPdfEvent(Event):
	def __init__(self,filename):
		self.name = "Show Pdf Event"
		self.filename = filename


# Client requests Server to show PDF on Display-Client
class ShowPdfNumberedRequest(Event):
	def __init__(self,filename,page):
		self.name = "Show Pdf Request"
		self.filename = filename
		self.page = page
	def Acknowledge(self):
		return ShowPdfNumberedEvent(self.filename,self.page)
# Server requests Display-Client to show PDF
class ShowPdfNumberedEvent(Event):
	def __init__(self,filename,page):
		self.name = "Show Pdf Event"
		self.filename = filename
		self.page = page


class ShowPdfNextPageRequest(Event):
	def __init__(self):
		self.name = "Show Pdf Next Page Request"
	def Acknowledge(self):
		return ShowPdfNextPageEvent()
class ShowPdfNextPageEvent(Event):
	def __init__(self):
		self.name = "Show Pdf Next Page Event"


class ShowPdfPreviousPageRequest(Event):
	def __init__(self):
		self.name = "Show Pdf Previous Page Request"
	def Acknowledge(self):
		return ShowPdfPreviousPageEvent()
class ShowPdfPreviousPageEvent(Event):
	def __init__(self):
		self.name = "Show Pdf Previous Page Event"


# ShowPdfRequest
# Direction: Client to Server only
MixInCopyClasses(ShowPdfRequest)
pb.setUnjellyableForClass(ShowPdfRequest, ShowPdfRequest)

# ShowPdfEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowPdfEvent)
pb.setUnjellyableForClass(ShowPdfEvent, ShowPdfEvent)



# ShowPdfNumberedRequest
# Direction: Client to Server only
MixInCopyClasses(ShowPdfNumberedRequest)
pb.setUnjellyableForClass(ShowPdfNumberedRequest, ShowPdfNumberedRequest)

# ShowPdfNumberedEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowPdfNumberedEvent)
pb.setUnjellyableForClass(ShowPdfNumberedEvent, ShowPdfNumberedEvent)



# ShowPdfNextPageRequest
# Direction: Client to Server only
MixInCopyClasses(ShowPdfNextPageRequest)
pb.setUnjellyableForClass(ShowPdfNextPageRequest, ShowPdfNextPageRequest)

# ShowPdfNextPageEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowPdfNextPageEvent)
pb.setUnjellyableForClass(ShowPdfNextPageEvent, ShowPdfNextPageEvent)



# ShowPdfPreviousPageRequest
# Direction: Client to Server only
MixInCopyClasses(ShowPdfPreviousPageRequest)
pb.setUnjellyableForClass(ShowPdfPreviousPageRequest, ShowPdfPreviousPageRequest)

# ShowPdfPreviousPageEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowPdfPreviousPageEvent)
pb.setUnjellyableForClass(ShowPdfPreviousPageEvent, ShowPdfPreviousPageEvent)



plugin = PluginDisplayPdfCreator()
