
#*********************************************************************
# $Id: PluginDisplayPdf.py 179 2008-12-24 00:44:07Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2008 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import tempfile

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb
	from twisted.spread import banana # just to find out how large the content may get

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *

	from pdfreader import PdfReader
	from PIL import Image
	import base64

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class PluginDisplayPdfCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(ShowPdfContentRequest)
		self.serverToDisplayEvents.append(ShowPdfContentEvent)

	def PluginCreateServer(self):
		return PluginDisplayPdf("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayPdf("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayPdf("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayPdf("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayPdf(object):
	implements(ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None

	# local constants
	pluginname = "DisplayPdf"
	pluginshortname = "Pdf"

	# local variables
	NewPdfFileName = None
	PdfFilePage = 1
	PdfFileName = None
	PdfFileSize = (0,0)
	PdfFileString = ""
	SpriteGroup = pygame.sprite.Group()

	def __init__(self,component):
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

		self.PdfFileName = os.path.join(os.getcwd(),"document.pdf")

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowPdfContentRequest)
		elif self.component == "server":
			return isinstance(event, ShowPdfContentRequest)
		elif self.component == "display":
			return isinstance(event, ShowPdfContentEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowPdfContentEvent)

	def GuiFireButton(self):
		self.ButtonPdf = gtk.Button(self.pluginshortname)
		self.ButtonPdf.connect("clicked", self.GuiFireCallback, "pdf")
		self.ButtonPdf.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonPdf.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonPdf.show()
		return self.ButtonPdf

	def GuiFireCallback(self, widget, data):
		ev = ShowPdfContentRequest(self.PdfFileString, self.PdfFileSize)
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Sizeoption = self.DragAndDropFieldSubSubDivider.join([str(self.PdfFileSize[0]),str(self.PdfFileSize[1])])
		Options = self.DragAndDropFieldSubDivider.join([self.PdfFileString,Sizeoption])
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		imagefilecontent,imagefilesizestring = OptionString.split(self.DragAndDropFieldSubDivider)
		imagefilesize = imagefilesizestring.split(self.DragAndDropFieldSubSubDivider)
		imagefilesizetuple = (int(imagefilesize[0]), int(imagefilesize[1]))
		return ShowPdfContentRequest(imagefilecontent,imagefilesizetuple)

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowPdfContentRequest):
			Options = self.DragAndDropFieldSubDivider.join([event.imagecontent,event.imagesize])
			return (self.OptionStringStart(),Options)
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		self.ButtonPdfConfig = gtk.Button("Pdf Config")
		self.ButtonPdfConfig.connect("clicked", self.GuiConfigureCallback, "pdffile")
		self.ButtonPdfConfig.show()
		return self.ButtonPdfConfig



	def GuiConfigureCallback(self, widget, data):
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.set_title("Pdf")

		self.dialog.connect("delete_event", self.delete_PdfConfig)

		self.dialog.set_border_width(10)

		vbox = gtk.VBox()
		self.dialog.add(vbox)

		MainTable = gtk.Table(3,1)
		vbox.pack_start(MainTable, False, True, 0)

		ButtonPdfFile = gtk.Button("Pdf File")
		ButtonPdfFile.connect("clicked", self.Fileselector, "pdffile")
		MainTable.attach(ButtonPdfFile, 0, 1, 0, 1, xpadding = 5, ypadding = 5)
		ButtonPdfFile.show()

		AdjPdfPage = gtk.Adjustment(self.PdfFilePage, 0.0, 301.0, 1.0, 10.0, 1.0)
		self.PdfPageSpin = gtk.SpinButton(AdjPdfPage, 0, 0)
		MainTable.attach(self.PdfPageSpin, 0, 1, 1, 2, xpadding = 5, ypadding = 5)
		self.PdfPageSpin.show()

		MainTable.show()


		separator = gtk.VSeparator()
		vbox.pack_start(separator, False, True, 0)
		separator.show()

		bbox = gtk.HButtonBox()
		bbox.set_border_width(5)
		bbox.set_layout(gtk.BUTTONBOX_SPREAD)
		bbox.set_spacing(20)

		button = gtk.Button(stock=gtk.STOCK_OK)
		button.connect("clicked", self.delete_PdfConfig,"ok")
		bbox.add(button)
		button.show()

		button = gtk.Button(stock=gtk.STOCK_CANCEL)
		button.connect("clicked", self.delete_PdfConfig,"cancel")
		bbox.add(button)
		button.show()

		vbox.pack_end(bbox, False, True, 0)
		bbox.show()

		vbox.show()
		self.dialog.show()


	def Fileselector(self, widget, data):
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		chooser.set_current_folder(os.path.split(self.PdfFileName)[0])
		chooser.set_filename(self.PdfFileName)

		filter = gtk.FileFilter()
		filter.set_name("Pdf-files")
		filter.add_mime_type("application/pdf")
		filter.add_pattern("*.pdf")
		chooser.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		chooser.add_filter(filter)


		response = chooser.run()
		if response == gtk.RESPONSE_OK:
			self.NewPdfFileName = chooser.get_filename()

		chooser.destroy()


	def delete_PdfConfig(self, widget, data):
		if data == "ok":
			self.PdfFilePage = self.PdfPageSpin.get_value_as_int()

			if os.path.isfile(self.NewPdfFileName):
				MyPdfReader = PdfReader(self.NewPdfFileName,(1024,768))
				ImageFile = MyPdfReader.PageToImage(self.PdfFilePage)

				if ImageFile != None:
					ImageFileString = pygame.image.tostring(ImageFile,"RGBA")
					if len(ImageFileString) < banana.SIZE_LIMIT - 1024:
						self.PdfFileString = base64.standard_b64encode(ImageFileString)
						self.PdfFileName = self.NewPdfFileName
						self.PdfFileSize = ImageFile.get_size()
					else:
						print "Image %s is to large: %s instead of %s"%(self.NewPdfFileName,len(ImageFileString),(banana.SIZE_LIMIT-1024))
						print "Change SIZE_LIMIT in twisted.spread.banana.py to something like 4 * 1024 * 1024"
				else:
					print "Pdfreader returned an empty image"

		self.dialog.hide()

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				if isinstance(event, ShowPdfContentRequest):
					#reformat event to change direction
					self.CallbackNotify(event.Acknowledge())
			
		elif self.component == "display":
			if isinstance(event, ShowPdfContentEvent):
				self.SpriteGroup = pygame.sprite.Group()
				Sprite = SpritePdf(self.Window.get_size(),base64.standard_b64decode(event.imagecontent),event.imagesize)
				self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, ShowPdfContentEvent):
				NewImagefile = Image.fromstring("RGBA",event.imagesize,base64.standard_b64decode(event.imagecontent))
				TempImageFileName = tempfile.mkstemp(suffix=".png", prefix='image-', dir=self.Directory)[1]
				NewImagefile.save(TempImageFileName)
				
				htmltitle = 'CTF-Vis'
				htmlpage = '<img width="%%100" src="%s" />'%TempImageFileName
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpritePdf(pygame.sprite.Sprite):
	def __init__(self,rectsize,imagecontent,imagesize):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()

		BACKGROUNDCOLOR = (0,0,0)

		self.image.fill(BACKGROUNDCOLOR)
		ImgImg = pygame.image.fromstring(imagecontent,imagesize,"RGBA")
		ImgImg.convert()
		ImgScaled = pygame.transform.scale(ImgImg, rectsize)
		self.image.blit(ImgScaled, (0,0))


# Client requests Server to show PDF on Display-Client
class ShowPdfContentRequest(Event):
	def __init__(self,imagecontent,imagesize):
		self.name = "Show Pdf Request"
		self.imagecontent = imagecontent
		self.imagesize = imagesize
	def Acknowledge(self):
		return ShowPdfContentEvent(self.imagecontent,self.imagesize)
# Server requests Display-Client to show PDF
class ShowPdfContentEvent(Event):
	def __init__(self,imagecontent,imagesize):
		self.name = "Show Pdf Event"
		self.imagecontent = imagecontent
		self.imagesize = imagesize


# ShowPdfContentRequest
# Direction: Client to Server only
MixInCopyClasses(ShowPdfContentRequest)
pb.setUnjellyableForClass(ShowPdfContentRequest, ShowPdfContentRequest)

# ShowPdfContentEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowPdfContentEvent)
pb.setUnjellyableForClass(ShowPdfContentEvent, ShowPdfContentEvent)



plugin = PluginDisplayPdfCreator()
