
#*********************************************************************
# $Id: PluginDisplayScoreHistory.py 180 2008-12-24 01:15:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os
	import shutil
	import tempfile

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *
	from Numeric import *
	import Gnuplot, Gnuplot.funcutils

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)










# SQL-Client requests the Server to Announce new Scorehistory
class NewScoreHistoryAnnouncementRequest(Event):
	def __init__(self,ScoreHistoryString):
		self.name = "New ScoreHistory Announcement Request"
		self.ScoreHistoryString = ScoreHistoryString
	def Acknowledge(self):
		return NewScoreHistoryAnnouncementEvent(self.ScoreHistoryString)
# Server announces new Scorehistory
class NewScoreHistoryAnnouncementEvent(Event):
	def __init__(self,ScoreHistoryString):
		self.name = "New ScoreHistory Announcement Event"
		self.ScoreHistoryString = ScoreHistoryString


# Client requests Server to show ScoreHistory on Display-Client
class ShowScoreHistoryRequest(Event):
	def __init__(self):
		self.name = "Show ScoreHistory Request"
	def Acknowledge(self):
		return ShowScoreHistoryEvent()
# Server requests Display-Client to show ScoreHistory
class ShowScoreHistoryEvent(Event):
	def __init__(self):
		self.name = "Show ScoreHistory Event"


# NewScoreHistoryAnnouncementRequest
# Direction: Client to Server only
MixInCopyClasses(NewScoreHistoryAnnouncementRequest)
pb.setUnjellyableForClass(NewScoreHistoryAnnouncementRequest, NewScoreHistoryAnnouncementRequest)

# NewScoreHistoryAnnouncementEvent
# Direction: Server to Client only
MixInCopyClasses(NewScoreHistoryAnnouncementEvent)
pb.setUnjellyableForClass(NewScoreHistoryAnnouncementEvent, NewScoreHistoryAnnouncementEvent)


# ShowScoreHistoryRequest
# Direction: Client to Server only
MixInCopyClasses(ShowScoreHistoryRequest)
pb.setUnjellyableForClass(ShowScoreHistoryRequest, ShowScoreHistoryRequest)

# ShowScoreHistoryEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowScoreHistoryEvent)
pb.setUnjellyableForClass(ShowScoreHistoryEvent, ShowScoreHistoryEvent)



class Score:
	"""The score of a CtfTeam"""
	def __init__(self,OffScore,DefScore,AdvScore,ExtraScore):
		self.OffScore = OffScore
		self.DefScore = DefScore
		self.AdvScore = AdvScore
		self.ExtraScore = ExtraScore
		self.MaxOffScore = 1
		self.MaxDefScore = 1
		self.MaxAdvScore = 1
		self.MaxExtraScore = 1
		self.MaxTotalScore = 1

	def FromString(self, String):
		self.OffScore = String[0]
		self.DefScore = String[1]
		self.AdvScore = String[2]
		self.ExtraScore = String[3]
		self.MaxOffScore = 1
		self.MaxDefScore = 1
		self.MaxAdvScore = 1
		self.MaxExtraScore = 1
		self.MaxTotalScore = 1
		
	def get_OffScore(self):
		return self.OffScore

	def get_DefScore(self):
		return self.DefScore

	def get_AdvScore(self):
		return self.AdvScore

	def get_ExtraScore(self):
		return self.ExtraScore

	def get_TotalScore(self):
		return self.OffScore + self.DefScore + self.AdvScore + self.ExtraScore

	def set_MaxScores(self, maxOffScore, maxDefScore, maxAdvScore, maxExtraScore):
		self.MaxOffScore = maxOffScore
		self.MaxDefScore = maxDefScore
		self.MaxAdvScore = maxAdvScore
		self.MaxExtraScore = maxExtraScore

	def set_MaxScore(self, maxScore):
		self.MaxTotalScore = maxScore

	def get_OffNormScore(self):
		return self.OffScore * 100 / self.MaxOffScore

	def get_DefNormScore(self):
		return self.DefScore * 100 / self.MaxDefScore

	def get_AdvNormScore(self):
		return self.AdvScore * 100 / self.MaxAdvScore

	def get_ExtraNormScore(self):
		return self.ExtraScore * 100 / self.MaxExtraScore

	def get_TotalNormScore(self):
		return ((self.OffScore * 100 / self.MaxOffScore) + (self.DefScore * 100 / self.MaxDefScore) + (self.ExtraScore * 100 / self.MaxExtraScore)) * 100 / self.MaxTotalScore

	def get_SortScore(self):
		return (self.OffScore * 100000 / self.MaxOffScore) + (self.DefScore * 100000 / self.MaxDefScore) + (self.ExtraScore * 100000 / self.MaxExtraScore)

	def get_string(self):
		return (self.OffScore, self.DefScore, self.AdvScore, self.ExtraScore)




class CtfTeam:
	"""Information about a playing Team including name and score"""
	def __init__(self,id,name,vulnbox,network):
		self.id = id
		self.name = name
		self.ipnetwork = network
		self.ipvulnbox = vulnbox
		self.Score = Score(0,0,0,0)
		self.Scorehistory = []
		self.PositionChange = 0

	def FromString(self, String):
		self.id = String[0]
		self.name = String[1]
		self.Score = Score(0,0,0,0)
		self.Score.FromString(String[2])
		self.Scorehistory = String[3]
		self.ipnetwork = String[4]
		self.ipvulnbox = String[5]

	def set_score(self,Score):
		self.Score = Score

	def set_scorehistory(self,Scorehistory):
		self.Scorehistory = Scorehistory

	def set_positionchange(self,positionchange):
		self.PositionChange = positionchange

	def get_positionchange(self):
		return self.PositionChange

	def get_id(self):
		return self.id

	def get_name(self):
		return self.name

	def get_score(self):
		return self.Score

	def get_scorehistory(self):
		return self.Scorehistory

	def get_string(self):
		return (self.id, self.name, self.Score.get_string(), self.Scorehistory, self.ipnetwork, self.ipvulnbox)

	def get_network(self):
		return self.ipnetwork

	def get_vulnbox(self):
		return self.ipvulnbox


class ScoreState:
	"""Collection of all CtfTeams, with functions to sort by score"""
	def __init__(self):
		self.Teams = []
		self.OldTeams = []

	def FromString(self, String):
		self.OldTeams = self.Teams
		self.Teams = []
		for Team in String:
			NewTeam = CtfTeam(0, "dummy", "dummy", "dummy")
			NewTeam.FromString(Team)
			self.Teams.append(NewTeam)
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		self.ScoreDifferences()

	def ScoreDifferences(self):
		self.Differences(self.OldTeams, self.Teams)
				
	def Differences(self, Oldlist, Newlist):
		NewPos = 0
		for New in Newlist:
			OldPos = 0
			for Old in Oldlist:
				if New.get_id() == Old.get_id():
					New.set_positionchange(OldPos - NewPos)
				OldPos += 1
			NewPos += 1
		

	def AddTeam(self,TeamID,TeamName, VulnBox, Network):
		NewTeam = CtfTeam(TeamID, TeamName, VulnBox, Network)
		self.Teams.append(NewTeam)

	def NewScore(self,TeamID,Score):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_score(Score)

	def NewScoreHistory(self,TeamID,ScoreHistory):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_scorehistory(ScoreHistory)

	def Count(self):
		return len(self.Teams)

	def Normalize(self,Teams):
		max_offscore = 1
		max_defscore = 1
		max_advscore = 1
		max_extrascore = 1
		max_score = 1
		for Team in Teams:
			if Team.get_score().get_OffScore() >= max_offscore:
				max_offscore = Team.get_score().get_OffScore()
			if Team.get_score().get_DefScore() >= max_defscore:
				max_defscore = Team.get_score().get_DefScore()
			if Team.get_score().get_AdvScore() >= max_advscore:
				max_advscore = Team.get_score().get_AdvScore()
			if Team.get_score().get_ExtraScore() >= max_extrascore:
				max_extrascore = Team.get_score().get_ExtraScore()
		for Team in Teams:
			Team.get_score().set_MaxScores(max_offscore, max_defscore, max_advscore, max_extrascore)
			if Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_AdvNormScore() + Team.get_score().get_ExtraNormScore() >= max_score:
				max_score = Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_AdvNormScore() + Team.get_score().get_ExtraNormScore()
		#print "MaxScores:",max_offscore,max_defscore,max_extrascore,max_score
		for Team in self.Teams:
			Team.get_score().set_MaxScore(max_score)
		

	def Sort(self,Teams):
		Teams.sort(key = lambda obj:obj.get_score().get_SortScore())


		#for Team in Teams:
		#	print "sorted: Team Name=",Team.get_id(),"Team Score:",Team.get_score().get_OffScore(),Team.get_score().get_DefScore(),Team.get_score().get_AdvScore(),Team.get_score().get_ExtraScore() ,Team.get_score().get_TotalScore()
		#	print "sorted: Team Name=",Team.get_id(),"Norm Score:",Team.get_score().get_OffNormScore(),Team.get_score().get_DefNormScore(),Team.get_score().get_AdvNormScore(),Team.get_score().get_ExtraNormScore() ,Team.get_score().get_TotalNormScore()
		#print "----"
		Teams.reverse()

	def Places(self):
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		placelist = []
		for Team in self.Teams:
			placelist.append(Team.get_id())
		return placelist

	def TeamIDs(self):
		TeamIDs = []
		for Team in self.Teams:
			TeamIDs.append(Team.get_id())
		return TeamIDs

	def TeamNames(self):
		TeamNames = []
		for Team in self.Teams:
			TeamNames.append(Team.get_name())
		return TeamNames

	def Team(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team

	def TeamName(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_name()

	def TeamScore(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_score()

	def TeamNetwork(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_network()

	def TeamVuln(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_vulnbox()

	def TeamScoreHistory(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_scorehistory()

	def String(self):
		string = []
		for Team in self.Teams:
			string.append(Team.get_string())
		return string


MixInCopyClasses(ScoreState)
pb.setUnjellyableForClass(ScoreState, ScoreState)

MixInCopyClasses(Score)
pb.setUnjellyableForClass(Score, Score)










class PluginDisplayScoreHistoryCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.serverToDisplayEvents.append(ShowScoreHistoryEvent)
		self.clientToServerEvents.append(NewScoreHistoryAnnouncementRequest)
		self.serverToDisplayEvents.append(NewScoreHistoryAnnouncementEvent)
		self.serverToMasterEvents.append(NewScoreHistoryAnnouncementEvent)
		self.clientToServerEvents.append(ShowScoreHistoryRequest)

	def PluginCreateServer(self):
		return PluginDisplayScoreHistory("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayScoreHistory("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayScoreHistory("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayScoreHistory("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayScoreHistory(object):
	implements(IPlugin, ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None

	# local constants
	pluginname = "DisplayScoreHistory"
	pluginshortname = "ScoreHistory"

	# local variables
	SpriteGroup = pygame.sprite.Group()

	TimeEvent = None
	ScoreHistoryEvent = None
	ScoreState = ScoreState()

	def __init__(self,component):
		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowScoreHistoryRequest)
		elif self.component == "server":
			return isinstance(event, ShowScoreHistoryRequest) or isinstance(event, NewScoreHistoryAnnouncementRequest) or isinstance(event, ClientConnectEvent)
		elif self.component == "display":
			return isinstance(event, ShowScoreHistoryEvent) or isinstance(event, NewScoreHistoryAnnouncementEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowScoreHistoryEvent) or isinstance(event, NewScoreHistoryAnnouncementEvent)

	def GuiFireButton(self):
		self.ButtonScoreHistory = gtk.Button(self.pluginshortname)
		self.ButtonScoreHistory.connect("clicked", self.GuiFireCallback, "score")
		self.ButtonScoreHistory.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonScoreHistory.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonScoreHistory.show()
		return self.ButtonScoreHistory

	def GuiFireCallback(self, widget, data):
		ev = ShowScoreHistoryRequest()
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Options = ""
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		return ShowScoreHistoryRequest()

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowScoreHistoryRequest):
			return (self.OptionStringStart(),"")
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		return None

	def GuiConfigureCallback(self, widget, data):
		return

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				# reformat event to change direction
				if isinstance(event, ShowScoreHistoryRequest):
					self.CallbackNotify(event.Acknowledge())

			elif eventsource == "database":
				# cache Scores on server
				if isinstance(event, NewScoreHistoryAnnouncementRequest):
					self.ScoreHistoryEvent = event.Acknowledge()
					self.CallbackNotify(event.Acknowledge())

			else:
				# send cached scores to newly connected client
				if isinstance(event, ClientConnectEvent):
					if self.TimeEvent != None:
						self.CallbackNotify(self.TimeEvent)
					if self.ScoreHistoryEvent != None:
						self.CallbackNotify(self.ScoreHistoryEvent)
			
		elif self.component == "display":
			self.SpriteGroup = pygame.sprite.Group()
			if isinstance(event, NewScoreHistoryAnnouncementEvent):
				self.ScoreState.FromString(event.ScoreHistoryString)
				return (False,False,self.SpriteGroup)
			elif isinstance(event, ShowScoreHistoryEvent):
				Sprite = SpriteScoreHistory(self.Window.get_size(), self.ScoreState)
				self.SpriteGroup.add(Sprite)
				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, NewScoreHistoryAnnouncementEvent):
				self.ScoreState.FromString(event.ScoreHistoryString)
				return (False,"","")
			elif isinstance(event, ShowScoreHistoryEvent):
				scorehistoryimage = ScoreHistoryImage(self.ScoreState)
				GraphFilename = scorehistoryimage.PaintGraph(self.Directory,(1024,768))
				htmltitle = 'CTF-Vis'
				htmlpage = '<img width="%%100" src="%s" />'%GraphFilename
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpriteScoreHistory(pygame.sprite.Sprite):
	def __init__(self,rectsize,ScoreHistory):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()
		self.rect.topleft = (0,0)


		scorehistoryimage = ScoreHistoryImage(ScoreHistory)
		GraphFilename = scorehistoryimage.PaintGraph("/tmp",rectsize)
		image = pygame.Surface(rectsize)
		try:
			if os.path.exists(GraphFilename):
				GraphImg = pygame.image.load(GraphFilename)
				GraphImg.convert()
				image.blit(GraphImg, (rectsize[0]/2 - GraphImg.get_width()/2, rectsize[1]/2 - GraphImg.get_height()/2))
			else:
				print "File `%s' does not exist"%filename
		except:
			print "Error opening `%s'"%filename
		self.image.blit(image, (0,0))


class ScoreHistoryImage():
	def __init__(self,scorehistory):
		self.scorehistory = scorehistory

	def PaintGraph(self,directory,rectsize):
		TempGraphFileName = tempfile.mkstemp(suffix='.png', prefix='graph-', dir=directory)[1]

		data = []
		for TeamID in self.scorehistory.TeamIDs():
			datasetvalues = []
			for datavalue in self.scorehistory.TeamScoreHistory(TeamID):
				#print "Value for Team %s, Teamid %s : '%s'"%(ScoreHistory.TeamName(TeamID),TeamID,datavalue)
				try:
					datasetvalues.append(float(datavalue))
				except:
					datasetvalues.append(float(0))
			data.append(Gnuplot.Data(datasetvalues,title=self.scorehistory.TeamName(TeamID)))

		GraphPlot = Gnuplot.Gnuplot()
		#GraphPlot.title("Title")
		GraphPlot.xlabel("Time")
		GraphPlot.ylabel("Score")
		GraphPlot("""set data style linespoints""")
		GraphPlot("""set terminal png medium size %d,%d"""%rectsize)
		GraphPlot("""set output "%s" """%TempGraphFileName)
		GraphPlot("""set key left top Left width 0""")
		GraphPlot("""set size 1,1""")

		GraphPlot.plot(*data)

		return TempGraphFileName


plugin = PluginDisplayScoreHistoryCreator()
