
#*********************************************************************
# $Id: PluginDisplayScoreKoth.py 180 2008-12-24 01:15:08Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import os

	from twisted.plugin import IPlugin
	from zope.interface import implements
	from plugininterface import ICtfVisPluginCreator,ICtfVisPlugin
	from twisted.spread import pb

	import pygtk
	pygtk.require('2.0')
	import gtk

	import pygame
	from pygame.locals import *
	import datetime

	from events import *
	from network import MixInCopyClasses

except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)










# SQL-Client requests the Server to Announce new Start/End Times
class NewTimeAnnouncementRequest(Event):
	def __init__(self,TimeStart,TimeStop):
		self.name = "New Time Announcement Request"
		self.TimeStart = TimeStart
		self.TimeStop = TimeStop
	def Acknowledge(self):
		return NewTimeAnnouncementEvent(self.TimeStart,self.TimeStop)
# Server announces new Start/End Times
class NewTimeAnnouncementEvent(Event):
	def __init__(self,TimeStart,TimeStop):
		self.name = "New Time Announcement Event"
		self.TimeStart = TimeStart
		self.TimeStop = TimeStop


# SQL-Client requests the Server to Announce new Scores
class NewScoreAnnouncementRequest(Event):
	def __init__(self,ScoreString):
		self.name = "New Score Announcement Request"
		self.ScoreString = ScoreString
	def Acknowledge(self):
		return NewScoreAnnouncementEvent(self.ScoreString)
# Server announces new Scores
class NewScoreAnnouncementEvent(Event):
	def __init__(self,ScoreString):
		self.name = "New Score Announcement Event"
		self.ScoreString = ScoreString


# SQL-Client requests the Server to Announce new Scores
class NewScoreKothAnnouncementRequest(Event):
	def __init__(self,TeamID):
		self.name = "New Score KotH Announcement Request"
		self.TeamID = TeamID
	def Acknowledge(self):
		return NewScoreKothAnnouncementEvent(self.TeamID)
# Server announces new Scores
class NewScoreKothAnnouncementEvent(Event):
	def __init__(self,TeamID):
		self.name = "New Score KotH Announcement Event"
		self.TeamID = TeamID


# Client requests Server to show Score on Display-Client
class ShowScoreRequest(Event):
	def __init__(self):
		self.name = "Show Score Request"
	def Acknowledge(self):
		return ShowScoreEvent()
# Server requests Display-Client to show Score
class ShowScoreEvent(Event):
	def __init__(self):
		self.name = "Show Score Event"


# NewTimeAnnouncementRequest
# Direction: Client to Server only
MixInCopyClasses(NewTimeAnnouncementRequest)
pb.setUnjellyableForClass(NewTimeAnnouncementRequest, NewTimeAnnouncementRequest)

# NewTimeAnnouncementEvent
# Direction: Server to Client only
MixInCopyClasses(NewTimeAnnouncementEvent)
pb.setUnjellyableForClass(NewTimeAnnouncementEvent, NewTimeAnnouncementEvent)



# NewScoreKothAnnouncementRequest
# Direction: Client to Server only
MixInCopyClasses(NewScoreKothAnnouncementRequest)
pb.setUnjellyableForClass(NewScoreKothAnnouncementRequest, NewScoreKothAnnouncementRequest)

# NewScoreKothAnnouncementEvent
# Direction: Server to Client only
MixInCopyClasses(NewScoreKothAnnouncementEvent)
pb.setUnjellyableForClass(NewScoreKothAnnouncementEvent, NewScoreKothAnnouncementEvent)



# NewScoreAnnouncementRequest
# Direction: Client to Server only
MixInCopyClasses(NewScoreAnnouncementRequest)
pb.setUnjellyableForClass(NewScoreAnnouncementRequest, NewScoreAnnouncementRequest)

# NewScoreAnnouncementEvent
# Direction: Server to Client only
MixInCopyClasses(NewScoreAnnouncementEvent)
pb.setUnjellyableForClass(NewScoreAnnouncementEvent, NewScoreAnnouncementEvent)



# ShowScoreRequest
# Direction: Client to Server only
MixInCopyClasses(ShowScoreRequest)
pb.setUnjellyableForClass(ShowScoreRequest, ShowScoreRequest)

# ShowScoreEvent
# Direction: Server to Showclient only
MixInCopyClasses(ShowScoreEvent)
pb.setUnjellyableForClass(ShowScoreEvent, ShowScoreEvent)



class Score:
	"""The score of a CtfTeam"""
	def __init__(self,OffScore,DefScore,ExtraScore):
		self.OffScore = OffScore
		self.DefScore = DefScore
		self.ExtraScore = ExtraScore
		self.MaxOffScore = 1
		self.MaxDefScore = 1
		self.MaxExtraScore = 1
		self.MaxTotalScore = 1

	def FromString(self, String):
		self.OffScore = String[0]
		self.DefScore = String[1]
		self.ExtraScore = String[2]
		self.MaxOffScore = 1
		self.MaxDefScore = 1
		self.MaxExtraScore = 1
		self.MaxTotalScore = 1
		
	def get_OffScore(self):
		return self.OffScore

	def get_DefScore(self):
		return self.DefScore

	def get_ExtraScore(self):
		return self.ExtraScore

	def get_TotalScore(self):
		return self.OffScore + self.DefScore + self.ExtraScore

	def set_MaxScores(self, maxOffScore, maxDefScore, maxExtraScore):
		self.MaxOffScore = maxOffScore
		self.MaxDefScore = maxDefScore
		self.MaxExtraScore = maxExtraScore

	def set_MaxScore(self, maxScore):
		self.MaxTotalScore = maxScore

	def get_OffNormScore(self):
		return self.OffScore * 100 / self.MaxOffScore

	def get_DefNormScore(self):
		return self.DefScore * 100 / self.MaxDefScore

	def get_ExtraNormScore(self):
		return self.ExtraScore * 100 / self.MaxExtraScore

	def get_TotalNormScore(self):
		return ((self.OffScore * 100 / self.MaxOffScore) + (self.DefScore * 100 / self.MaxDefScore) + (self.ExtraScore * 100 / self.MaxExtraScore)) * 100 / self.MaxTotalScore

	def get_SortScore(self):
		return (self.OffScore * 100000 / self.MaxOffScore) + (self.DefScore * 100000 / self.MaxDefScore) + (self.ExtraScore * 100000 / self.MaxExtraScore)

	def get_string(self):
		return (self.OffScore, self.DefScore, self.ExtraScore)




class CtfTeam:
	"""Information about a playing Team including name and score"""
	def __init__(self,id,name,vulnbox,network):
		self.id = id
		self.name = name
		self.ipnetwork = network
		self.ipvulnbox = vulnbox
		self.Score = Score(0,0,0)
		self.Scorehistory = []
		self.PositionChange = 0

	def FromString(self, String):
		self.id = String[0]
		self.name = String[1]
		self.Score = Score(0,0,0)
		self.Score.FromString(String[2])
		self.Scorehistory = String[3]
		self.ipnetwork = String[4]
		self.ipvulnbox = String[5]

	def set_score(self,Score):
		self.Score = Score

	def set_scorehistory(self,Scorehistory):
		self.Scorehistory = Scorehistory

	def set_positionchange(self,positionchange):
		self.PositionChange = positionchange

	def get_positionchange(self):
		return self.PositionChange

	def get_id(self):
		return self.id

	def get_name(self):
		return self.name

	def get_score(self):
		return self.Score

	def get_scorehistory(self):
		return self.Scorehistory

	def get_string(self):
		return (self.id, self.name, self.Score.get_string(), self.Scorehistory, self.ipnetwork, self.ipvulnbox)

	def get_network(self):
		return self.ipnetwork

	def get_vulnbox(self):
		return self.ipvulnbox


class ScoreState:
	"""Collection of all CtfTeams, with functions to sort by score"""
	def __init__(self):
		self.Teams = []
		self.OldTeams = []

	def FromString(self, String):
		self.OldTeams = self.Teams
		self.Teams = []
		for Team in String:
			NewTeam = CtfTeam(0, "dummy", "dummy", "dummy")
			NewTeam.FromString(Team)
			self.Teams.append(NewTeam)
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		self.ScoreDifferences()

	def ScoreDifferences(self):
		self.Differences(self.OldTeams, self.Teams)
				
	def Differences(self, Oldlist, Newlist):
		NewPos = 0
		for New in Newlist:
			OldPos = 0
			for Old in Oldlist:
				if New.get_id() == Old.get_id():
					New.set_positionchange(OldPos - NewPos)
				OldPos += 1
			NewPos += 1
		

	def AddTeam(self,TeamID,TeamName, VulnBox, Network):
		NewTeam = CtfTeam(TeamID, TeamName, VulnBox, Network)
		self.Teams.append(NewTeam)

	def NewScore(self,TeamID,Score):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_score(Score)

	def NewScoreHistory(self,TeamID,ScoreHistory):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				Team.set_scorehistory(ScoreHistory)

	def Count(self):
		return len(self.Teams)

	def Normalize(self,Teams):
		max_offscore = 1
		max_defscore = 1
		max_extrascore = 1
		max_score = 1
		for Team in Teams:
			if Team.get_score().get_OffScore() >= max_offscore:
				max_offscore = Team.get_score().get_OffScore()
			if Team.get_score().get_DefScore() >= max_defscore:
				max_defscore = Team.get_score().get_DefScore()
			if Team.get_score().get_ExtraScore() >= max_extrascore:
				max_extrascore = Team.get_score().get_ExtraScore()
		for Team in Teams:
			Team.get_score().set_MaxScores(max_offscore, max_defscore, max_extrascore)
			if Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_ExtraNormScore() >= max_score:
				max_score = Team.get_score().get_OffNormScore() + Team.get_score().get_DefNormScore() + Team.get_score().get_ExtraNormScore()
		#print "MaxScores:",max_offscore,max_defscore,max_extrascore,max_score
		for Team in self.Teams:
			Team.get_score().set_MaxScore(max_score)
		

	def Sort(self,Teams):
		Teams.sort(key = lambda obj:obj.get_score().get_SortScore())


		#for Team in Teams:
		#	print "sorted: Team Name=",Team.get_id(),"Team Score:",Team.get_score().get_OffScore() ,Team.get_score().get_DefScore() ,Team.get_score().get_ExtraScore() ,Team.get_score().get_TotalScore()
		#	print "sorted: Team Name=",Team.get_id(),"Norm Score:",Team.get_score().get_OffNormScore() ,Team.get_score().get_DefNormScore() ,Team.get_score().get_ExtraNormScore() ,Team.get_score().get_TotalNormScore()
		#print "----"
		Teams.reverse()

	def Places(self):
		self.Normalize(self.Teams)
		self.Sort(self.Teams)
		placelist = []
		for Team in self.Teams:
			placelist.append(Team.get_id())
		return placelist

	def TeamIDs(self):
		TeamIDs = []
		for Team in self.Teams:
			TeamIDs.append(Team.get_id())
		return TeamIDs

	def Team(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team

	def TeamName(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_name()

	def TeamScore(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_score()

	def TeamNetwork(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_network()

	def TeamVuln(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_vulnbox()

	def TeamScoreHistory(self,TeamID):
		for Team in self.Teams:
			if Team.get_id() == TeamID:
				return Team.get_scorehistory()

	def String(self):
		string = []
		for Team in self.Teams:
			string.append(Team.get_string())
		return string


MixInCopyClasses(ScoreState)
pb.setUnjellyableForClass(ScoreState, ScoreState)

MixInCopyClasses(Score)
pb.setUnjellyableForClass(Score, Score)










class PluginDisplayScoreCreator(object):
	implements(IPlugin, ICtfVisPluginCreator)

	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		self.clientToServerEvents.append(NewTimeAnnouncementRequest)
		self.serverToDisplayEvents.append(NewTimeAnnouncementEvent)
		self.serverToMasterEvents.append(NewTimeAnnouncementEvent)
		self.clientToServerEvents.append(NewScoreAnnouncementRequest)
		self.serverToMasterEvents.append(NewScoreAnnouncementEvent)
		self.serverToDisplayEvents.append(NewScoreAnnouncementEvent)
		self.serverToDatabaseEvents.append(NewScoreAnnouncementEvent)
		self.clientToServerEvents.append(NewScoreKothAnnouncementRequest)
		self.serverToMasterEvents.append(NewScoreKothAnnouncementEvent)
		self.serverToDisplayEvents.append(NewScoreKothAnnouncementEvent)
		self.serverToDatabaseEvents.append(NewScoreKothAnnouncementEvent)
		self.clientToServerEvents.append(ShowScoreRequest)
		self.serverToDisplayEvents.append(ShowScoreEvent)

	def PluginCreateServer(self):
		return PluginDisplayScore("server")

	def PluginCreateGamemasterGtk(self):
		return PluginDisplayScore("gamemastergtk")

	def PluginCreateDisplay(self):
		return PluginDisplayScore("display")

	def PluginCreateHtmlout(self):
		return PluginDisplayScore("htmlout")

	def EventDirections(self):
		return (self.serverToDisplayEvents, self.serverToDatabaseEvents, self.serverToMasterEvents, self.clientToServerEvents)


class PluginDisplayScore(object):
	implements(IPlugin, ICtfVisPlugin)

	# global constants
	COMPONENTS = ["display", "server", "gamemastergtk", "htmlout"]

	PlaylistDragAndDrop = [("PLAYLIST",gtk.TARGET_SAME_APP,0)]

	DragAndDropFieldDivider = ":::"
	DragAndDropFieldSubDivider = ";;;"
	DragAndDropFieldSubSubDivider = "|||"

	# global variables
	CallbackDuration = None
	CallbackDisplays = None
	CallbackNotify = None
	Window = None
	Directory = None
	FontSizeScoreLine = 30

	# local constants
	pluginname = "DisplayScore"
	pluginshortname = "Score"

	# local variables
	SpriteGroup = pygame.sprite.Group()

	TimeEvent = None
	ScoreEvent = None
	ScoreKothEvent = None
	EndTime = None
	ScoreState = ScoreState()
	KothTeam = 0

	# configurable
	LEFTLOGO = ""
	RIGHTLOGO = "artwork/25c3.png"

	def __init__(self,component):
		self.EndTime = datetime.datetime.now()

		# sanity check
		if component in self.COMPONENTS:
			self.component = component

	def setGamemasterCallbacks(self,CallbackDuration,CallbackDisplays,CallbackNotify):
		self.CallbackDuration = CallbackDuration
		self.CallbackDisplays = CallbackDisplays
		self.CallbackNotify = CallbackNotify

	def setServerCallbacks(self,CallbackNotify):
		self.CallbackNotify = CallbackNotify

	def setWindow(self,Window):
		self.Window = Window
		self.FontSizeScoreLine = self.Window.get_width()/17

	def setDirectory(self,Directory):
		self.Directory = Directory

	def HandledEvent(self,event):
		if self.component == "gamemastergtk":
			return isinstance(event, ShowScoreRequest)
		elif self.component == "server":
			return isinstance(event, ShowScoreRequest) or isinstance(event, NewTimeAnnouncementRequest) or isinstance(event, NewScoreAnnouncementRequest) or isinstance(event, NewScoreKothAnnouncementRequest) or isinstance(event, ClientConnectEvent)
		elif self.component == "display":
			return isinstance(event, ShowScoreEvent) or isinstance(event, NewTimeAnnouncementEvent) or isinstance(event, NewScoreAnnouncementEvent) or isinstance(event, NewScoreKothAnnouncementEvent)
		elif self.component == "htmlout":
			return isinstance(event, ShowScoreEvent) or isinstance(event, NewTimeAnnouncementEvent) or isinstance(event, NewScoreAnnouncementEvent) or isinstance(event, NewScoreKothAnnouncementEvent)

	def GuiFireButton(self):
		self.ButtonScore = gtk.Button(self.pluginshortname)
		self.ButtonScore.connect("clicked", self.GuiFireCallback, "score")
		self.ButtonScore.drag_source_set(gtk.gdk.BUTTON1_MASK, self.PlaylistDragAndDrop, gtk.gdk.ACTION_COPY)
		self.ButtonScore.connect("drag_data_get", self.GuiFireDragDropCallback)
		self.ButtonScore.show()
		return self.ButtonScore

	def GuiFireCallback(self, widget, data):
		ev = ShowScoreRequest()
		if self.CallbackDisplays:
			clientlist = self.CallbackDisplays()
			event = AdressedEvent(ev, clientlist)
			if self.CallbackNotify:
				self.CallbackNotify(event)
			else:
				print "Cannot find Notify"
		else:
			print "Cannot find Displays"

	def GuiFireDragDropCallback(self, widget, context, selection, targetType, eventTime):
		Options = ""
		# Options should never contain self.DragAndDropFieldDivider
	
		# should stay the same
		Duration = str(int(self.CallbackDuration()))
		data = self.DragAndDropFieldDivider.join([self.OptionStringStart(), Duration, Options])
		selection.set(selection.target, 8, data)

	def OptionStringStart(self):
		return self.pluginshortname

	def EventFromOptions(self,OptionString):
		return ShowScoreRequest()

	def OptionsFromEvent(self,event):
		if isinstance(event, ShowScoreRequest):
			return (self.OptionStringStart(),"")
		else:
			print "Unknown Event",str(event)
			return ("","")

	def GuiConfigureButton(self):
		return None

	def GuiConfigureCallback(self, widget, data):
		return

	def HandleEvent(self,event,eventsource):
		if self.component == "server":
			if eventsource == "gamemaster":
				# reformat event to change direction
				if isinstance(event, ShowScoreRequest):
					self.CallbackNotify(event.Acknowledge())

			elif eventsource == "database":
				# cache Scores on server
				if isinstance(event, NewTimeAnnouncementRequest):
					self.TimeEvent = event.Acknowledge()
					self.CallbackNotify(event.Acknowledge())
				elif isinstance(event, NewScoreAnnouncementRequest):
					self.ScoreEvent = event
					self.CallbackNotify(event.Acknowledge())
				elif isinstance(event, NewScoreKothAnnouncementRequest):
					self.ScoreKothEvent = event
					self.CallbackNotify(event.Acknowledge())

			else:
				# send cached scores to newly connected client
				if isinstance(event, ClientConnectEvent):
					if self.TimeEvent != None:
						self.CallbackNotify(self.TimeEvent)
					if self.ScoreEvent != None:
						self.CallbackNotify(self.ScoreEvent)
					if self.ScoreKothEvent != None:
						self.CallbackNotify(self.ScoreKothEvent)
			
		elif self.component == "display":
			if isinstance(event, NewTimeAnnouncementEvent):
				self.EndTime = datetime.datetime.fromtimestamp(event.TimeStop)
				return (False,False,self.SpriteGroup)
			elif isinstance(event, NewScoreKothAnnouncementEvent):
				self.KothTeam = event.TeamID
				return (False,False,self.SpriteGroup)
			elif isinstance(event, NewScoreAnnouncementEvent):
				self.ScoreState.FromString(event.ScoreString)
				return (False,False,self.SpriteGroup)
			elif isinstance(event, ShowScoreEvent):
				self.SpriteGroup = pygame.sprite.Group()
				MAXTEAMSONSCOREBOARD = 10

				# Paint Clock
				ClockWidth = self.Window.get_width()/2
				ClockHeight = self.FontSizeScoreLine
				ClockTop = self.Window.get_height() - ClockHeight -10
				ClockSprite = SpriteClock(self.EndTime,self.FontSizeScoreLine,(ClockWidth,ClockHeight),(self.Window.get_width()/2-ClockWidth/2,ClockTop))
				self.SpriteGroup.add(ClockSprite)
		
				# Paint Logos
				if os.path.exists(self.LEFTLOGO):
					Sprite = SpriteLogo(self.Window.get_size(),self.LEFTLOGO,"TL")
					self.SpriteGroup.add(Sprite)
				if os.path.exists(self.RIGHTLOGO):
					Sprite = SpriteLogo(self.Window.get_size(),self.RIGHTLOGO,"TR")
					self.SpriteGroup.add(Sprite)

				# Paint Scores
				if self.ScoreState.Count() > 0:
					ScoreTableTop = self.Window.get_height()/6
					ScoreTableHeight = self.Window.get_height()/3*2
					ScoreTableBottom = ScoreTableTop + ScoreTableHeight

					ScoreLineWidth = self.Window.get_width()
					if self.ScoreState.Count() <= MAXTEAMSONSCOREBOARD:
						ScoreLineHeight = ScoreTableHeight/self.ScoreState.Count()
					else:
						ScoreLineHeight = ScoreTableHeight/MAXTEAMSONSCOREBOARD
					ScoreLineSize = (ScoreLineWidth,ScoreLineHeight)
					ScoreTableLeft = 0
					ScoreTableRight = ScoreTableLeft + ScoreLineWidth
					
					ScoreLineSprite = SpriteScoreLineLegend(ScoreLineSize,(ScoreTableLeft, ScoreTableTop),self.FontSizeScoreLine) 
					self.SpriteGroup.add(ScoreLineSprite)
					# Paint Scorelines for all Teams
					ScoreLineNumber = 1
					places = self.ScoreState.Places()
					for PlaceID in places[0:MAXTEAMSONSCOREBOARD]:
						#print "Placed Team ",PlaceID, "Name",self.ScoreState.TeamName(PlaceID), " with score of " , self.ScoreState.TeamScore(PlaceID).get_TotalNormScore(), self.ScoreState.TeamScore(PlaceID).get_TotalScore()
						ScoreLinePos = (ScoreTableLeft, ScoreTableTop + ScoreLineNumber*ScoreLineHeight)
						IsKoth = (self.KothTeam == self.ScoreState.Team(PlaceID).get_id())
						ScoreLineSprite = SpriteScoreLine(ScoreLineSize, ScoreLinePos, self.FontSizeScoreLine, self.ScoreState.Team(PlaceID),ScoreLineNumber,IsKoth)
						self.SpriteGroup.add(ScoreLineSprite)
						ScoreLineNumber += 1

				return (True,True,self.SpriteGroup)
			else:
				return (False,False,self.SpriteGroup)

		elif self.component == "htmlout":
			if isinstance(event, NewTimeAnnouncementEvent):
				self.EndTime = datetime.datetime.fromtimestamp(event.TimeStop)
				return (False,"","")
			elif isinstance(event, NewScoreKothAnnouncementEvent):
				self.KothTeam = event.TeamID
				return (False,"","")
			elif isinstance(event, NewScoreAnnouncementEvent):
				self.ScoreState.FromString(event.ScoreString)
				return (False,"","")
			elif isinstance(event, ShowScoreEvent):
				TEAMLOGOPATH = "team-logos"
				ARTWORKPATH = "artwork"

				TOTALIMAGE = "sum_64.png"
				ATTACKIMAGE = "swords_64.png"
				DEFENDIMAGE = "shield_64.png"
				ADVISORYIMAGE = "paper_64.png"
				EXTRAIMAGE = "medal_64.png"

				htmltitle = 'CTF-Vis'
				htmlpage = '<h1>Ranking:</h1>\n'
				if self.ScoreState.Count() > 0:
					htmlpage += '<table width="100%">\n'
					htmlpage += '<tr><td class="rank">Rank</td><td class="teamlogo">Logo</td><td class="teamname">Teamname</td><td class="positionchange">+ / -</td><td  class="TotalScore"><img src="%s" alt="Total Score" /></td><td class="OffScore"><img src="%s" alt="Offensive Score" /></td><td  class="DefScore"><img src="%s" alt="Defensive Score" /></td><td class="AdvScore"><img src="%s" alt="Advisory Score" /></td><td class="ExtraScore"><img src="%s" alt="Rulecompliance Score" /></td></tr>\n'%(os.path.join(ARTWORKPATH, TOTALIMAGE),os.path.join(ARTWORKPATH, ATTACKIMAGE),os.path.join(ARTWORKPATH, DEFENDIMAGE),os.path.join(ARTWORKPATH, ADVISORYIMAGE),os.path.join(ARTWORKPATH, EXTRAIMAGE))

					for ScoreLineNumber,PlaceID in enumerate(places):
						teamscore = self.ScoreState.Team(PlaceID).get_score()
						TotalScore = teamscore.get_TotalNormScore()
						OffScore = teamscore.get_OffNormScore()
						DefScore = teamscore.get_DefNormScore()
						AdvScore = teamscore.get_AdvNormScore()
						ExtraScore = teamscore.get_ExtraNormScore()
						try:
							TotalScore = TotalScore.split(".")[0]
						except:
							pass
						try:
							OffScore = OffScore.split(".")[0]
						except:
							pass
						try:
							DefScore = DefScore.split(".")[0]
						except:
							pass
						try:
							AdvScore = DefScore.split(".")[0]
						except:
							pass
						try:
							ExtraScore = ExtraScore.split(".")[0]
						except:
							pass
						htmlpage += '<tr><td class="rank">%s</td><td class="teamlogo"><img src="%s" /></td><td class="teamname">%s</td><td class="positionchange">%s</td><td class="TotalScore">%s</td><td class="OffScore">%s</td><td class="DefScore">%s</td><td class="AdvScore">%s</td><td class="ExtraScore">%s</td></tr>\n'%(ScoreLineNumber,teamimage,team.get_name(),team.get_positionchange(),TotalScore,OffScore,DefScore,AdvScore,ExtraScore) 
					htmlpage +='</table>\n'
				return (True,htmltitle,htmlpage)
			else:
				return (False,"","")


class SpriteClock(pygame.sprite.Sprite):
	def __init__(self,time,fontsize,rectsize,pos):
		pygame.sprite.Sprite.__init__(self)

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()
		self.rect.topleft = pos
		self.time = time
		self.fontsize = fontsize

		self.WriteTime()

	def clear(self):
		print "clear called"

	def UpdateTime(self,time):
		self.time = time

	def update(self):
		self.WriteTime()

	def WriteTime(self):
		BACKGROUNDCOLOR = (0,0,0)
		TEXTCOLOR = (255,255,255)

		# split off mikroseconds. may be empty if no difference in microseconds (THIS DOES HAPPEN!!)
		timetoplay = self.time - datetime.datetime.now()
		if timetoplay >= datetime.timedelta(0,0,0):
			strtimetoplay = str(timetoplay)
			try:
				strtimetoplay = strtimetoplay.split(".")[0]
			except:
				pass
		else:
			strtimetoplay = "The Game has ended"

		self.image.fill(BACKGROUNDCOLOR)
		font = pygame.font.Font(None, self.fontsize)
		textImg = font.render(strtimetoplay, 1, TEXTCOLOR)
		self.image.blit(textImg, (self.rect.width/2 - textImg.get_width()/2, self.rect.height/2 - textImg.get_height()/2))


class SpriteLogo(pygame.sprite.Sprite):
	def __init__(self,rectsize,filename,PosForCorner):
		pygame.sprite.Sprite.__init__(self)

		maxheight = rectsize[1]/7
		margin = (rectsize[1]/6 - rectsize[1]/7)/2
		try:
			if os.path.exists(filename):
				self.image = pygame.image.load(filename)
				self.image.convert()
				if self.image.get_height() > maxheight:
					self.image = pygame.transform.scale(self.image, (self.image.get_width() * maxheight / self.image.get_height(), maxheight))

				self.rect = self.image.get_rect()


				if PosForCorner == "TL":
					self.rect.topleft = (margin,margin)
				elif PosForCorner == "TR":
					self.rect.topright = (rectsize[0] - margin, margin)
				elif PosForCorner == "BL":
					self.rect.bottomleft = (margin, rectsize[1] - margin)
				elif PosForCorner == "BR":
					self.rect.bottomright = (rectsize[0] - margin, rectsize[1] - margin)
				else:
					print "Unknown position for Logo"
			else:
				print "File `%s' does not exist"%filename
		except:
			print "Error opening `%s'"%filename



class SpriteScoreLineLegend(pygame.sprite.Sprite):
	def __init__(self,rectsize,pos,fontsize):
		pygame.sprite.Sprite.__init__(self)

		ARTWORKPATH = "artwork"

		TOTALIMAGE = "sum_64.png"
		ATTACKIMAGE = "swords_64.png"
		DEFENDIMAGE = "shield_64.png"
		EXTRAIMAGE = "medal_64.png"

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()
		self.rect.topleft = pos
		self.fontsize = fontsize

		scorewidth = rectsize[0]/20*7 #must be the same for ScoreLine
		scorestart = rectsize[0] - scorewidth

		iconsize = (scorewidth/4, rectsize[1])
		image = self.Image(iconsize,os.path.join(ARTWORKPATH, TOTALIMAGE))
		self.image.blit(image, (scorestart,0))
		image = self.Image(iconsize,os.path.join(ARTWORKPATH, ATTACKIMAGE))
		self.image.blit(image, (scorestart + scorewidth/4,0))
		image = self.Image(iconsize,os.path.join(ARTWORKPATH, DEFENDIMAGE))
		self.image.blit(image, (scorestart + scorewidth/2,0))
		image = self.Image(iconsize,os.path.join(ARTWORKPATH, EXTRAIMAGE))
		self.image.blit(image, (scorestart + scorewidth/4*3,0))

	def Image(self,rectsize,filename):

		Img = pygame.Surface(rectsize)
		try:
			if os.path.exists(filename):
				ImgImg = pygame.image.load(filename)
				ImgImg = pygame.transform.scale(ImgImg, (rectsize[1], rectsize[1])) #quadratic
				ImgImg.convert()
				Img.blit(ImgImg, (rectsize[0]/2 - ImgImg.get_width()/2, rectsize[1]/2 - ImgImg.get_height()/2))
			#else:
				#print "File `%s' does not exist"%filename
		except:
			print "Error opening `%s'"%filename
		return Img

class SpriteScoreLine(pygame.sprite.Sprite):
	def __init__(self,rectsize,pos,fontsize,Team,Place,KotH):

		self.rectsize = rectsize
		self.Team = Team
		self.fontsize = fontsize
		self.Place = Place
		self.KotH = KotH
		pygame.sprite.Sprite.__init__(self)

		TEAMLOGOPATH = "team-logos"
		KOTHLOGOPATH = "artwork"
		KOTHLOGO = "kothlogo.png"
		PAINTPOSITIONCHANGE = True

		self.image = pygame.Surface(rectsize)
		self.rect = self.image.get_rect()
		self.rect.topleft = pos
		height = self.rectsize[1]

		#Paint Teams Place
		placewidth = self.rectsize[1]/3*4
		placestart = 0
		image = self.TeamPlace((placewidth,height),self.Place)
		self.image.blit(image, (placestart,0))

		#Paint Teams Logo
		logowidth = self.rectsize[1] #quadratic
		logostart = placestart + placewidth
		if KotH:
			image = self.TeamImage((logowidth,height),os.path.join(KOTHLOGOPATH, KOTHLOGO))
		else:
			image = self.TeamImage((logowidth,height),os.path.join(TEAMLOGOPATH, str(self.Team.get_id())+".png"))
		self.image.blit(image, (logostart,0))

		#Paint Teams Name
		scorewidth = self.rectsize[0]/20*7 #must be the same for Legend
		nameandperhapspositionchangewidth = self.rectsize[0] - scorewidth - logowidth - placewidth
		positionchangewidth = self.rectsize[1] #quadratic
		namestart = logostart + logowidth
		if PAINTPOSITIONCHANGE:
			namewidth = nameandperhapspositionchangewidth - positionchangewidth
		else:
			namewidth = nameandperhapspositionchangewidth 
		image = self.TeamName((namewidth,height),self.Team.get_name())
		self.image.blit(image, (namestart,0))

		if PAINTPOSITIONCHANGE:
			#Paint Position Change
			positionchangestart = namestart + namewidth
			change = self.Team.get_positionchange()
			image = self.TeamPositionChange((positionchangewidth,height),change)
			self.image.blit(image, (positionchangestart,0))

		#Paint Teams Score
		# scorewidth defined above
		scorestart = namestart + nameandperhapspositionchangewidth
		score = self.Team.get_score()
		image = self.TeamScore((scorewidth,height),score)
		self.image.blit(image, (scorestart,0))


	def TeamPlace(self,rectsize,place):
		BORDERCOLOR = (255,255,255)
		TEXTCOLOR = (255,255,255)


		ScoreImg = pygame.Surface(rectsize)
		font = pygame.font.Font(None, self.fontsize)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		textPlace = font.render(str(place), 1, TEXTCOLOR)
		ScoreImg.blit(textPlace, (rectsize[0] / 8 * 7 - textPlace.get_width(), rectsize[1]/2 - textPlace.get_height()/2))
		return ScoreImg

	def TeamImage(self,rectsize,filename):
		BORDERCOLOR = (255,255,255)
		TEAMDOTCOLOR = (0,0,255)

		ScoreImg = pygame.Surface(rectsize)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		try:
			if os.path.exists(filename):
				ImgImg = pygame.image.load(filename)
				ImgImg = pygame.transform.scale(ImgImg, (rectsize[0]/3*2, rectsize[1]/3*2))
				ImgImg.convert()
				ScoreImg.blit(ImgImg, (rectsize[0]/2 - ImgImg.get_width()/2, rectsize[1]/2 - ImgImg.get_height()/2))
			else:
				#print "File `%s' does not exist"%filename
				pygame.draw.circle(ScoreImg, TEAMDOTCOLOR, (rectsize[0]/2, rectsize[1]/2), rectsize[1]/3)
		except:
			print "Error opening `%s'"%filename
			pygame.draw.circle(ScoreImg, TEAMDOTCOLOR, (rectsize[0]/2, rectsize[1]/2), rectsize[1]/3)
		return ScoreImg

	def TeamName(self,rectsize,teamname):
		TEXTCOLOR = (255,255,255)
		BORDERCOLOR = (255,255,255)

		ScoreImg = pygame.Surface(rectsize)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		font = pygame.font.Font(None, self.fontsize)
		textTeam = font.render(teamname, 1, TEXTCOLOR)
		ScoreImg.blit(textTeam, (rectsize[1]/6*1, rectsize[1]/2 - textTeam.get_height()/2))
		return ScoreImg

	def TeamPositionChange(self,rectsize,change):
		BORDERCOLOR = (255,255,255)
		TEXTCOLOR = (0,0,0)

		UPCOLOR = (0,255,0)
		DOWNCOLOR = (255,0,0)

		ScoreImg = pygame.Surface(rectsize)
		font = pygame.font.Font(None, self.fontsize/3*2)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		if change > 0:
			UpArrowPointlist = []
			UpArrowPointlist.append((rectsize[0]/10*5,rectsize[1]/10*1))
			UpArrowPointlist.append((rectsize[0]/10*9,rectsize[1]/10*4))
			UpArrowPointlist.append((rectsize[0]/10*7,rectsize[1]/10*4))
			UpArrowPointlist.append((rectsize[0]/10*7,rectsize[1]/10*9))
			UpArrowPointlist.append((rectsize[0]/10*3,rectsize[1]/10*9))
			UpArrowPointlist.append((rectsize[0]/10*3,rectsize[1]/10*4))
			UpArrowPointlist.append((rectsize[0]/10*1,rectsize[1]/10*4))

			pygame.draw.polygon(ScoreImg, UPCOLOR, UpArrowPointlist)
			textChange = font.render(str(change), 1, TEXTCOLOR)
			ScoreImg.blit(textChange, (rectsize[0]/2 - textChange.get_width()/2, rectsize[1]/2 - textChange.get_height()/2))
		elif change < 0:
			DownArrowPointlist = []
			DownArrowPointlist.append((rectsize[0]/10*5,rectsize[1]/10*9))
			DownArrowPointlist.append((rectsize[0]/10*9,rectsize[1]/10*6))
			DownArrowPointlist.append((rectsize[0]/10*7,rectsize[1]/10*6))
			DownArrowPointlist.append((rectsize[0]/10*7,rectsize[1]/10*1))
			DownArrowPointlist.append((rectsize[0]/10*3,rectsize[1]/10*1))
			DownArrowPointlist.append((rectsize[0]/10*3,rectsize[1]/10*6))
			DownArrowPointlist.append((rectsize[0]/10*1,rectsize[1]/10*6))

			pygame.draw.polygon(ScoreImg, DOWNCOLOR, DownArrowPointlist)
			textChange = font.render(str(- change), 1, TEXTCOLOR)
			ScoreImg.blit(textChange, (rectsize[0]/2 - textChange.get_width()/2, rectsize[1]/2 - textChange.get_height()/2))
		return ScoreImg

	def TeamScore(self,rectsize,teamscore):
		BORDERCOLOR = (255,255,255)

		TotalScore = teamscore.get_TotalNormScore()
		OffScore = teamscore.get_OffNormScore()
		DefScore = teamscore.get_DefNormScore()
		ExtraScore = teamscore.get_ExtraNormScore()
		try:
			TotalScore = TotalScore.split(".")[0]
		except:
			pass
		try:
			OffScore = OffScore.split(".")[0]
		except:
			pass
		try:
			DefScore = DefScore.split(".")[0]
		except:
			pass
		try:
			ExtraScore = ExtraScore.split(".")[0]
		except:
			pass
		ScoreImg = pygame.Surface(rectsize)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		TotalScoreImg = self.TeamScoreValue((rectsize[0]/4, rectsize[1]), TotalScore)
		OffScoreImg   = self.TeamScoreValue((rectsize[0]/4, rectsize[1]), OffScore)
		DefScoreImg   = self.TeamScoreValue((rectsize[0]/4, rectsize[1]), DefScore)
		ExtraScoreImg = self.TeamScoreValue((rectsize[0]/4, rectsize[1]), ExtraScore)
		ScoreImg.blit(TotalScoreImg, (0, 0))
		ScoreImg.blit(OffScoreImg,   (rectsize[0]/4, 0))
		ScoreImg.blit(DefScoreImg,   (rectsize[0]/2, 0))
		ScoreImg.blit(ExtraScoreImg, (rectsize[0]/4*3, 0))
		return ScoreImg


	def TeamScoreValue(self,rectsize,teamscore):
		TEXTCOLOR = (255,255,255)
		BORDERCOLOR = (255,255,255)

		ScoreImg = pygame.Surface(rectsize)
		pygame.draw.rect(ScoreImg, BORDERCOLOR, (1, 1, rectsize[0]-1, rectsize[1]-1), 2)
		font = pygame.font.Font(None, self.fontsize)
		textScore = font.render(str(teamscore), 1, TEXTCOLOR)
		ScoreImg.blit(textScore, (rectsize[0]/8*7 - textScore.get_width(), rectsize[1]/2 - textScore.get_height()/2))
		return ScoreImg
plugin = PluginDisplayScoreCreator()
