#!/usr/bin/env python
#*********************************************************************
# $Id: server.py 162 2008-12-16 12:48:49Z timo $
#
# CTF-VIS
# Visualization of Scoreboard and other Stuff for
# Capture the Flag - Games
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import getopt

	from twisted.spread import pb
	from twisted.internet import reactor,defer
	from twisted.cred import portal,checkers,credentials,error as credError
	from twisted.cred.credentials import IUsernameHashedPassword, IUsernamePassword
	from twisted.python import components
	from zope.interface import implements
	from twisted.internet.error import ConnectionLost
	from twisted.spread.pb import DeadReferenceError

	from manager import *
	from events import *
	import plugins
	from network import *
	import network
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class GameController:
	"""A controller that keeps the state of the Game"""
	def __init__(self, evManager, ClientList, pluginmgr):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.ClientList = ClientList

		self.pluginmgr = pluginmgr

		self.Playlists = []

 	def Notify(self, event):
		if isinstance(event, NewPlaylistRequest):
			#print "got new playlists"
			self.Playlists = event.playlists
			for Playlist in self.Playlists:
			#	print "============== New Playlist ================="
			#	Playlist.printqueue()
				
				Playlist.Install(self.evManager)
		elif isinstance(event, SecondEvent):
			QueueChanged = False
			for Playlist in self.Playlists:
			#	print "============== Ticking Playlist ================="
			#	Playlist.printqueue()

				if Playlist.Tick(self.evManager):
					QueueChanged = True

			if QueueChanged:
				event = NewPlaylistEvent(self.Playlists)
				self.evManager.Notify(event)

		for plugin in self.pluginmgr.List():
			if plugin.HandledEvent(event):
				plugin.HandleEvent(event,"")


class TimerController:
	"""A controller that sends of an event every second"""
	def __init__(self, evManager, reactor):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.reactor = reactor
		self.numClients = 0

	def NotifyApplicationStarted(self):
		self.reactor.callLater(1, self.Tick)

	def Tick(self):
		if self.numClients == 0:
			return

		ev = SecondEvent()
		self.evManager.Notify(ev)
		self.reactor.callLater(1, self.Tick)

	def Notify(self, event):
		if isinstance(event, ClientConnectEvent):
			self.numClients += 1
			if self.numClients == 1:
				self.Tick()
		if isinstance(event, ClientDisconnectEvent):
			self.numClients -= 1


class NetworkClientControllerDisplayPerspective(pb.Root,pb.Avatar):
	"""We RECEIVE events from the CLIENT through this object"""
	def __init__(self, evManager, pluginmgr):
		self.evManager = evManager
		self.evManager.RegisterListener(self)
		self.pluginmgr = pluginmgr

	def remote_ClientConnect(self, netClient):
		# needed if twisted >=2.4.0
		self.perspective_ClientConnect(netClient)
	def perspective_ClientConnect(self, netClient):
		print "CLIENT CONNECT"
		ev = ClientConnectEvent(netClient)
		self.evManager.Notify(ev)
		return 1

	def remote_ClientEvent(self, event):
		# needed if twisted >=2.4.0
		self.perspective_ClientEvent(event)
	def perspective_ClientEvent(self, event):
		for plugin in self.pluginmgr.List():
			if plugin.HandledEvent(event):
				plugin.HandleEvent(event,"display")

	def Notify(self, event):
		pass



class NetworkClientControllerDatabasePerspective(pb.Root,pb.Avatar):
	"""We RECEIVE events from the CLIENT through this object"""
	def __init__(self, evManager, pluginmgr):
		self.evManager = evManager
		self.evManager.RegisterListener(self)
		self.pluginmgr = pluginmgr

	def remote_ClientConnect(self, netClient):
		# needed if twisted >=2.4.0
		self.perspective_ClientConnect(netClient)
	def perspective_ClientConnect(self, netClient):
		print "CLIENT CONNECT"
		ev = ClientConnectEvent(netClient)
		self.evManager.Notify(ev)
		return 1

	def remote_ClientEvent(self, event):
		# needed if twisted >=2.4.0
		self.perspective_ClientEvent(event)
	def perspective_ClientEvent(self, event):
		for plugin in self.pluginmgr.List():
			if plugin.HandledEvent(event):
				plugin.HandleEvent(event,"database")

	def Notify(self, event):
		pass



class NetworkClientControllerGamemasterPerspective(pb.Root,pb.Avatar):
	"""We RECEIVE events from the CLIENT through this object"""
	def __init__(self, evManager, pluginmgr):
		self.evManager = evManager
		self.evManager.RegisterListener(self)
		self.pluginmgr = pluginmgr

	def remote_ClientConnect(self, netClient):
		# needed if twisted >=2.4.0
		self.perspective_ClientConnect(netClient)
	def perspective_ClientConnect(self, netClient):
		print "CLIENT CONNECT"
		ev = ClientConnectEvent(netClient)
		self.evManager.Notify(ev)
		return 1

	def remote_ClientEvent(self, event):
		# needed if twisted >=2.4.0
		self.perspective_ClientEvent(event)
	def perspective_ClientEvent(self, event):
		if isinstance(event, AdressedEvent):
			self.evManager.Notify(event)
		elif isinstance(event, NewPlaylistRequest):
			self.evManager.Notify(event)
		else:
			for plugin in self.pluginmgr.List():
				if plugin.HandledEvent(event):
					plugin.HandleEvent(event,"gamemaster")


	def Notify(self, event):
		pass




class NetworkClientView:
	"""We SEND events to the CLIENT through this object"""
	def __init__(self, evManager, clientlist, pluginmgr):
		self.evManager = evManager
		self.evManager.RegisterListener(self)

		self.pluginmgr = pluginmgr

		self.ClientList = clientlist
		self.pollSeconds = 0

 	def Pong(self):
		pass

 	def RemoteCallError(self, failure, client):
		#trap ensures that the rest will happen only if the failure was ConnectionLost
		failure.trap(ConnectionLost)
		self.DisconnectClient(client)
		return failure

	def DisconnectClient(self, client):
		print "Disconnecting Client", client
		self.ClientList.Remove(client)
		ev = ClientDisconnectEvent(client) #client id in here
		self.evManager.Notify(ev)

	def RemoteCall(self, client, fnName, *args):
		try:
			remoteCall = client.callRemote(fnName, *args)
			#remoteCall.addCallback(self.Pong)
			remoteCall.addErrback(self.RemoteCallError, client)
		except DeadReferenceError:
			self.DisconnectClient(client)


 	def Notify(self, event):
		clients = []
		if isinstance(event, SecondEvent):
			self.pollSeconds += 1
			if self.pollSeconds == 10:
				self.pollSeconds = 0
				for client in self.ClientList.GetAllRefs():
					self.RemoteCall(client, "Ping")

		# unpack adressed event
		if isinstance(event, AdressedEvent):
			print "got an `%s' that was adressed to `%s'"%(str(event.event),str(event.clientlist))
			for client in self.ClientList.GetClientsByName(event.clientlist):
				self.RemoteCall(client, "ServerEvent", event.event.Acknowledge())

		#don't send events that aren't Copyable
		elif not isinstance(event, pb.Copyable):
			eventName = event.__class__.__name__
			if not hasattr(network, "Copyable"+eventName):
				return
			copyableClass = getattr(network, "Copyable"+eventName)
			if (copyableClass not in self.pluginmgr.serverToDisplayEvents) and (copyableClass not in self.pluginmgr.serverToDatabaseEvents) and (copyableClass not in self.pluginmgr.serverToMasterEvents):
				return
			event = copyableClass(event)

			if copyableClass in self.pluginmgr.serverToDisplayEvents:
				clients += self.ClientList.GetDisplayRefs()
			if copyableClass in self.pluginmgr.serverToDatabaseEvents:
				clients += self.ClientList.GetDatabaseRefs()
			if copyableClass in self.pluginmgr.serverToMasterEvents:
				clients += self.ClientList.GetMasterRefs()

		elif ((event.__class__ not in self.pluginmgr.serverToDisplayEvents) and (event.__class__ not in self.pluginmgr.serverToDatabaseEvents) and (event.__class__ not in self.pluginmgr.serverToMasterEvents)) :
			#print "==============SERVER NOT SENDING: ", str(event)
			return
		else:
			if event.__class__ in self.pluginmgr.serverToDisplayEvents:
				clients += self.ClientList.GetDisplayRefs()
			if event.__class__ in self.pluginmgr.serverToDatabaseEvents:
				clients += self.ClientList.GetDatabaseRefs()
			if event.__class__ in self.pluginmgr.serverToMasterEvents:
				clients += self.ClientList.GetMasterRefs()

		#print "==============SERVER SENDING: ", str(event)
		for client in clients:
			#print "==============TO CLIENT: ", str(client)
			self.RemoteCall(client, "ServerEvent", event)



class ClientList:
	"""A List of all Connected clients divided by kind"""
	def __init__(self,evManager):
		self.evManager = evManager
		self.displays = []
		self.databases = []
		self.masters = []

	def AddDisplay(self,name,ref):
		self.displays.append((name,ref))
		print "========Got a Display: `%s'"%name
		ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
		self.evManager.Notify(ev)

	def AddDatabase(self,name,ref):
		self.databases.append((name,ref))
		print "========Got a Database: `%s'"%name
		ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
		self.evManager.Notify(ev)

	def AddMaster(self,name,ref):
		self.masters.append((name,ref))
		print "========Got a Master: `%s'"%name
		ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
		self.evManager.Notify(ev)

	def _GetName(self,object):
		return object[0]

	def _GetRef(self,object):
		return object[1]

	def GetDisplayRefs(self):
		return map(self._GetRef, self.displays)

	def GetDatabaseRefs(self):
		return map(self._GetRef, self.databases)

	def GetMasterRefs(self):
		return map(self._GetRef, self.masters)

	def GetAllRefs(self):
		return self.GetDisplayRefs() + self.GetDatabaseRefs() + self.GetMasterRefs()

	def GetDisplayNames(self):
		return map(self._GetName, self.displays)

	def GetDatabaseNames(self):
		return map(self._GetName, self.databases)

	def GetMasterNames(self):
		return map(self._GetName, self.masters)

	def GetClientsByName(self, names):
		refs = []
		for name in names:
			for display in self.displays:
				if name == self._GetName(display):
					refs.append(self._GetRef(display))
			for database in self.databases:
				if name == self._GetName(database):
					refs.append(self._GetRef(database))
			for master in self.masters:
				if name == self._GetName(master):
					refs.append(self._GetRef(master))
		return refs




	def Remove(self,ref):
		for display in self.displays[:]:
			if ref == self._GetRef(display):
				print "========Lost a Display: `%s'"%display[0]
				self.displays.remove(display)
				ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
				self.evManager.Notify(ev)
		for database in self.databases[:]:
			if ref == self._GetRef(database):
				print "========Lost a Database: `%s'"%database[0]
				self.databases.remove(database)
				ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
				self.evManager.Notify(ev)
		for master in self.masters[:]:
			if ref == self._GetRef(master):
				print "========Lost a Master: `%s'"%master[0]
				self.masters.remove(master)
				ev = ClientListEvent(self.GetDisplayNames(), self.GetDatabaseNames(), self.GetMasterNames())
				self.evManager.Notify(ev)





class CtfRealm(object):
	"""The Authetication Realm"""
	implements(portal.IRealm)

	def __init__(self,evManager,clientlist,pluginmgr,filename):
		self.evManager = evManager
		self.clientlist = clientlist
		self.pluginmgr = pluginmgr
		self.PasswordFile = PasswordFile(filename)

	def requestAvatar(self, avatarId, mind, *interfaces):
		if not pb.IPerspective in interfaces:
			raise NotImplementedError, "Not supported avatar interface"
		else:
			if self.PasswordFile.UserExists(avatarId):
				if self.PasswordFile.UserRole(avatarId, "Master"):
					avatar = NetworkClientControllerGamemasterPerspective(self.evManager,self.pluginmgr)
					self.clientlist.AddMaster(avatarId, mind)
				elif self.PasswordFile.UserRole(avatarId, "Database"):
					avatar = NetworkClientControllerDatabasePerspective(self.evManager,self.pluginmgr)
					self.clientlist.AddDatabase(avatarId, mind)
				elif self.PasswordFile.UserRole(avatarId, "Display"):
					avatar = NetworkClientControllerDisplayPerspective(self.evManager,self.pluginmgr)
					self.clientlist.AddDisplay(avatarId, mind)
				else:
					return defer.fail(credError.UnauthorizedLogin("Couldn't find role for User"))
			else:
				return defer.fail(credError.UnauthorizedLogin("User not found"))
		return pb.IPerspective, avatar, lambda a=self.clientlist: a.Remove(mind)





class PasswordFileChecker(object):
	"""Checking passwords against a file"""
	implements(checkers.ICredentialsChecker)
	credentialInterfaces = (credentials.IUsernamePassword, credentials.IUsernameHashedPassword)
	def __init__(self, filename):
		self.PasswordFile = PasswordFile(filename)

	def requestAvatarId(self, credentials):
		if not self.PasswordFile.UserExists(credentials.username):
			return defer.fail(credError.UnauthorizedLogin("User not found"))

		password = self.PasswordFile.password(credentials.username)
		if IUsernameHashedPassword.providedBy(credentials):
			# Let the hashed password checker do the checking
			if credentials.checkPassword(password):
				return defer.succeed(credentials.username)
			else:
				return defer.fail(credError.UnauthorizedLogin("Hashed Password didn't match"))
		elif IUsernamePassword.providedBy(credentials):
			# See if they match
			if credentials.password == password:
				return defer.succeed(credentials.username)
			else:
				return defer.fail(credError.UnauthorizedLogin("Password didn't match"))
		else:
			defer.fail(credError.UnhandledCredentials()) # OK, we don't know how to check this



class PasswordFile:
	"""Password file reader"""
	def __init__(self, filename):
		alloweduseraclparts = ["Display","Database","Master"]
		self.userdata = {}
		try:
			input = open(filename, 'r')
		except:
			print "Could not open Passwordfile"
		else:
			for line in input:
				try:
					linedata = line.rstrip()
					if linedata != "":
						if linedata[0] != ' ' and linedata[0] != '#':
							lineparts = linedata.split('\t')
							username = lineparts[0]
							userpassword = lineparts[1]
							useracl = lineparts[2]
							if useracl in alloweduseraclparts:
								self.userdata[username] = (userpassword,useracl)
							else:
								print "Parse Error in passwordfile (acl)"
							#print "got user `%s' with password `%s' and acls `%s'"%(username,userpassword,useracllist)
				except:
					print "Parse Error in passwordfile"

			input.close()


	def UserExists(self, username):
		return self.userdata.has_key(username)

	def password(self, username):
		if self.userdata.has_key(username):
			return self.userdata[username][0]
		else:
			#should never happen, if existence of user is checked
			return ""

	def UserRole(self, username, role):
		if role in self.userdata[username][1]:
			return True
		else:
			return False

class PluginManager:
	# A list of ALL possible events that a server can send to a client
	serverToDisplayEvents = []
	serverToDatabaseEvents = []
	serverToMasterEvents = []
	# A list of ALL possible events that a client can send to a server
	clientToServerEvents = []
	def __init__(self):
		print "Loading Plugins..."
		PluginsCreators = getPlugins(ICtfVisPluginCreator,plugins)
		self.list = []
		for PluginCreator in PluginsCreators:
			self.list.append(PluginCreator.PluginCreateServer())
			(serverToDisplayEvents, serverToDatabaseEvents, serverToMasterEvents, clientToServerEvents) = PluginCreator.EventDirections()
			self.serverToDisplayEvents.extend(serverToDisplayEvents)
			self.serverToDatabaseEvents.extend(serverToDatabaseEvents)
			self.serverToMasterEvents.extend(serverToMasterEvents)
			self.clientToServerEvents.extend(clientToServerEvents)

		self.list.sort()
		for Plugin in self.list:
			print "Got Plugin",Plugin.pluginname


	def List(self):
		return self.list

	def AddEventDirections(self,serverToDisplayEvents, serverToDatabaseEvents, serverToMasterEvents, clientToServerEvents):
		self.serverToDisplayEvents.extend(serverToDisplayEvents)
		self.serverToDatabaseEvents.extend(serverToDatabaseEvents)
		self.serverToMasterEvents.extend(serverToMasterEvents)
		self.clientToServerEvents.extend(clientToServerEvents)


def ParseOpt(argv):
	LISTENPORT=8000
	LISTENIP='127.0.0.1'
	PASSWORDFILE="password"
	try:
		opts,args=getopt.getopt(argv, "hi:p:P:", ["help","listen-ip=","listen-port=","passwordfile="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-i","--listen-ip"):
			LISTENIP=arg
		if opt in ("-p","--listen-port"):
			try:
				LISTENPORT=int(arg)
				assert LISTENPORT >= 0 and LISTENPORT <= 65535
			except:
				opterr("invalid parameter for --listen-port")
		if opt in ("-P","--passwordfile"):
			PASSWORDFILE=arg
	if len(args)>0:
		opterr("don't know what to do with additional arguments")

	return (LISTENIP,LISTENPORT,PASSWORDFILE)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -i, --listen-ip            ip/hostname to listen on"
	print "  -p, --listen-port          port to listen on"
	print "  -P, --passwordfile         location of the passwordfile"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(LISTENIP,LISTENPORT,PASSWORDFILE) = ParseOpt(argv)

	evManager = EventManager()
	clientlist = ClientList(evManager)

	PlugManager = PluginManager()
	for plugin in PlugManager.List():
		plugin.setServerCallbacks(evManager.Notify)
	PlugManager.AddEventDirections(serverToDisplayEvents, serverToDatabaseEvents, serverToMasterEvents, clientToServerEvents)


	log = TextLogView(evManager)
	timer = TimerController(evManager, reactor)
	gamecontr = GameController(evManager,clientlist,PlugManager)
	clientView = NetworkClientView(evManager,clientlist,PlugManager)


	ctfportal = portal.Portal(CtfRealm(evManager,clientlist,PlugManager,PASSWORDFILE))
	ctfportal.registerChecker(PasswordFileChecker(PASSWORDFILE))

	reactor.listenTCP(LISTENPORT, pb.PBServerFactory(ctfportal), interface=LISTENIP)
	print "CTF-VIS Server Listening on %s, Port %d"%(LISTENIP,LISTENPORT)

	reactor.run()

if __name__ == "__main__":
	main(sys.argv[1:])
