#!/usr/bin/env python
#*********************************************************************
# $Id:  $
#
# random sql database filler
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2008 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import getopt

	import time
	import psycopg2
	import random
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)

class GameDatabaseConnection:
	def __init__(self, PgsqlOptions, Frequency, TeamCount):
		(self.PGSQLHOST,self.PGSQLUSERNAME,self.PGSQLPASSWORD,self.PGSQLDATABASE) = PgsqlOptions
		self.Frequency = Frequency

		self.LastPoll = time.time()

		self.TimeStart = 0
		self.TimeEnd = 0

		self.LastPoll = time.time()

		self.Teams = []
		self.Services = []
		self.NewGame(TeamCount,5)

	def NewGame(self,TeamCount,ServiceCount):
		try:
			self.db = psycopg2.connect(host = self.PGSQLHOST, user = self.PGSQLUSERNAME, password = self.PGSQLPASSWORD, database = self.PGSQLDATABASE)
		except psycopg2.OperationalError,msg:
			print msg
			sys.exit(3)
			

		cursor = self.db.cursor()

		for TeamNumber in range(1,TeamCount+1):
			self.Teams.append({"Teamname":"Team%d"%TeamNumber,"uid":TeamNumber,"off":0,"def":0,"adv":0,"rule":0})

		for ServiceNumber in range(1,ServiceCount+1):
			self.Services.append((ServiceNumber,"Testservice %d"%ServiceNumber))

		for Team in self.Teams:
			cursor.execute("""INSERT INTO teams (uid, team_name, team_points_offensive, team_points_defensive, team_points_advisories, team_points_total, team_host, team_points_hacking) VALUES (%s, %s, 0, 0, 0, 0, "123.123.123.123", 0 ) """%(Team["uid"],Team["Teamname"]))
			print "Created Team %d"%Team["Teamname"]

		for Service in self.Services:
			cursor.execute("""INSERT INTO services (uid, service_name, service_script, service_script_type, service_check_interval, service_flags_per_check) VALUES (%s, Testservice, /bin/true, binary, 60, 1) """%Service)
			print "Created Service %d"%Service


	def InsertScoreLoop(self):
		while True:
			time.sleep(self.Frequency)
			self.InsertScore()

	def InsertScore(self):
		self.db = psycopg2PgSQLdb.connect(host = self.PGSQLHOST, user = self.PGSQLUSERNAME, passwd = self.PGSQLPASSWORD, db = self.PGSQLDATABASE)
		time = str(time.time()).split(".")[0]
		cursor = self.db.cursor()
		try:
			cursor.execute("""INSERT INTO `stats_times` (stats_time) VALUES ("%s") """%(time))
		except:
			print "Found a value that already exists"

		cursor.execute("""SELECT max(uid) FROM `stats_points` ORDER BY uid """)
		maxstatspointsid = cursor.fetchone()[0]
		for i,Team in enumerate(self.Teams):
			Team["off"] += random.randint(0, 5)
			Team["def"] += random.randint(0, 5)
			Team["adv"] += random.randint(0, 5)
			Team["rule"]+= random.randint(0, 5)
			try:
				cursor.execute("""INSERT INTO `stats_points` (uid, stats_time, stats_team, stats_points_offensive, stats_points_defensive, stats_points_advisory, stats_points_rulecompliance) VALUES ("%s", "%s", "%s", "%s", "%s", "%s", "%s", "%s") """%((maxstatspointsid[0]+i),time,Team["Teamname"],Team["off"],Team["def"],Team["adv"],Team["rule"]))

				cursor.execute("""UPDATE `teams` team_points_offensive, team_points_defensive, team_points_advisories, team_points_hacking WHERE uid="%s" ) """%(Team["off"],Team["def"],Team["adv"],Team["rule"],Team["Teamname"]))
			except:
				print "Found a value that already exists"

def ParseOpt(argv):
	PGSQLHOST = "127.0.0.1"
	PGSQLUSERNAME = "root"
	PGSQLPASSWORD = "ctf-test"
	PGSQLDATABASE = "ctf"
	FREQUENCY = 60
	TEAMCOUNT = 10
	try:
		opts,args=getopt.getopt(argv, "hm:u:w:d:f:t:", ["help","pgsql-host=","pgsql-username=","pgsql-password=","pgsql-database=","frequency=","teamcount="])
	except getopt.GetoptError,message:
		opterr(message)

	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-m","--pgsql-host"):
			PGSQLHOST = arg
		if opt in ("-u","--pgsql-username"):
			PGSQLUSERNAME = arg
		if opt in ("-w","--pgsql-password"):
			PGSQLPASSWORD = arg
		if opt in ("-d","--pgsql-database"):
			PGSQLDATABASE = arg
		if opt in ("-f","--frequency"):
			try:
				FREQUENCY = int(arg)
				assert FREQUENCY >= 0 and FREQUENCY <= 600
			except:
				opterr("invalid parameter for --frequency")
		if opt in ("-t","--teamcount"):
			try:
				TEAMCOUNT = int(arg)
				assert TEAMCOUNT >= 0 and TEAMCOUNT <= 50
			except:
				opterr("invalid parameter for --teamcount")
	if len(args) > 0:
		opterr("don't know what to do with additional arguments")

	PgsqlOptions = (PGSQLHOST,PGSQLUSERNAME,PGSQLPASSWORD,PGSQLDATABASE)
	return (PgsqlOptions,FREQUENCY,TEAMCOUNT)

def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)

def helpexit():
	print "Usage `%s [option]'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -m, --pgsql-host           pgsql servers hostname"
	print "  -u, --pgsql-username       username for the pgsql server"
	print "  -w, --pgsql-password       password for the pgsql server"
	print "  -d, --pgsql-database       database for the pgsql server"
	print "  -f, --frequency            pollfrequency"
	print "  -t, --teamcount            number of teams to create"
	print "  -h, --help                 shows this help and exits"
	sys.exit(0)

def main(argv):
	(PgsqlOptions,Frequency,TeamCount) = ParseOpt(argv)

	gameDatabaseConn = GameDatabaseConnection(PgsqlOptions, Frequency,TeamCount)

if __name__=="__main__":
	main(sys.argv[1:])
