#!/usr/bin/env python
#*********************************************************************
# $Id:  $
#
# videoviewer
#
# written by Timo Boettcher
# ctf-vis (near) spida (.) net
#
# (C) 2006 Timo Boettcher
# This is free software. You may redistribute copies of it under the
# terms of the GNU General Public License (Version 2)
# <http://www.gnu.org/licenses/gpl.html>.
# There is NO WARRANTY, to the extent permitted by law.
#
#*********************************************************************

try:
	import sys
	import getopt

	import pygame
	from pygame import movie
	from pygame.locals import *
except ImportError, err:
	print "couldn't load module. %s" % (err)
	sys.exit(2)


class VideoViewer:
	def __init__(self,filename,window,screensize=(0,0)):
		self.filename = filename
		self.window = window
		self.screensize = screensize

		pygame.mixer.quit()

		background = pygame.Surface(self.window.get_size())
		background.fill((0,0,0))
		self.window.blit(background, (0,0))
		pygame.display.flip()
		try:
			self.movie = pygame.movie.Movie(self.filename)
			self.movie.set_display(self.window,background.get_rect())
			self.movie.set_volume(1)
		except:
			print "Couldn't open `%s'"%self.filename

		

	def start(self):
		self.movie.play()

	def restart(self):
		self.movie.rewind()

	def pause(self):
		self.movie.pause()

	def forward(self,time):
		self.movie.skip(time)

	def halt(self):
		self.movie.stop()

	def running(self):
		return self.movie.get_busy()


#*********************************************************************
#
# FOR STANDALONE USE ONLY
#
#*********************************************************************

def main(argv):
	ArgPresentSize = False
	Fullscreen = False
	try:
		opts,args=getopt.getopt(argv, "hfs:", ["help","fullscreen","size="])
	except getopt.GetoptError,message:
		opterr(message)
		
	for opt,arg in opts:
		if opt in ("-h","--help"):
			helpexit()
		if opt in ("-f","--fullscreen"):
			Fullscreen = True
		if opt in ("-s","--size"):
			try:
				ArgPresentSize = True
				ScreenWidth,ScreenHeight = map(int,arg.split("x"))
				assert ScreenWidth >= 320 and ScreenWidth < 4096
				assert ScreenHeight >= 200 and ScreenHeight < 4096
			except:
				opterr("invalid parameter for --size")
	if len(args)<1:
		opterr("no file to play")
	elif len(args)>1:
		opterr("too much files to play")
	FileName=args[0]

	pygame.init()

	#find the maximum supported window resolution
	modes = pygame.display.list_modes()
	MaxModeX = 0
	MaxModeY = 0
	RequestedModeFound = False
	for mode in modes:
		if mode[0] > MaxModeX:
			MaxModeX = mode[0]
			MaxModeY = mode[1]
		if mode[0] == MaxModeX:
			if mode[1] > MaxModeY:
				MaxModeY = mode[1]

		if ArgPresentSize:
			if ScreenWidth == mode[0] and ScreenHeight == mode[1]:
				RequestedModeFound = 1
				
		
	if RequestedModeFound:
		window = pygame.display.set_mode((ScreenWidth,ScreenHeight))
	else:
		window = pygame.display.set_mode(modes[0])
	pygame.display.set_caption("VideoViewer %s"%FileName)
	background = pygame.Surface(window.get_size())
	background.fill((0,0,0))
	window.blit(background, (0,0))

	Viewer = VideoViewer(FileName,window,window.get_size())

	if Fullscreen:
		try:
			pygame.display.toggle_fullscreen()
		except:
			print "Couldn't activate fullscreen mode"

	pygame.display.flip()

	clock = pygame.time.Clock()
	while True:
		pygame.display.update()
		clock.tick(300)
		event = pygame.event.wait()
		if event.type is QUIT:
			sys.exit(0)
		elif event.type is KEYDOWN and event.key == K_q:
			sys.exit(0)
		elif event.type is KEYDOWN and event.key == K_f:
			try:
				pygame.display.toggle_fullscreen()
			except:
				print "Couldn't activate fullscreen mode"
		elif event.type is KEYDOWN and event.key == K_ESCAPE:
			sys.exit(0)


		elif event.type is KEYDOWN and event.key == K_SPACE:
			if Viewer.running():
				Viewer.pause()
			else:
				Viewer.start()
		elif event.type is KEYDOWN and event.key == K_r:
			Viewer.restart()
		elif event.type is KEYDOWN and event.key == K_LEFT:
			Viewer.restart()
		elif event.type is KEYDOWN and event.key == K_RIGHT:
			Viewer.forward(5)
		elif event.type is KEYDOWN and event.key == K_p:
			Viewer.pause()
		elif event.type is KEYDOWN and event.key == K_s:
			Viewer.start()
		elif event.type is KEYDOWN and event.key == K_h:
			Viewer.halt()
	

		pygame.display.flip()


def opterr(msg):
	print "command line parse error:",msg
	print "use `%s -h' to get help"%sys.argv[0]
	sys.exit(1)
	
def helpexit():
	print "Usage `%s [option] file.pdf'"%sys.argv[0]
	print ""
	print "Valid Options are:"
	print "  -f, --fullscreen           displays PDF in Fullscreen"
	print "  -s, --size WIDTHxHEIGHT    sets the viewable area"
	print "  -h, --help                 shows this help and exits "
	print ""
	print ""
	print "Keycodes when running:"
	print " q, [ESC]                    quit"
	print " f                           toggles fullscreen mode"
	print "[space], p                   toggles pause"
	print "s                            starts playback"
	print "h                            halts playback"
	print "[LEFT]                       seeks to beginning of the movie"
	print "[RIGHT]                      seeeks forward 5 seconds"
	print ""


	sys.exit(0)

if __name__=="__main__":
	main(sys.argv[1:])

	
